﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;

namespace ICallisto
{
    /// <summary>Primary plugin communication interface</summary>
    public interface IHost
    {
        /// <summary>Print a message to all users in all vrooms</summary>
        void Print(object text);
        /// <summary>Print a message to all users in all vrooms [or also to linked room]</summary>
        void Print(object text, bool link);
        /// <summary>Print a message to a single user</summary>
        void Print(IUser user, object text);
        /// <summary>Print a message to all users in a vroom</summary>
        void Print(ushort vroom, object text);
        /// <summary>Print a message to all users in a vroom [or also to linked room]</summary>
        void Print(ushort vroom, object text, bool link);
        /// <summary>Print a message to all admins</summary>
        void PrintToAdmins(object text);
        /// <summary>Print a message to all admins [or also linked admins]</summary>
        void PrintToAdmins(object text, bool link);
        /// <summary>Print a message to the linked room</summary>
        void PrintToLink(object text);
        /// <summary>Send a one-off URL update to all users</summary>
        void SendURL(String address, object text);
        /// <summary>Send a fake Private Message to a user</summary>
        void SendPM(IUser target, String sender, String text);
        /// <summary>Send a fake Public Message to a user</summary>
        void SendText(IUser target, String sender, String text);
        /// <summary>Send a fake Emote Message to a user</summary>
        void SendEmote(IUser target, String sender, String text);
        /// <summary>Clear the server's ban list</summary>
        void ClearBans();
        /// <summary>Get first user object in local user pool</summary>
        IUser FirstUser { get; }
        /// <summary>Get first user object in linked user pool</summary>
        IUser FirstLinked { get; }
        /// <summary>Get first user object in banned user pool</summary>
        IBan FirstBanned { get; }
        /// <summary>Get a user object by name substring</summary>
        IUser User(String name);
        /// <summary>Get a user object by id</summary>
        IUser User(ushort id);
        /// <summary>Chatroom stats helper</summary>
        IStats Stats { get; }
        /// <summary>Chatroom link module controller</summary>
        ILinker Linker { get; }
        /// <summary>Chatroom core settings helper</summary>
        IRoom Room { get; }
        /// <summary>Script engine credentials</summary>
        IScript Script { get; }
        /// <summary>ZipLib tools helper</summary>
        ICompress Zip { get; }
        /// <summary>Hashlink tools helper</summary>
        IHashlink Hashlink { get; }
        /// <summary>Get this plugin's path for reading and writing data files</summary>
        String DataPath { get; }
        /// <summary>Get a current selection of UDP server nodes</summary>
        IPEndPoint[] ServerNodes { get; }
    }
}
