﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ICallisto
{
    /// <summary>Plugin entry point</summary>
    public interface IPlugin
    {
        /// <summary>Callback from host server</summary>
        IHost Host { get; set; }
        /// <summary>Custom GUI for plugin</summary>
        UserControl GUI { get; }
        /// <summary>Scripting</summary>
        void STT();
        /// <summary>Captcha event</summary>
        void OnCaptcha(IUser userobj, CaptchaState state);
        /// <summary>Fires when the plugin is loaded</summary>
        void OnPluginLoaded(IHost plugin);
        /// <summary>Fires when the plugin is terminated</summary>
        void OnPluginKilled(IHost plugin);
        /// <summary>Fires when the server has been started or restarted</summary>
        void OnServerStarted();
        /// <summary>Fires when the server has been stopped</summary>
        void OnServerStopped();
        /// <summary>Fires every one second</summary>
        void OnTimer();
        /// <summary>Fires before a user has joined</summary>
        bool OnJoinCheck(IUser userobj);
        /// <summary>Fires when a user was rejected access</summary>
        void OnRejected(IUser userobj, NativeRejectionType type, ref String msg);
        /// <summary>Fires after a user has joined</summary>
        void OnJoin(IUser userobj);
        /// <summary>Fires when a user has left the room</summary>
        void OnPart(IUser userobj);
        /// <summary>Fires before OnTextBefore (ignores custom names etc)</summary>
        void OnTextReceived(IUser userobj, String text);
        /// <summary>Fires before a user's text message is published</summary>
        String OnTextBefore(IUser userobj, String text);
        /// <summary>Fires after a user's text message is published</summary>
        void OnTextAfter(IUser userobj, String text);
        /// <summary>Fires before OnEmoteBefore (ignores custom names etc)</summary>
        void OnEmoteReceived(IUser userobj, String text);
        /// <summary>Fires before a user's emote message is published</summary>
        String OnEmoteBefore(IUser userobj, String text);
        /// <summary>Fires after a user's emote message is published</summary>
        void OnEmoteAfter(IUser userobj, String text);
        /// <summary>Fires when a user submits a room command</summary>
        void OnCommand(IUser userobj, String command_text, IUser target, String args);
        /// <summary>Fires when a user types #help</summary>
        void OnHelp(IUser userobj);
        /// <summary>Fires before a user has changed vroom</summary>
        bool OnVroomJoinCheck(IUser userobj, ushort vroom);
        /// <summary>Fires after a user has changed vroom</summary>
        void OnVroomJoin(IUser userobj);
        /// <summary>Fires after a user has left a vroom</summary>
        void OnVroomPart(IUser userobj);
        /// <summary>Fires before a user is about to flood out of the room</summary>
        bool OnFlood(IUser userobj, ProtoMessage msg);
        /// <summary>Fires after a user has submitted a shared file</summary>
        void OnFileReceived(IUser userobj, String filename, MimeMessage msg);
        /// <summary>Fires before a user publishes a private message</summary>
        bool OnPM(IUser userobj, IUser target, IPMObject pm);
        /// <summary>Fires after a user PMs the bot</summary>
        void OnBotPM(IUser userobj, String text);
        /// <summary>Fires after a user has logged in or attempted to log in</summary>
        void OnLoginRequested(IUser userobj);
        /// <summary>Fires when this plugin throws an exception</summary>
        void OnError(String plugin_name, Exception e);
        /// <summary>Fires when a room link has been established</summary>
        void OnLinkJoin(ILink room);
        /// <summary>Fires when a room link has been disconnected</summary>
        void OnLinkPart(ILink room);
        /// <summary>Fires when a room link throws an exception</summary>
        void OnLinkError(ILink room, LinkError code);
        /// <summary>Fires when a linked admin submites a room command</summary>
        void OnLinkCommand(IUser userobj, String command, IUser target, String args);
        /// <summary>Fires when a user uploads an avatar</summary>
        bool OnAvatarReceived(IUser userobj);
        /// <summary>Fires when a user uploads a personal message</summary>
        bool OnPersonalReceived(IUser userobj, String msg);
    }
}
