﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;

namespace ICallisto
{
    /// <summary>Local/Linked/Parted user object</summary>
    public interface IUser
    {
        /// <summary>Ban user from the chatroom</summary>
        void Ban();
        /// <summary>Disconnect user from the chatroom</summary>
        void Disconnect();
        /// <summary>Redirect the user to a different chatroom</summary>
        void Redirect(String hashlink);
        /// <summary>Clone the user</summary>
        void SendText(String text);
        /// <summary>Clone the user</summary>
        void SendEmote(String text);
        /// <summary>Clone the user's PM</summary>
        void SendPM(String sender, String text);
        /// <summary>Send red text to the user</summary>
        void SendPrint(String text);
        /// <summary>Set a virtual topic for the user</summary>
        void SetTopic(String text);
        /// <summary>Restore user's avatar</summary>
        void RestoreAvatar();
        /// <summary>Download a replacement avatar for the user</summary>
        void DownloadAvatar(String avatar_address);
        /// <summary>Set a URL tag for the user</summary>
        void UpdateURL(String address, String text);
        /// <summary>Send custom client data to this user</summary>
        void SendCustomClientData(String ident, String sender, byte[] data);
        /// <summary>Get the next user in this pool</summary>
        IUser NextUser { get; }
        /// <summary>Get the user's custom font</summary>
        IAresFont Font { get; set; }
        /// <summary>Get/Set the user's custom name</summary>
        String CustomName { get; set; }
        /// <summary>Get the user's original name</summary>
        String OrgName { get; }
        /// <summary>Get/Set the user's current name</summary>
        String Name { get; set; }
        /// <summary>Get the user's client version</summary>
        String Version { get; }
        /// <summary>Get/Set the user's personal message</summary>
        String PersonalMessage { get; set; }
        /// <summary>Get the user's regional location</summary>
        String Region { get; }
        /// <summary>Get the user's guid</summary>
        String Guid { get; }
        /// <summary>Get the user's external ip address</summary>
        IPAddress ExternalIP { get; }
        /// <summary>Get the user's local ip address</summary>
        IPAddress LocalIP { get; }
        /// <summary>Get/Set the user's current vroom</summary>
        ushort Vroom { get; set; }
        /// <summary>Get the user's id number</summary>
        int Id { get; }
        /// <summary>Get the user's file count</summary>
        ushort FileCount { get; }
        /// <summary>Get the user's data port</summary>
        ushort Port { get; }
        /// <summary>Get/Set the user's muzzled status</summary>
        bool Muzzled { get; set; }
        /// <summary>Get the user's age (0 = secret)</summary>
        byte Age { get; set; }
        /// <summary>Get the user's sex (0 = secret, 1 = male, 2 = female)</summary>
        byte Sex { get; set; }
        /// <summary>Get the user's country location code (0 = secret)</summary>
        byte Country { get; }
        /// <summary>Get/Set the user's admin level</summary>
        AdminLevel Level { get; set; }
        /// <summary>Get the user's browse enabled status</summary>
        bool Browsable { get; }
        /// <summary>Get local host status</summary>
        bool IsLocalHost { get; }
        /// <summary>Custom variables and values for this user</summary>
        Dictionary<String, object> CustomData { get; }
        /// <summary>Get an array of users currently being ignored by this user</summary>
        String[] IgnoreList { get; }
        /// <summary>Get linked status of this user</summary>
        bool IsLinked { get; }
        /// <summary>Get/Set the user's avatar</summary>
        byte[] Avatar { get; set; }
        /// <summary>Get/Set the user's cloak status</summary>
        bool Cloaked { get; set; }
        /// <summary>Get the user's fast ping status</summary>
        bool FastPing { get; }
        /// <summary>Get custom client tags</summary>
        String[] CustomClientTags { get; }
    }
}
