﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using System.IO;
using ICallisto;

namespace callisto
{
    class CallistoPluginManager
    {
        public static String CurrentPlugin = String.Empty;

        private List<CallistoPlugin> plugins = new List<CallistoPlugin>();

        public CallistoPlugin[] GetPlugins
        {
            get { return this.plugins.ToArray(); }
        }

        public bool LoadPlugin(PluginListboxItem item)
        {
            try
            {
                CallistoPlugin plugin = new CallistoPlugin();
                plugin.Name = item.plugin_name;
                Assembly asm = Assembly.Load(File.ReadAllBytes(item.path + "\\" + item.plugin_name + ".dll"));

                foreach (Type pluginType in asm.GetTypes())
                {
                    if (pluginType.IsPublic)
                    {
                        if (!pluginType.IsAbstract)
                        {
                            Type typeInterface = pluginType.GetInterface("ICallisto.IPlugin", true);

                            if (typeInterface != null)
                            {
                                plugin.Plugin = (IPlugin)Activator.CreateInstance(asm.GetType(pluginType.ToString()));
                                plugin.Plugin.Host = new CallistoPluginHost(item.path + "\\");
                                this.plugins.Add(plugin);
                                asm = null;
                                CurrentPlugin = plugin.Name;

                                try
                                {
                                    plugin.Plugin.OnPluginLoaded(plugin.Plugin.Host);
                                }
                                catch (Exception e) { ServerEvents.OnError(plugin, e); }
                            }

                            typeInterface = null;
                        }
                    }
                }

                asm = null;
                return true;
            }
            catch { }

            return false;
        }

        public void KillPlugin(PluginListboxItem item)
        {
            CallistoPlugin p = this.plugins.Find(x => x.Name == item.plugin_name);

            if (p != null)
            {
                CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnPluginKilled(p.Plugin.Host);
                }
                catch (Exception e) { ServerEvents.OnError(p, e); }

                this.plugins.RemoveAll(x => x.Name == item.plugin_name);

                foreach (UserObject u in UserPool.Pool)
                    if (u.Connected)
                        if (u.logged_in)
                            u.custom_data_manager.DeleteData(item.plugin_name);

                if (CallistoServer.Linker != null)
                    foreach (LinkedUserObject u in CallistoServer.Linker.linked_users)
                        u.custom_data_manager.DeleteData(item.plugin_name);
            }
        }
    }    
}
