﻿using System;
using System.Linq;
using System.Windows.Forms;
using System.Diagnostics;

namespace callisto
{
    public partial class Form1 : Form
    {
        private CallistoScreen callisto_screen = new CallistoScreen();
        private RoomSettingsScreen room_settings_screen = new RoomSettingsScreen();
        private AdminSystemScreen admin_system_screen = new AdminSystemScreen();
        private RoomLinkingScreen room_linking_screen = new RoomLinkingScreen();
        private ScriptingScreen scripting_screen = new ScriptingScreen();
        private PluginsScreen plugins_screen = new PluginsScreen();
        private ChannelListingScreen channel_listing_screen = new ChannelListingScreen();
        private About about_screen = new About();
        private CallistoServer chatserver = new CallistoServer();
        private PleaseWait please_wait = new PleaseWait();

        public Form1()
        {
            Settings.Load();
            UdpServerPool.Load();
            UserPool.CreateUserPool();
            WebPool.CreateUserPool();
            CallistoPluginStats.ResetStats();
            ServerEvents.Plugins = new CallistoPluginManager();
            InitializeComponent();

            Settings.CanSave = false;
            Passwords.Load();
            TrustedServers.Load();
            BanManager.Load();
            Proxies.Load();

            this.callisto_screen.SetValues();
            this.callisto_screen.OnStatusChanged += new CallistoScreen.CallistoStatusDelegate(this.OnStatusChanged);
            this.callisto_screen.OnRestart += new CallistoScreen.CallistoRestartDelegate(this.OnRestart);
            this.room_settings_screen.SetValues();
            this.admin_system_screen.SetValues();
            this.room_linking_screen.SetValues();
            this.plugins_screen.SetValues();
            this.plugins_screen.OnPluginLoaded += new PluginsScreen.PluginLoadedDelegate(this.OnPluginLoaded);
            this.plugins_screen.OnPluginUnloaded += new PluginsScreen.PluginUnloadedDelegate(this.OnPluginUnloaded);
            this.plugins_screen.OnStallRequired += new PleaseWaitDelegate(this.ShowPleaseWait);
            this.plugins_screen.SetAutoLoads();
            this.scripting_screen.SetValues();
            this.channel_listing_screen.SetValues();
            Settings.CanSave = true;
            this.chatserver.OnUdpReport += new CallistoServer.UdpReportDelegate(this.OnUdpReport);
            
            if (Settings.auto_start)
                this.OnStatusChanged(true, true); // auto start
        }

        private void OnRestart()
        {
            CallistoPlugin[] plugins = ServerEvents.Plugins.GetPlugins;
            this.toolStripStatusLabel1.Text = "Status:  Server is restarting...";
            this.chatserver.Stop();
            Settings.ServerRunning = false;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnServerStopped();
                }
                catch (Exception e) { ServerEvents.OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;

            this.ShowPleaseWait();

            this.toolStripStatusLabel1.Text = "Status:  Server is running";
            Settings.StartTime = Helpers.UnixTime;
            this.chatserver.Start();
            Settings.ServerRunning = true;
            cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnServerStarted();
                }
                catch (Exception e) { ServerEvents.OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
        }

        private void OnPluginUnloaded(string name)
        {
            this.treeView1.Nodes[0].Nodes[4].Nodes.RemoveByKey(name);
        }

        private void OnPluginLoaded(string name)
        {
            TreeNode t = new TreeNode();
            t.Name = name;
            t.Text = name;
            this.treeView1.Nodes[0].Nodes[4].Nodes.Add(t);
            this.treeView1.Nodes[0].Nodes[4].Expand();
        }

        private void OnUdpReport()
        {
            this.channel_listing_screen.UpdateStats();
            this.room_settings_screen.CheckTopic();
        }

        private void OnStatusChanged(bool enabled)
        {
            this.OnStatusChanged(enabled, false);
        }

        private void OnStatusChanged(bool enabled, bool auto_starting)
        {
            this.room_settings_screen.SetReadOnly(!enabled);
            CallistoPlugin[] plugins = ServerEvents.Plugins.GetPlugins;

            if (enabled) // start server
            {
                this.toolStripStatusLabel1.Text = "Status:  Server is running";
                Settings.StartTime = Helpers.UnixTime;
                this.chatserver.Start();
                Settings.ServerRunning = true;
                String cur_plugin = CallistoPluginManager.CurrentPlugin;

                foreach (CallistoPlugin p in plugins)
                {
                    CallistoPluginManager.CurrentPlugin = p.Name;

                    try
                    {
                        p.Plugin.OnServerStarted();
                    }
                    catch (Exception e) { ServerEvents.OnError(p, e); }
                }

                CallistoPluginManager.CurrentPlugin = cur_plugin;

                if (!auto_starting)
                    this.ShowPleaseWait();
            }
            else // stop server
            {
                this.toolStripStatusLabel1.Text = "Status:  Server is stopped";
                this.chatserver.Stop();
                Settings.ServerRunning = false;
                String cur_plugin = CallistoPluginManager.CurrentPlugin;

                foreach (CallistoPlugin p in plugins)
                {
                    CallistoPluginManager.CurrentPlugin = p.Name;

                    try
                    {
                        p.Plugin.OnServerStopped();
                    }
                    catch (Exception e) { ServerEvents.OnError(p, e); }
                }

                CallistoPluginManager.CurrentPlugin = cur_plugin;
            }
        }

        private void SetPanel(UserControl obj)
        {
            this.splitContainer1.Panel2.Controls.Clear();
            obj.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            obj.AutoScroll = true;
     //       obj.BackColor = System.Drawing.Color.White;
            obj.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
            obj.Font = new System.Drawing.Font("Verdana", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            obj.Location = new System.Drawing.Point(3, 3);
            obj.Size = new System.Drawing.Size(426, 315);
            this.splitContainer1.Panel2.Controls.Add(obj);
        }

        private void ShowPleaseWait() // stall GUI input for a second to allow for thread sync
        {
            this.please_wait.Start();
            this.please_wait.ShowDialog();
        }

        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            switch (e.Node.Name)
            {
                case "Node0": // callisto
                    e.Node.Expand();
                    this.SetPanel(this.callisto_screen);
                    break;

                case "Node1": // room settings
                    this.SetPanel(this.room_settings_screen);
                    break;

                case "Node2": // admin settings
                    this.SetPanel(this.admin_system_screen);
                    break;

                case "Node3": // room linking
                    this.SetPanel(this.room_linking_screen);
                    break;

                case "Node4": // scripting
                    this.SetPanel(this.scripting_screen);
                    break;

                case "Node5": // plugins
                    e.Node.Expand();
                    this.SetPanel(this.plugins_screen);
                    break;

                case "Node6": // channel listing
                    this.SetPanel(this.channel_listing_screen);
                    break;

                default: // plugin sub GUI
                    CallistoPlugin plugin = ServerEvents.Plugins.GetPlugins.FirstOrDefault<CallistoPlugin>(x => x.Name == e.Node.Name);

                    if (plugin != null)
                    {
                        CallistoPluginManager.CurrentPlugin = plugin.Name;
                        this.SetPanel(plugin.Plugin.GUI);
                    }
                    break;
            }
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.about_screen.ShowDialog();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            DialogResult d = MessageBox.Show("Are you sure you want to quit Callisto?", "Callisto Server", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

            if (d == DialogResult.No)
            {
                e.Cancel = true;
                return;
            }

            this.chatserver.Stop(true);

            foreach (CallistoPlugin p in ServerEvents.Plugins.GetPlugins)
            {
                try
                {
                    p.Plugin.OnPluginKilled(p.Plugin.Host);
                }
                catch { }
            }
        }

        private void Form1_Resize(object sender, EventArgs e) // minimize to tray
        {
            if (this.WindowState == FormWindowState.Minimized)
            {
                this.Hide();
                this.notifyIcon1.Visible = true;
            }
        }

        private void notifyIcon1_MouseClick(object sender, MouseEventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
            {
                this.Show();
                this.WindowState = FormWindowState.Normal;
                this.notifyIcon1.Visible = false;
            }
        }

        private void openMainDataFolderToolStripMenuItem_Click(object sender, EventArgs e) // open data folder
        {
            try
            {
                Process.Start(Settings.Path);
            }
            catch { }
        }

        private void openMainProgramFolderToolStripMenuItem_Click(object sender, EventArgs e) // open program folder
        {
            try
            {
                Process.Start(AppDomain.CurrentDomain.BaseDirectory);
            }
            catch { }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e) // exit
        {
            this.Close();
        }

        private void viewTheCallistoJavascriptWebsiteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                Process.Start("http://oobe.arca-eclipse.com/callistojs/index.html");
            }
            catch { }
        }

        private void checkForUpdatesToolStripMenuItem_Click(object sender, EventArgs e) // updates
        {
            new UpdateChecker();
        }

        private void scriptModelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                Process.Start("http://callisto.codeplex.com/documentation");
            }
            catch { }
        }

    }
}
