﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using ICallisto;

namespace callisto
{
    class LinkPacketHelpers
    {
        public static byte[] LinkError(LinkError code)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteByte((byte)code);
            return packet.ToAresPacket(ProtoMessage.MSG_SERVER_LINK_ERROR);
        }

        public static byte[] RequestLinkAckPacket()
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteInt16(UserPool.UserCountForLinking);
            packet.WriteInt16(Settings.LINK_PROTO);
            packet.WriteInt16((ushort)Settings.ServerPort);
            packet.WriteString(Settings.ServerName);
            return packet.ToAresPacket(ProtoMessage.MSG_SERVER_TOHUB_LOGINACK);
        }

        public static byte[] RequestLinkPacket()
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteInt16(UserPool.UserCountForLinking);
            packet.WriteInt16(Settings.LINK_PROTO);
            packet.WriteInt16((ushort)Settings.ServerPort);
            packet.WriteString(Settings.ServerName);
            packet.WriteString("english");
            return packet.ToAresPacket(ProtoMessage.MSG_SERVER_TOHUB_LOGINREQ);
        }

        public static byte[] Part(LinkedUserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.name);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_PART);
        }

        public static byte[] Pong()
        {
            return new AresPacketWriter().ToAresPacket(ProtoMessage.MSG_SERVER_PONG);
        }

        public static byte[] UserListItem(LinkedUserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteInt16(userobj.file_count);
            packet.WriteInt32(0);
            packet.WriteIP(userobj.external_ip);
            packet.WriteInt16(userobj.dc_port);
            packet.WriteIP(userobj.node_ip);
            packet.WriteInt16(userobj.node_port);
            packet.WriteByte(0);
            packet.WriteString(userobj.name);
            packet.WriteIP(userobj.local_ip);
            packet.WriteByte(userobj.browsable ? (byte)1 : (byte)0);
            packet.WriteByte((byte)userobj.level);
            packet.WriteByte(userobj.age);
            packet.WriteByte(userobj.sex);
            packet.WriteByte(userobj.country);
            packet.WriteString(userobj.region);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_CHANNEL_USER_LIST);
        }

        public static byte[] PersonalMessage(LinkedUserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.name);
            packet.WriteString(userobj.personal_message, false);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_PERSONAL_MESSAGE);
        }

        public static byte[] Avatar(LinkedUserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.name);
            packet.WriteBytes(userobj.avatar);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_AVATAR);
        }

        public static byte[] AvatarCleared(LinkedUserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.name);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_AVATAR);
        }

        public static byte[] Update(LinkedUserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteInt16(userobj.file_count);
            packet.WriteInt32(0);
            packet.WriteIP(userobj.external_ip);
            packet.WriteInt16(userobj.dc_port);
            packet.WriteIP(userobj.node_ip);
            packet.WriteInt16(userobj.node_port);
            packet.WriteByte(0);
            packet.WriteString(userobj.name);
            packet.WriteIP(userobj.local_ip);
            packet.WriteByte(userobj.browsable ? (byte)1 : (byte)0);
            packet.WriteByte((byte)userobj.level);
            packet.WriteByte(userobj.age);
            packet.WriteByte(userobj.sex);
            packet.WriteByte(userobj.country);
            packet.WriteString(userobj.region);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_UPDATE_USER_STATUS);
        }

        public static byte[] RelayToUser(String target, byte[] data)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(target);
            packet.WriteBytes(data);
            return packet.ToAresPacket(ProtoMessage.MSG_SERVER_RELAYTOUSER);
        }

        public static byte[] Broadcast(byte[] data)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteBytes(data);
            return packet.ToAresPacket(ProtoMessage.MSG_SERVER_BROADCAST);
        }

        public static byte[] AdminMessage(String text)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(text, false);
            return packet.ToAresPacket(ProtoMessage.MSG_SERVER_ADMIN_MESSAGE);
        }

        public static byte[] AdminCommand(String admin_name, String target_name, String command_ident, String args)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(admin_name);
            packet.WriteString(target_name);
            packet.WriteString(command_ident);
            packet.WriteString(args, false);
            return packet.ToAresPacket(ProtoMessage.MSG_SERVER_ADMIN_COMMAND);
        }

        public static byte[] ServerCommand(String target_name, String command_ident, String[] args)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(target_name);
            packet.WriteString(command_ident);

            for (int i = 0; i < args.Length; i++)
                if (i == (args.Length - 1))
                    packet.WriteString(args[i], false);
                else
                    packet.WriteString(args[i]);

            return packet.ToAresPacket(ProtoMessage.MSG_SERVER_SERVER_COMMAND);
        }

        public static byte[] ServerCommand(String target_name, String command_ident, byte[] args)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(target_name);
            packet.WriteString(command_ident);
            packet.WriteBytes(args);
            return packet.ToAresPacket(ProtoMessage.MSG_SERVER_SERVER_COMMAND);
        }

        public static byte[] BrowseRequest(UserObject userobj, LinkedUserObject target, ushort ident, byte mime)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.name);
            packet.WriteInt16(ident);
            packet.WriteByte(80);
            packet.WriteByte(mime);
            packet.WriteString(target.name);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_CLIENT_BROWSE);
        }

        public static byte[] Join(LinkedUserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteInt16(userobj.file_count);
            packet.WriteInt32(0);
            packet.WriteIP(userobj.external_ip);
            packet.WriteInt16(userobj.dc_port);
            packet.WriteIP(userobj.node_ip);
            packet.WriteInt16(userobj.node_port);
            packet.WriteByte(0);
            packet.WriteString(userobj.name);
            packet.WriteIP(userobj.local_ip);
            packet.WriteByte(userobj.browsable ? (byte)1 : (byte)0);
            packet.WriteByte((byte)userobj.level);
            packet.WriteByte(userobj.age);
            packet.WriteByte(userobj.sex);
            packet.WriteByte(userobj.country);
            packet.WriteString(userobj.region);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_JOIN);
        }

        public static byte[] Generic(ProtoMessage msg, byte[] data)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteBytes(data);
            return packet.ToAresPacket(msg);
        }

        public static byte[] TextReceived(IUser userobj, String text)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.Name);
            packet.WriteString(text, false);
            return packet.ToAresPacket(ProtoMessage.MSG_SERVER_TEXT_RECEIVED);
        }

        public static byte[] EmoteReceived(IUser userobj, String text)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.Name);
            packet.WriteString(text, false);
            return packet.ToAresPacket(ProtoMessage.MSG_SERVER_EMOTE_RECEIVED);
        }

        public static byte[] ForcePM(IUser target, String sender, String text)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(sender);
            packet.WriteString(target.Name);
            packet.WriteString(text, false);
            return packet.ToAresPacket(ProtoMessage.MSG_SERVER_FORCE_PM);
        }

        public static byte[] SetCustomTags(IUser userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.Name);

            foreach (String str in userobj.CustomClientTags)
                packet.WriteString(str);

            return packet.ToAresPacket(ProtoMessage.MSG_SERVER_CUSTOM_SET_TAGS);
        }

        public static byte[] SendCustomNameText(IUser userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.Name);
            packet.WriteString(userobj.CustomName);
            return packet.ToAresPacket(ProtoMessage.MSG_SERVER_CUSTOM_NAME_TEXT);
        }

        public static byte[] SendMuzzleUpdate(IUser userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.Name);
            packet.WriteByte(userobj.Muzzled ? (byte)1 : (byte)0);
            return packet.ToAresPacket(ProtoMessage.MSG_SERVER_MUZZLE_UPDATE);
        }

    }
}
