﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ICallisto;

namespace callisto
{
    class LinkPacketProcessor
    {
        public static void Eval(ProtoMessage msg, AresPacketReader packet, uint time)
        {
            switch (msg)
            {
                case ProtoMessage.MSG_CHAT_SERVER_UPDATE_USER_STATUS:
                    ProcessUpdate(packet);
                    break;

                case ProtoMessage.MSG_CHAT_SERVER_AVATAR:
                    ProcessAvatar(packet);
                    break;

                case ProtoMessage.MSG_CHAT_SERVER_PERSONAL_MESSAGE:
                    ProcessPersonalMessage(packet);
                    break;

                case ProtoMessage.MSG_CHAT_SERVER_PUBLIC:
                    ProcessPublic(packet);
                    break;

                case ProtoMessage.MSG_CHAT_SERVER_EMOTE:
                    ProcessEmote(packet);
                    break;

                case ProtoMessage.MSG_CHAT_SERVER_JOIN:
                    ProcessJoin(packet, false);
                    break;

                case ProtoMessage.MSG_CHAT_SERVER_PART:
                    ProcessPart(packet);
                    break;

                case ProtoMessage.MSG_CHAT_SERVER_CHANNEL_USER_LIST:
                    ProcessJoin(packet, true);
                    break;

                case ProtoMessage.MSG_CHAT_CLIENT_BROWSE:
                    ProcessBrowseRequest(packet);
                    break;

                case ProtoMessage.MSG_SERVER_LINK_ERROR:
                    ProcessLinkError(packet);
                    break;

                case ProtoMessage.MSG_SERVER_BROADCAST:
                    UserPool.BroadcastToVroom(0, packet.ReadBytes());
                    break;

                case ProtoMessage.MSG_SERVER_RELAYTOUSER:
                    ProcessRelayToUser(packet);
                    break;

                case ProtoMessage.MSG_SERVER_PONG:
                    CallistoServer.Linker.last_update_received_time = time;
                    break;

                case ProtoMessage.MSG_SERVER_TOHUB_LOGINACK:
                    ProcessAck(packet);
                    break;

                case ProtoMessage.MSG_SERVER_ADMIN_COMMAND: // custom commands going to OnLinkCommand event
                    ProcessAdminCommand(packet);
                    break;

                case ProtoMessage.MSG_SERVER_SERVER_COMMAND: // user.SendText, user.SendPM and so on
                    ProcessServerCommand(packet);
                    break;

                case ProtoMessage.MSG_SERVER_ADMIN_MESSAGE:
                    ProcessAdminMessage(packet);
                    break;

                case ProtoMessage.MSG_SERVER_TEXT_RECEIVED:
                    ProcessTextReceived(packet);
                    break;

                case ProtoMessage.MSG_SERVER_EMOTE_RECEIVED:
                    ProcessEmoteReceived(packet);
                    break;

                case ProtoMessage.MSG_SERVER_FORCE_PM:
                    ProcessForcePM(packet);
                    break;

                case ProtoMessage.MSG_SERVER_CUSTOM_SET_TAGS:
                    ProcessCustomDataSetTags(packet);
                    break;

                case ProtoMessage.MSG_SERVER_CUSTOM_NAME_TEXT:
                    ProcessCustomNameText(packet);
                    break;

                case ProtoMessage.MSG_SERVER_MUZZLE_UPDATE:
                    ProcessMuzzleUpdate(packet);
                    break;
            }
        }

        private static void ProcessCustomNameText(AresPacketReader packet)
        {
            String name = packet.ReadString();
            String text = packet.ReadString();
            LinkedUserObject target = CallistoServer.Linker.linked_users.Find(l => l.name == name);

            if (target != null)
                target.custom_name = text;
        }

        private static void ProcessCustomDataSetTags(AresPacketReader packet)
        {
            String name = packet.ReadString();
            UserObject u = UserPool.FindUserByName(name);

            if (u != null)
            {
                u.custom_tags.Clear();

                while (packet.Remaining() > 0)
                    u.custom_tags.Add(packet.ReadString());
            }
        }

        private static void ProcessUpdate(AresPacketReader packet)
        {
            String name = packet.ReadString();
            LinkedUserObject target = CallistoServer.Linker.linked_users.Find(l => l.name == name);

            if (target != null)
            {
                packet.SkipBytes(13);
                target.level = packet.ReadByte() > 0 ? AdminLevel.Moderator : AdminLevel.Regular;

                if (target.visible)
                {
                    UserPool.BroadcastToVroom(0, LinkPacketHelpers.Update(target));
                    WebPool.BroadcastAresUpdate(target);
                }
            }
        }

        private static void ProcessAvatar(AresPacketReader packet)
        {
            String name = packet.ReadString();
            LinkedUserObject target = CallistoServer.Linker.linked_users.Find(l => l.name == name);

            if (target != null)
            {
                if (!ServerEvents.OnAvatarReceived(target))
                    return;

                byte[] new_avatar = packet.ReadBytes();

                if (target.avatar != null)
                    if (target.avatar.SequenceEqual<byte>(new_avatar))
                        return; // it's a duplicate

                new_avatar = new_avatar.Length <= 1 ? null : new_avatar;

                if (target.avatar == null && new_avatar == null)
                    return; // it's a duplicate

                target.avatar = new_avatar;

                if (target.visible)
                {
                    if (target.avatar != null)
                        UserPool.BroadcastToVroom(0, LinkPacketHelpers.Avatar(target));
                    else UserPool.BroadcastToVroom(0, LinkPacketHelpers.AvatarCleared(target));
                }
            }
        }

        private static void ProcessPersonalMessage(AresPacketReader packet)
        {
            String name = packet.ReadString();
            LinkedUserObject target = CallistoServer.Linker.linked_users.Find(l => l.name == name);

            if (target != null)
            {
                String text = packet.ReadString();

                if (!ServerEvents.OnPersonalReceived(target, text))
                    return;

                target.personal_message = text;

                if (target.visible)
                    UserPool.BroadcastToVroom(0, LinkPacketHelpers.PersonalMessage(target));
            }
        }

        private static void ProcessTextReceived(AresPacketReader packet)
        {
            String name = packet.ReadString();
            String text = packet.ReadString();
            LinkedUserObject target = CallistoServer.Linker.linked_users.Find(l => l.name == name);

            if (text.Length == 0 || target == null)
                return;

            ServerEvents.OnTextReceived(target, text);
        }

        private static void ProcessEmoteReceived(AresPacketReader packet)
        {
            String name = packet.ReadString();
            String text = packet.ReadString();
            LinkedUserObject target = CallistoServer.Linker.linked_users.Find(l => l.name == name);

            if (text.Length == 0 || target == null)
                return;

            ServerEvents.OnEmoteReceived(target, text);
        }

        private static void ProcessPublic(AresPacketReader packet)
        {
            String name = packet.ReadString();
            String text = packet.ReadString();
            LinkedUserObject target = CallistoServer.Linker.linked_users.Find(l => l.name == name);
            
            if (text.Length == 0 || target == null)
                return;
            
            Helpers.SoftenFlooding();
            text = ServerEvents.OnTextBefore(target, text);
            
            if (text.Length > 0)
            {
                if (target.CustomName.Length > 0 && Settings.CAN_CUSTOM_NAME)
                {
                    UserPool.BroadcastToUnignored(target, ChatPacketHelpers.NoSuch(target.CustomName + text));
                    WebPool.BroadcastCustomNamePublic(target, text);
                }
                else
                {
                    UserPool.BroadcastToUnignored(target, ChatPacketHelpers.Public(target.name, text));
                    WebPool.BroadcastPublic(target, text);
                }

                ServerEvents.OnTextAfter(target, text);
            }
        }

        private static void ProcessEmote(AresPacketReader packet)
        {
            String name = packet.ReadString();
            String text = packet.ReadString();
            LinkedUserObject target = CallistoServer.Linker.linked_users.Find(l => l.name == name);

            if (text.Length == 0 || target == null)
                return;

            Helpers.SoftenFlooding();
            text = ServerEvents.OnEmoteBefore(target, text);

            if (text.Length > 0)
            {
                UserPool.BroadcastToUnignored(target, ChatPacketHelpers.Emote(target.name, text));
                WebPool.BroadcastEmote(target, text);
                ServerEvents.OnEmoteAfter(target, text);
            }
        }

        private static void ProcessMuzzleUpdate(AresPacketReader packet)
        {
            String name = packet.ReadString();
            bool is_muzzled = packet.ReadByte() == 1;
            LinkedUserObject target = CallistoServer.Linker.linked_users.Find(l => l.name == name);

            if (target != null)
                target.muz = is_muzzled;
        }

        private static void ProcessJoin(AresPacketReader packet, bool userlist_item)
        {
            LinkedUserObject target = new LinkedUserObject();
            target.file_count = packet.ReadInt16();
            packet.SkipBytes(4);
            target.external_ip = packet.ReadIP();
            target.dc_port = packet.ReadInt16();
            target.node_ip = packet.ReadIP();
            target.node_port = packet.ReadInt16();
            packet.SkipByte();
            target.name = packet.ReadString();
            target.local_ip = packet.ReadIP();
            target.browsable = packet.ReadByte() == 1;

            if (!target.browsable)
                target.file_count = 0;

            target.level = packet.ReadByte() >= 1 ? AdminLevel.Moderator : AdminLevel.Regular;
            target.version = packet.ReadString();
            target.guid = packet.ReadGuid();
            target.muz = packet.ReadByte() == 1;
            target.age = packet.ReadByte();
            target.sex = packet.ReadByte();
            target.country = packet.ReadByte();
            target.region = packet.ReadString();

            if (CallistoServer.Linker.linked_users.FindIndex(l => l.name == target.name) == -1)
            {
                CallistoServer.Linker.linked_users.Add(target);

                if (UserPool.FindUserByName(target.name, 0) == null)
                {
                    target.visible = true;
                    UserPool.BroadcastToVroom(0, LinkPacketHelpers.Join(target));
                    WebPool.BroadcastAresJoin(target);
                }

                ServerEvents.OnJoin(target);
            }
        }

        private static void ProcessPart(AresPacketReader packet)
        {
            String name = packet.ReadString();
            LinkedUserObject target = CallistoServer.Linker.linked_users.Find(l => l.name == name);

            IUser existing = UserPool.FindUserByName(name, 0);

            if (existing == null)
                existing = WebPool.FindUserByName(name, 0);

            if (target != null)
            {
                if (existing == null)
                {
                    UserPool.RemoveIgnoredUser(name);
                    UserPool.BroadcastToVroom(0, LinkPacketHelpers.Part(target));
                    WebPool.BroadcastAresPart(target);
                }

                ServerEvents.OnPart(target);
                CallistoServer.Linker.linked_users.RemoveAll(l => l.name == name);
            }
        }

        private static void ProcessBrowseRequest(AresPacketReader packet)
        {
            String requester = packet.ReadString();
            ushort ident = packet.ReadInt16();
            packet.SkipByte();
            byte mime = packet.ReadByte();
            String _target = packet.ReadString();
            UserObject target = UserPool.FindUserByName(_target);
            List<byte> compressed_packets = new List<byte>();

            if (target == null)
            {
                CallistoServer.Linker.SendPacket(LinkPacketHelpers.RelayToUser(requester, ChatPacketHelpers.BrowseError(ident)));
                return;
            }

            if (target.files.Count == 0)
            {
                CallistoServer.Linker.SendPacket(LinkPacketHelpers.RelayToUser(requester, ChatPacketHelpers.BrowseError(ident)));
                return;
            }

            SharedFileObject[] files;

            if (mime == 255)
                files = target.files.ToArray();
            else
                files = target.files.FindAll(f => (byte)f.type == mime).ToArray();

            CallistoServer.Linker.SendPacket(LinkPacketHelpers.RelayToUser(requester, ChatPacketHelpers.StartOfBrowse(ident, (ushort)files.Length)));

            for (int i = 0; i < files.Length; i++)
            {
                compressed_packets.AddRange(LinkPacketHelpers.RelayToUser(requester, ChatPacketHelpers.BrowseItem(ident, files[i])));

                if (compressed_packets.Count > 800)
                {
                    CallistoServer.Linker.SendPacket(ChatPacketHelpers.ClientCompressed(compressed_packets.ToArray()));
                    compressed_packets.Clear();
                }
            }

            if (compressed_packets.Count > 0)
            {
                CallistoServer.Linker.SendPacket(ChatPacketHelpers.ClientCompressed(compressed_packets.ToArray()));
                compressed_packets.Clear();
            }

            CallistoServer.Linker.SendPacket(LinkPacketHelpers.RelayToUser(requester, ChatPacketHelpers.EndOfBrowse(ident)));
        }

        private static void ProcessLinkError(AresPacketReader packet)
        {
            LinkError error = (LinkError)packet.ReadByte();

            switch (error)
            {
                case LinkError.BannedUser:
                case LinkError.LinkingDisabled:
                case LinkError.LinkToSelf:
                case LinkError.NoInbound:
                case LinkError.NotOnAllowList:
                case LinkError.OldProtocol:
                case LinkError.PingTimeout:
                case LinkError.WrongLanguage:
                    CallistoServer.Linker.PartAndDisconnect();
                    ServerEvents.OnLinkError(CallistoServer.Linker, error);
                    break;

                default:
                    ServerEvents.OnLinkError(CallistoServer.Linker, error);
                    break;
            }
        }

        private static void ProcessAdminCommand(AresPacketReader packet)
        {
            if (!Settings.AdminLinking)
            {
                CallistoServer.Linker.SendPacket(LinkPacketHelpers.LinkError(LinkError.LinkAdminDisabled));
                return;
            }

            String _admin = packet.ReadString();
            String _target = packet.ReadString();
            String command = packet.ReadString();
            String args = packet.ReadString();
            IUser admin = CallistoServer.Linker.linked_users.Find(l => l.name == _admin);
            IUser target = UserPool.FindUserByName(_target);

            if (target == null)
                target = WebPool.FindUserByName(_target);

            if (admin != null && target != null)
                ServerEvents.OnLinkCommand(admin, command, target, args);
        }

        private static void ProcessAdminMessage(AresPacketReader packet)
        {
            if (!Settings.AdminLinking)
                return;

            UserPool.BroadcastToAdmins(ChatPacketHelpers.NoSuch(packet.ReadString()));
        }

        private static void ProcessServerCommand(AresPacketReader packet)
        {
            if (!Settings.AdminLinking)
                return;

            String _name = packet.ReadString();

            IUser target = UserPool.FindUserByName(_name);

            if (target == null)
                target = WebPool.FindUserByName(_name);

            if (target == null)
                return;

            String command = packet.ReadString();

            switch (command)
            {
                case "vroom":
                    if (target.IsLocalHost || target.Level > 0) return;
                    target.Vroom = ushort.Parse(packet.ReadString());
                    break;

                case "personal_message":
                    target.PersonalMessage = packet.ReadString();
                    break;

                case "avatar":
                    target.Avatar = packet.ReadBytes();
                    break;

                case "avatar_blank":
                    target.Avatar = null;
                    break;

                case "avatar_download":
                    target.DownloadAvatar(packet.ReadString());
                    break;

                case "avatar_org":
                    target.RestoreAvatar();
                    break;

                case "rename":
                    if (target.IsLocalHost || target.Level > 0) return;
                    target.Name = packet.ReadString();
                    break;

                case "topic":
                    target.SetTopic(packet.ReadString());
                    break;

                case "pm":
                    target.SendPM(packet.ReadString(), packet.ReadString());
                    break;

                case "emote":
                    target.SendEmote(packet.ReadString());
                    break;

                case "public":
                    target.SendText(packet.ReadString());
                    break;

                case "redirect":
                    if (target.IsLocalHost || target.Level > 0) return;
                    target.Redirect(packet.ReadString());
                    break;

                case "ban":
                    if (target.IsLocalHost || target.Level > 0) return;
                    target.Ban();
                    break;

                case "disconnect":
                    if (target.IsLocalHost || target.Level > 0) return;
                    target.Disconnect();
                    break;

                case "url":
                    target.UpdateURL(packet.ReadString(), packet.ReadString());
                    break;

                case "muzzle":
                    if (target.IsLocalHost || target.Level > 0) return;
                    target.Muzzled = packet.ReadByte() == 1;
                    break;

                case "custom_name":
                    target.CustomName = packet.ReadString();
                    break;
            }
        }

        private static void ProcessAck(AresPacketReader packet)
        {
            packet.SkipBytes(2);
            CallistoServer.Linker.protocol = packet.ReadInt16();

            if (CallistoServer.Linker.protocol < Settings.LINK_PROTO)
            {
                CallistoServer.Linker.SendPacket(LinkPacketHelpers.LinkError(LinkError.OldProtocol));
                CallistoServer.Linker.SendAndDisconnect();
                ServerEvents.OnLinkError(CallistoServer.Linker, LinkError.OldProtocol);
                return;
            }

            packet.SkipBytes(2);
            CallistoServer.Linker.room_name = packet.ReadString();
            CallistoServer.Linker.logged_in = true;
            CallistoServer.Linker.BuildHashlink();
            UserPool.UncloakAll();
            UserPool.SendUserlistToLink(CallistoServer.Linker);
            UserPool.SendAvatarsToLink(CallistoServer.Linker);
            UserPool.SendCustomTagsToLink(CallistoServer.Linker);
            UserPool.SendCustomNamesToLink(CallistoServer.Linker);
            ServerEvents.OnLinkJoin(CallistoServer.Linker);
        }

        private static void ProcessRelayToUser(AresPacketReader packet)
        {
            String _name = packet.ReadString();
            IUser target = UserPool.FindUserByName(_name);

            if (target == null)
                target = WebPool.FindUserByName(_name);

            if (target == null)
                return;

            packet.SkipBytes(2);
            ProtoMessage msg = (ProtoMessage)packet.ReadByte();

            switch (msg)
            {
                case ProtoMessage.MSG_CHAT_SERVER_PVT:
                    ProcessPrivateMessage(target, packet);
                    break;

                case ProtoMessage.MSG_CHAT_SERVER_STARTOFBROWSE:
                    ProcessStartOfBrowse(target, packet);
                    break;

                case ProtoMessage.MSG_CHAT_SERVER_BROWSEITEM:
                    ProcessBrowseItem(target, packet);
                    break;

                case ProtoMessage.MSG_CHAT_SERVER_ENDOFBROWSE:
                    ProcessEndOfBrowse(target, packet);
                    break;

                case ProtoMessage.MSG_CHAT_SERVER_PUBLIC:
                    ProcessPublicToUser(target, packet);
                    break;

                case ProtoMessage.MSG_CHAT_SERVER_EMOTE:
                    ProcessEmoteToUser(target, packet);
                    break;

                default:
                    if (target is UserObject)
                        ((UserObject)target).SendPacket(LinkPacketHelpers.Generic(msg, packet.ReadBytes()));
                    break;
            }
        }

        private static void ProcessPublicToUser(IUser target, AresPacketReader packet)
        {
            String sender = packet.ReadString();
            String text = packet.ReadString();

            if (target is UserObject)
                ((UserObject)target).SendPacket(ChatPacketHelpers.Public(sender, text));
            else if (target is WebUser)
                ((WebUser)target).QueuePacket(WebPackets.PublicTo((WebUser)target, sender, text));
        }

        private static void ProcessEmoteToUser(IUser target, AresPacketReader packet)
        {
            String sender = packet.ReadString();
            String text = packet.ReadString();

            if (target is UserObject)
                ((UserObject)target).SendPacket(ChatPacketHelpers.Emote(sender, text));
            else if (target is WebUser)
                ((WebUser)target).QueuePacket(WebPackets.EmoteTo((WebUser)target, sender, text));
        }

        private static void ProcessPrivateMessage(IUser userobj, AresPacketReader packet)
        {
            if (!(userobj is UserObject))
                return;

            UserObject tmp = (UserObject)userobj;
            String name = packet.ReadString();
            String text = packet.ReadString();
            LinkedUserObject target = CallistoServer.Linker.linked_users.Find(l => l.name == name);

            if (target == null)
                return;

            if (tmp.ignores.Contains(target.name))
            {
                CallistoServer.Linker.SendPacket(LinkPacketHelpers.RelayToUser(target.name, ChatPacketHelpers.IsIgnoringYou(tmp.name)));
            }
            else
            {
                text = ServerEvents.OnPM(target, userobj, text);

                if (text.Length > 0)
                    tmp.SendPacket(ChatPacketHelpers.Private(target.name, text));
            }
        }

        private static void ProcessForcePM(AresPacketReader packet)
        {
            String from_name = packet.ReadString();
            String to_name = packet.ReadString();
            String text = packet.ReadString();
            UserObject u = UserPool.FindUserByName(to_name);

            if (u != null)
                u.SendPluginPM(from_name, text);
        }

        private static void ProcessStartOfBrowse(IUser usr, AresPacketReader packet)
        {
            if (!(usr is UserObject))
                return;

            UserObject userobj = (UserObject)usr;

            userobj.SendPacket(LinkPacketHelpers.Generic(ProtoMessage.MSG_CHAT_SERVER_STARTOFBROWSE, packet.ReadBytes()));

            if (userobj.supports_compression)
                userobj.link_data_in.Clear();
        }

        private static void ProcessBrowseItem(IUser usr, AresPacketReader packet)
        {
            if (!(usr is UserObject))
                return;

            UserObject userobj = (UserObject)usr;

            byte[] data = LinkPacketHelpers.Generic(ProtoMessage.MSG_CHAT_SERVER_BROWSEITEM, packet.ReadBytes());

            if (userobj.supports_compression)
            {
                userobj.link_data_in.AddRange(data);

                if (userobj.link_data_in.Count > 800)
                {
                    data = ChatPacketHelpers.ClientCompressed(userobj.link_data_in.ToArray());
                    userobj.link_data_in.Clear();
                    userobj.SendPacket(data);
                }
            }
            else { userobj.SendPacket(data); }
        }

        private static void ProcessEndOfBrowse(IUser usr, AresPacketReader packet)
        {
            if (!(usr is UserObject))
                return;

            UserObject userobj = (UserObject)usr;

            if (userobj.supports_compression)
            {
                if (userobj.link_data_in.Count > 0)
                {
                    byte[] data = ChatPacketHelpers.ClientCompressed(userobj.link_data_in.ToArray());
                    userobj.link_data_in.Clear();
                    userobj.SendPacket(data);
                }
            }

            userobj.SendPacket(LinkPacketHelpers.Generic(ProtoMessage.MSG_CHAT_SERVER_ENDOFBROWSE, packet.ReadBytes()));
        }

    }
}
