﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using ICallisto;

namespace callisto
{
    class LinkedUserObject : IUser
    {
        internal CallistoCustomDataManager custom_data_manager = new CallistoCustomDataManager();
        internal bool browsable;
        internal AdminLevel level;
        internal byte country;
        internal byte sex;
        internal byte age;
        internal ushort dc_port;
        internal ushort file_count;
        internal IPAddress local_ip;
        internal IPAddress external_ip;
        internal String region;
        internal String personal_message = String.Empty;
        internal String version;
        internal String name;
        internal bool visible = false;
        internal byte[] avatar;
        internal IPAddress node_ip;
        internal ushort node_port;
        internal Guid guid;
        internal List<String> custom_tags = new List<String>();
        internal bool muz = false;
        internal String custom_name = String.Empty;

        public bool Muzzled
        {
            get { return this.muz; }
            set
            {
                CallistoServer.Linker.SendPacket(LinkPacketHelpers.ServerCommand(this.name, "muzzle", new byte[] { value ? (byte)1 : (byte)0 }));
            }
        }

        public String CustomName
        {
            get { return this.custom_name; }
            set
            {
                String str = value;

                if (str == null)
                    str = String.Empty;

                CallistoServer.Linker.SendPacket(LinkPacketHelpers.ServerCommand(this.name, "custom_name", new String[] { str }));
            }
        }

        public bool FastPing { get { return false; } }

        public bool Cloaked
        {
            get { return false; }
            set { }
        }

        public IAresFont Font
        {
            get { return null; }
            set { }
        }

        public String[] CustomClientTags
        {
            get { return this.custom_tags.ToArray(); }
        }

        public byte[] Avatar
        {
            get { return this.avatar; }
            set
            {
                if (value.Length > 0)
                    CallistoServer.Linker.SendPacket(LinkPacketHelpers.ServerCommand(this.name, "avatar", value));
                else CallistoServer.Linker.SendPacket(LinkPacketHelpers.ServerCommand(this.name, "avatar_clear", new String[] { }));
            }
        }

        public void DownloadAvatar(String avatar_address)
        {
            CallistoServer.Linker.SendPacket(LinkPacketHelpers.ServerCommand(this.name, "avatar_download", new String[] { avatar_address }));
        }

        public void RestoreAvatar()
        {
            CallistoServer.Linker.SendPacket(LinkPacketHelpers.ServerCommand(this.name, "avatar_org", new String[] { }));
        }

        public void SendCustomClientData(String ident, String sender, byte[] data)
        {
            CallistoServer.Linker.SendPacket(LinkPacketHelpers.RelayToUser(this.name, ChatPacketHelpers.CustomData(sender, ident, data)));
        }

        public bool IsLinked { get { return true; } }

        public String[] IgnoreList { get { return new String[] { }; } }

        public Dictionary<String, object> CustomData { get { return this.custom_data_manager.GetData; } }

        public bool IsLocalHost { get { return false; } }

        public bool Browsable { get { return this.browsable; } }

        public AdminLevel Level
        {
            get { return this.level; }
            set { }
        }

        public byte Country { get { return this.country; } }

        public byte Sex { get { return this.sex; } set { } }

        public byte Age { get { return this.age; } set { } }

        public ushort Port { get { return this.dc_port; } }

        public ushort FileCount { get { return this.file_count; } }

        public int Id { get { return -1; } }

        public ushort Vroom
        {
            get { return 0; }
            set
            {
                CallistoServer.Linker.SendPacket(LinkPacketHelpers.ServerCommand(this.name, "vroom", new String[] { value.ToString() }));
            }
        }

        public IPAddress LocalIP { get { return this.local_ip; } }

        public IPAddress ExternalIP { get { return this.external_ip; } }

        public String Guid { get { return Convert.ToBase64String(this.guid.ToByteArray()); } }

        public String Region { get { return this.region; } }

        public String PersonalMessage
        {
            get { return this.personal_message; }
            set
            {
                CallistoServer.Linker.SendPacket(LinkPacketHelpers.ServerCommand(this.name, "personal_message", new String[] { value }));
            }
        }

        public IUser NextUser
        {
            get
            {
                int i = CallistoServer.Linker.linked_users.FindIndex(l => l.name == this.name);

                if (i == -1)
                    return null;

                if ((i + 1) >= CallistoServer.Linker.linked_users.Count)
                    return null;

                return CallistoServer.Linker.linked_users[i + 1];
            }
        }

        public String Version { get { return this.version; } }

        public String Name
        {
            get { return this.name; }
            set
            {
                CallistoServer.Linker.SendPacket(LinkPacketHelpers.ServerCommand(this.name, "rename", new String[] { value }));
            }
        }

        public String OrgName { get { return this.name; } }

        public void SetTopic(String text)
        {
            CallistoServer.Linker.SendPacket(LinkPacketHelpers.ServerCommand(this.name, "topic", new String[] { text }));
        }

        public void SendPM(String name, String text)
        {
            CallistoServer.Linker.SendPacket(LinkPacketHelpers.ServerCommand(this.name, "pm", new String[] { name, text }));
        }

        public void SendEmote(String text)
        {
            CallistoServer.Linker.SendPacket(LinkPacketHelpers.ServerCommand(this.name, "emote", new String[] { text }));
        }

        public void SendText(String text)
        {
            CallistoServer.Linker.SendPacket(LinkPacketHelpers.ServerCommand(this.name, "public", new String[] { text }));
        }

        public void SendPrint(String text)
        {
            CallistoServer.Linker.SendPacket(LinkPacketHelpers.RelayToUser(this.name, ChatPacketHelpers.NoSuch(text)));
        }

        public void Redirect(String hashlink)
        {
            CallistoServer.Linker.SendPacket(LinkPacketHelpers.ServerCommand(this.name, "redirect", new String[] { hashlink }));
        }

        public void Disconnect()
        {
            CallistoServer.Linker.SendPacket(LinkPacketHelpers.ServerCommand(this.name, "disconnect", new String[] { }));
        }

        public void Ban()
        {
            CallistoServer.Linker.SendPacket(LinkPacketHelpers.ServerCommand(this.name, "ban", new String[] { }));
        }

        public void UpdateURL(String address, String text)
        {
            CallistoServer.Linker.SendPacket(LinkPacketHelpers.ServerCommand(this.name, "url", new String[] { address, text }));
        }

        public override bool Equals(object obj)
        {
            IUser user = (IUser)obj;
            return this.Guid.Equals(user.Guid) || this.ExternalIP.Equals(user.ExternalIP);
        }

        public override int GetHashCode() { return base.GetHashCode(); }
    }
}
