﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.IO;
using System.Xml;

namespace callisto
{
    public partial class PluginsScreen : UserControl
    {
        private ToolTip tip = new ToolTip();
        private List<PluginListboxItem> available_list = new List<PluginListboxItem>();
        private List<PluginListboxItem> loaded_list = new List<PluginListboxItem>();

        public delegate void PluginLoadedDelegate(String name);
        public delegate void PluginUnloadedDelegate(String name);
        public event PluginLoadedDelegate OnPluginLoaded;
        public event PluginUnloadedDelegate OnPluginUnloaded;
        public event PleaseWaitDelegate OnStallRequired;

        public PluginsScreen()
        {
            InitializeComponent();
        }

        public void SetValues()
        {
            this.available_list.Clear();
            this.listBox2.Items.Clear();

            String[] plugins = Directory.GetDirectories(Settings.Path + "Plugins");

            foreach (String str in plugins)
            {
                try
                {
                    XmlDocument xml = new XmlDocument();
                    xml.LoadXml(File.ReadAllText(str + "\\callisto.xml"));
                    XmlNode node1 = xml.DocumentElement;
                    String d = node1.SelectSingleNode("/plugin/description").InnerText;
                    String a = node1.SelectSingleNode("/plugin/author").InnerText;
                    String v = node1.SelectSingleNode("/plugin/version").InnerText;
                    bool g;

                    if (!bool.TryParse(node1.SelectSingleNode("/plugin/gui").InnerText, out g))
                        g = false;

                    String p = str;
                    String n = new DirectoryInfo(p).Name;
                    PluginListboxItem item = new PluginListboxItem(d, a, v, p, n, g);

                    if (this.available_list.Find(x => x.plugin_name == item.plugin_name) == null)
                    {
                        this.available_list.Add(item);
                        this.listBox2.Items.Add(item.plugin_name);
                    }
                }
                catch { }
            }
        }

        public void SetAutoLoads()
        {
            this.loaded_list.Clear();
            this.listBox1.Items.Clear();

            try
            {
                FileStream f = new FileStream(Settings.Path + "plugins.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("plugins");

                while (xml.ReadToFollowing("item"))
                {
                    TrustedServerItems a = new TrustedServerItems();

                    xml.ReadSubtree().ReadToFollowing("name");
                    String plugin_name = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    
                    PluginListboxItem i = this.available_list.Find(x => x.plugin_name == plugin_name);

                    if (i != null) // plugin exists in plugin folder
                    {
                        if (this.loaded_list.Find(x => x.plugin_name == i.plugin_name) == null) // plugin is not already loaded
                        {
                            if (ServerEvents.Plugins.LoadPlugin(i)) // plugin loaded ok
                            {
                                try
                                {
                                    if (i.gui)
                                        this.OnPluginLoaded(i.plugin_name);
                                }
                                catch { }

                                this.loaded_list.Add(i);
                                this.listBox1.Items.Add(i);
                            }
                        }
                    }

                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }
        }

        private void UpdateAutoLoadList()
        {
            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(Settings.Path + "plugins.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("plugins");

                foreach (PluginListboxItem a in this.loaded_list)
                {
                    xml.WriteStartElement("item");
                    xml.WriteElementString("name", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.plugin_name)));
                    xml.WriteEndElement();
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                Process.Start(Settings.Path + "Plugins");
            }
            catch { }
        }

        private void ShowToolTip(object sender, EventArgs e)
        {
            int i = ((ListBox)sender).IndexFromPoint(((ListBox)sender).PointToClient(ListBox.MousePosition));

            if (i > -1)
            {
                PluginListboxItem item = this.available_list.Find(x => x.plugin_name == (String)((ListBox)sender).Items[i]);

                if (item != null)
                {
                    String str = item.plugin_name + "\r\n";
                    str += item.description + "\r\n";
                    str += "by " + item.author + "\r\n";
                    str += item.version;
                    this.tip.SetToolTip(((ListBox)sender), str);
                }
            }
        }

        private void listBox2_MouseDoubleClick(object sender, MouseEventArgs e) // load or reload plugin
        {
            int i = ((ListBox)sender).IndexFromPoint(((ListBox)sender).PointToClient(ListBox.MousePosition));

            if (i > -1)
            {
                this.OnStallRequired();
                PluginListboxItem item = this.available_list.Find(x => x.plugin_name == (String)((ListBox)sender).Items[i]);

                if (item != null)
                {
                    if (this.loaded_list.Find(x => x.plugin_name == item.plugin_name) != null) // reload
                    {
                        ServerEvents.Plugins.KillPlugin(item);

                        try
                        {
                            if (item.gui)
                                this.OnPluginUnloaded(item.plugin_name);
                        }
                        catch { }

                        if (ServerEvents.Plugins.LoadPlugin(item))
                        {
                            try
                            {
                                if (item.gui)
                                    this.OnPluginLoaded(item.plugin_name);
                            }
                            catch { }
                        }
                        else MessageBox.Show("Failed in loading plugin!");
                    }
                    else // load
                    {
                        if (ServerEvents.Plugins.LoadPlugin(item))
                        {
                            try
                            {
                                if (item.gui)
                                    this.OnPluginLoaded(item.plugin_name);
                            }
                            catch { }

                            this.loaded_list.Add(item);
                            this.listBox1.Items.Add(item);
                            this.UpdateAutoLoadList();
                        }
                        else MessageBox.Show("Failed in loading plugin!");
                    }
                }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            this.OnStallRequired();
            this.SetValues();
        }

        private void unloadPluginToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.listBox1.SelectedIndex > -1)
            {
                PluginListboxItem item = this.loaded_list.Find(x => x.plugin_name == ((PluginListboxItem)this.listBox1.Items[this.listBox1.SelectedIndex]).plugin_name);

                if (item != null)
                {
                    this.OnStallRequired();
                    ServerEvents.Plugins.KillPlugin(item);
                    this.loaded_list.RemoveAll(x => x.plugin_name == item.plugin_name);
                    this.listBox1.Items.RemoveAt(this.listBox1.SelectedIndex);

                    try
                    {
                        if (item.gui)
                            this.OnPluginUnloaded(item.plugin_name);
                    }
                    catch { }

                    this.UpdateAutoLoadList();
                }
            }
        }
    }

    class PluginListboxItem
    {
        public String description;
        public String author;
        public String version;
        public String path;
        public String plugin_name;
        public bool gui;

        public PluginListboxItem(String d, String a, String v, String p, String n, bool g)
        {
            this.description = d;
            this.author = a;
            this.version = v;
            this.path = p;
            this.plugin_name = n;
            this.gui = g;
        }

        public override string ToString()
        {
            return this.plugin_name;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }
}
