﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Jurassic;
using Jurassic.Library;
using ICallisto;

namespace callisto.Scripting
{
    class CallistoScript
    {
        public String ScriptName { get; private set; }
        public ScriptEngine JS { get; private set; }
        public List<Objects.User> local_users = new List<Objects.User>();
        public List<Objects.User> linked_users = new List<Objects.User>();
        public List<ulong> timer_idents = new List<ulong>();

        public Statics.Global gbl;

        public CallistoScript(String name)
        {
            this.ScriptName = name;
            this.JS = new ScriptEngine();
            this.JS.ScriptName = name;

            // set up global functions
            this.gbl = new Statics.Global(this);
            this.JS.SetGlobalFunction("print", new Action<object, object>((a, b) => this.gbl.Print(a, b)));
            this.JS.Global.DefineProperty("print", new PropertyDescriptor(this.JS.Global["print"], PropertyAttributes.Sealed), throwOnError: true);
            this.JS.SetGlobalFunction("firstUser", new Func<Objects.User>(() => this.gbl.FirstUser()));
            this.JS.Global.DefineProperty("firstUser", new PropertyDescriptor(this.JS.Global["firstUser"], PropertyAttributes.Sealed), throwOnError: true);
            this.JS.SetGlobalFunction("firstLinked", new Func<Objects.User>(() => this.gbl.FirstLinked()));
            this.JS.Global.DefineProperty("firstLinked", new PropertyDescriptor(this.JS.Global["firstLinked"], PropertyAttributes.Sealed), throwOnError: true);
            this.JS.SetGlobalFunction("firstBanned", new Func<Objects.BannedUser>(() => this.gbl.FirstBanned()));
            this.JS.Global.DefineProperty("firstBanned", new PropertyDescriptor(this.JS.Global["firstBanned"], PropertyAttributes.Sealed), throwOnError: true);
            this.JS.SetGlobalFunction("user", new Func<object, object, Objects.User>((a, b) => this.gbl.User(a, b)));
            this.JS.Global.DefineProperty("user", new PropertyDescriptor(this.JS.Global["user"], PropertyAttributes.Sealed), throwOnError: true);
            this.JS.SetGlobalFunction("canScript", new Func<object, bool>(a => this.gbl.CanScript(a)));
            this.JS.Global.DefineProperty("canScript", new PropertyDescriptor(this.JS.Global["canScript"], PropertyAttributes.Sealed), throwOnError: true);
            this.JS.SetGlobalFunction("sendPM", new Action<object, object, object>((a, b, c) => this.gbl.SendPM(a, b, c)));
            this.JS.Global.DefineProperty("sendPM", new PropertyDescriptor(this.JS.Global["sendPM"], PropertyAttributes.Sealed), throwOnError: true);
            this.JS.SetGlobalFunction("clrName", new Func<object, String>(a => this.gbl.ClrName(a)));
            this.JS.Global.DefineProperty("clrName", new PropertyDescriptor(this.JS.Global["clrName"], PropertyAttributes.Sealed), throwOnError: true);
            this.JS.SetGlobalFunction("byteLength", new Func<object, int>(a => this.gbl.ByteLength(a)));
            this.JS.Global.DefineProperty("byteLength", new PropertyDescriptor(this.JS.Global["byteLength"], PropertyAttributes.Sealed), throwOnError: true);

            // set up static classes
            this.JS.Global.DefineProperty("Script", new PropertyDescriptor(new Statics.Script(this.JS), PropertyAttributes.Sealed), throwOnError: true);
            this.JS.Global.DefineProperty("File", new PropertyDescriptor(new Statics.File(this.JS), PropertyAttributes.Sealed), throwOnError: true);
            this.JS.Global.DefineProperty("Base64", new PropertyDescriptor(new Statics.Base64(this.JS), PropertyAttributes.Sealed), throwOnError: true);
            this.JS.Global.DefineProperty("Hashlink", new PropertyDescriptor(new Statics.Hashlink(this.JS), PropertyAttributes.Sealed), throwOnError: true);
            this.JS.Global.DefineProperty("Stats", new PropertyDescriptor(new Statics.Stats(this.JS), PropertyAttributes.Sealed), throwOnError: true);
            this.JS.Global.DefineProperty("Room", new PropertyDescriptor(new Statics.Room(this.JS), PropertyAttributes.Sealed), throwOnError: true);
            this.JS.Global.DefineProperty("Linker", new PropertyDescriptor(new Statics.Linker(this.JS), PropertyAttributes.Sealed), throwOnError: true);
            this.JS.Global.DefineProperty("Registry", new PropertyDescriptor(new Statics.Reg(this.JS), PropertyAttributes.Sealed), throwOnError: true);
            this.JS.Global.DefineProperty("Autorun", new PropertyDescriptor(new Statics.Autorun(this.JS), PropertyAttributes.Sealed), throwOnError: true);
            this.JS.Global.DefineProperty("Users", new PropertyDescriptor(new Statics.Users(this.JS), PropertyAttributes.Sealed), throwOnError: true);

            // set up instance classes
            this.JS.SetGlobalValue("Scribble", new Objects.Scribble(this.JS));
            this.JS.SetGlobalValue("HttpRequest", new Objects.HttpRequest(this.JS));
            this.JS.SetGlobalValue("Avatar", new Objects.Avatar(this.JS));
            this.JS.SetGlobalValue("Timer", new Objects.Timer(this.JS));
            this.JS.SetGlobalValue("XmlParser", new Objects.XmlParser(this.JS));
            this.JS.SetGlobalValue("Sql", new Objects.Sql(this.JS));
            this.JS.SetGlobalValue("Query", new Objects.Query(this.JS));

            // set up extensions
            foreach (System.Reflection.Assembly asm in ScriptManager.Extensions)
            {
                foreach (Type type in asm.GetTypes())
                {
                    if (type.Namespace == "Extensions.Objects")
                    {
                        if (type.BaseType.Equals(typeof(ClrFunction)))
                        {
                            String _name = type.Name;

                            if (this.JS.GetGlobalValue(_name) is Undefined)
                            {
                                object _obj = Activator.CreateInstance(type, this.JS);
                                this.JS.SetGlobalValue(_name, _obj);
                            }
                        }
                    }
                    else if (type.Namespace == "Extensions.Statics")
                    {
                        if (type.BaseType.Equals(typeof(ObjectInstance)))
                        {
                            String _name = type.Name;

                            if (this.JS.GetGlobalValue(_name) is Undefined)
                            {
                                object _obj = Activator.CreateInstance(type, this.JS);
                                this.JS.Global.DefineProperty(_name, new PropertyDescriptor(_obj, PropertyAttributes.Sealed), throwOnError: true);
                            }
                        }
                    }
                }
            }            

            // set up default events
            StringBuilder events = new StringBuilder();
            events.AppendLine("function onTextReceived(userobj, text) { }");
            events.AppendLine("function onTextBefore(userobj, text) { return text; }");
            events.AppendLine("function onTextAfter(userobj, text) { }");
            events.AppendLine("function onEmoteReceived(userobj, text) { }");
            events.AppendLine("function onEmoteBefore(userobj, text) { return text; }");
            events.AppendLine("function onEmoteAfter(userobj, text) { }");
            events.AppendLine("function onError(script, line, message) { }");
            events.AppendLine("function onJoinCheck(userobj) { return true; }");
            events.AppendLine("function onJoin(userobj) { }");
            events.AppendLine("function onPart(userobj) { }");
            events.AppendLine("function onTimer() { }");
            events.AppendLine("function onHelp(userobj) { }");
            events.AppendLine("function onCommand(userobj, command, target, args) { }");
            events.AppendLine("function onAvatar(userobj) { return true; }");
            events.AppendLine("function onPersonalMessage(userobj, msg) { return true; }");
            events.AppendLine("function onRejected(userobj) { }");
            events.AppendLine("function onLoad() { }");
            events.AppendLine("function onUnload() { }");
            events.AppendLine("function onVroomJoinCheck(userobj, vroom) { return true; }");
            events.AppendLine("function onVroomJoin(userobj) { }");
            events.AppendLine("function onVroomPart(userobj) { }");
            events.AppendLine("function onLoginAttempted(userobj, success) { }");
            events.AppendLine("function onFileReceived(userobj, filename) { }");
            events.AppendLine("function onFlood(userobj) { return true; }");
            events.AppendLine("function onBotPM(userobj, text) { return true; }");
            events.AppendLine("function onPM(userobj, target, pm) { return true; }");
            events.AppendLine("function onLinkConnected(link) { }");
            events.AppendLine("function onLinkDisconnected(link) { }");
            events.AppendLine("function onLinkError(link, code) { }");
            events.AppendLine("function onLinkCommand(userobj, command, target, args) { }");
            events.AppendLine("function onDnsReceived(userobj) { }");

            this.JS.Evaluate(events.ToString());
        }

        public void KillScript()
        {
            this.local_users.Clear();
            this.linked_users.Clear();
            this.JS = null;

            lock (SyncList.List)
                SyncList.List.RemoveAll(x => x.ScriptName == this.ScriptName);

            this.timer_idents.Clear();
            TimerList.RemoveScriptTimers(this.ScriptName);
        }

        public bool LoadScript(String path)
        {
            try
            {
                this.JS.ExecuteFile(path);
                return true;
            }
            catch (JavaScriptException e)
            {
                ScriptManager.OnError(this.ScriptName, e.Message, e.LineNumber);
            }
            catch { }

            return false;
        }

        public void AddUser(IUser userobj)
        {
            if (userobj.IsLinked)
            {
                this.linked_users.RemoveAll(x => x.Name == userobj.Name);
                this.linked_users.Add(new Objects.User(this.JS.Object.InstancePrototype, userobj, this.ScriptName));
            }
            else
            {
                this.local_users.RemoveAll(x => x.Id == userobj.Id || x.Name == userobj.Name);
                this.local_users.Add(new Objects.User(this.JS.Object.InstancePrototype, userobj, this.ScriptName));
            }
        }

        public void RemoveUser(IUser userobj)
        {
            if (userobj.IsLinked)
                this.linked_users.RemoveAll(x => x.Name == userobj.Name);
            else
                this.local_users.RemoveAll(x => x.Name == userobj.Name);
        }

        public Objects.User GetUser(IUser userobj)
        {
            return this.GetUser(userobj, false);
        }

        public Objects.User GetUser(IUser userobj, bool linked)
        {
            if (userobj == null)
                return null;

            if (linked)
                return this.linked_users.Find(x => x.Name == userobj.Name);

            if (userobj.IsLinked)
                return this.linked_users.Find(x => x.Name == userobj.Name);

            return this.local_users.Find(x => x.Name == userobj.Name);
        }


    }
}
