﻿using System;
using System.Collections.Generic;
using System.Collections.Concurrent;
using System.Linq;
using System.Text;
using System.Net;

namespace callisto.Scripting
{
    class DNSManager
    {
        private static ConcurrentBag<DNSItem> items = new ConcurrentBag<DNSItem>();
        private static BlockingCollection<DNSItem> pending = new BlockingCollection<DNSItem>();

        public static void AddDns(IPAddress ip)
        {
            if (items.FirstOrDefault(x => x.IP.Equals(ip)) == null)
                Dns.BeginGetHostEntry(ip, new AsyncCallback(ResultReceived), ip);
        }

        public static void DequeuePending()
        {
            while (pending.Count > 0)
            {
                DNSItem item = pending.Take();

                if (item != null)
                    ServerEvents.OnDnsReceived(item);
            }
        }

        private static void ResultReceived(IAsyncResult r)
        {
            try
            {
                IPHostEntry i = Dns.EndGetHostEntry(r);
                String host = i.HostName;
                IPAddress ip = (IPAddress)r.AsyncState;
                DNSItem item = new DNSItem { Host = host, IP = ip };

                if (items.FirstOrDefault(x => x.IP.Equals(ip)) == null)
                {
                    items.Add(item);
                    pending.Add(item);
                }
            }
            catch
            {
                IPAddress ip = (IPAddress)r.AsyncState;
                String host = ip.ToString();
                DNSItem item = new DNSItem { Host = host, IP = ip };

                if (items.FirstOrDefault(x => x.IP.Equals(ip)) == null)
                {
                    items.Add(item);
                    pending.Add(item);
                }
            }
        }

        public static String GetDns(IPAddress ip)
        {
            DNSItem d = items.FirstOrDefault(x => x.IP.Equals(ip));

            if (d != null)
                return d.Host;

            return ip.ToString();
        }

        public static bool DnsResolved(IPAddress ip)
        {
            return items.FirstOrDefault(x => x.IP.Equals(ip)) != null;
        }

        private static String[] bad = new String[]
        {
            "webfusion",
            "cyberghost",
            "redstation",
            "center",
            "centre",
            "softlayer",
            "server",
            "choopa",
            "lstn.net",
            "kryptotel",
            "system1",
            "vpn",
            "pptp"
        };

        public static bool BadDns(String dns)
        {
            String d = dns.ToLower();

            foreach (String s in bad)
                if (d.Contains(s))
                    return true;

            return false;
        }
    }

    class DNSItem
    {
        public IPAddress IP { get; set; }
        public String Host { get; set; }
    }
}
