﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography;
using Jurassic;
using Jurassic.Library;
using ICallisto;

namespace callisto.Scripting.Objects
{
    class BannedUser : ObjectInstance
    {
        private String ScriptName { get; set; }
        private String guid { get; set; }

        public BannedUser(ObjectInstance prototype, IBan user, String script)
            : base(prototype)
        {
            this.PopulateFunctions();
            this.parent = user;
            this.ScriptName = script;
            this.guid = null;
            this.DefineProperty("nextUser", new PropertyDescriptor(this["nextUser"], PropertyAttributes.Sealed), throwOnError: true);
        }

        internal IBan parent;

        [JSProperty(Name = "banned")]
        public bool Banned
        {
            get { return this.parent.Banned; }
            set { this.parent.Banned = value; }
        }

        [JSProperty(Name = "externalIp")]
        public String ExternalIP
        {
            get { return this.parent.ExternalIP.ToString(); }
            set { }
        }

        [JSProperty(Name = "localIp")]
        public String LocalIP
        {
            get { return this.parent.LocalIP.ToString(); }
            set { }
        }

        [JSProperty(Name = "name")]
        public String Name
        {
            get { return this.parent.Name; }
            set { }
        }

        [JSProperty(Name = "version")]
        public String Version
        {
            get { return this.parent.Version; }
            set { }
        }

        [JSProperty(Name = "port")]
        public int Port
        {
            get { return this.parent.Port; }
            set { }
        }

        [JSProperty(Name = "guid")]
        public String Guid
        {
            get
            {
                if (this.guid == null)
                {
                    byte[] buf = Convert.FromBase64String(this.parent.Guid);
                    buf = SHA1.Create().ComputeHash(buf);
                    this.guid = Convert.ToBase64String(buf);
                }

                return this.guid;
            }
            set { }
        }

        [JSFunction(Name = "nextUser")]
        public BannedUser NextUser()
        {
            IBan b = BanManager.GetNextBanned(this.parent);

            if (b != null)
                return new BannedUser(this.Engine.Object.InstancePrototype, b, this.ScriptName);

            return null;
        }

        public override string ToString()
        {
            return "[object BannedUser]";
        }
    }
}
