﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Jurassic;
using Jurassic.Library;

namespace callisto.Scripting.Objects
{
    class Timer : ClrFunction
    {
        public Timer(ScriptEngine engine)
            : base(engine.Function.InstancePrototype, "Timer", new TimerInstance(engine.Object.InstancePrototype))
        {

        }

        [JSConstructorFunction]
        public TimerInstance Construct()
        {
            return new TimerInstance(this.InstancePrototype);
        } 
    }

    public class TimerInstance : ObjectInstance
    {
        public TimerInstance(ObjectInstance prototype)
            : base(prototype)
        {
            this.PopulateFunctions();
            this.ScriptName = this.Engine.ScriptName;
            this.StartTime = 0;
            this.DefineProperty("stop", new PropertyDescriptor(this["stop"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("start", new PropertyDescriptor(this["start"], PropertyAttributes.Sealed), throwOnError: true);

            CallistoScript script = ScriptManager.Scripts.Find(x => x.ScriptName == this.ScriptName);

            if (script != null)
            {
                this.IDENT = TimerList.NextIdent++;
                script.timer_idents.Add(this.IDENT);
            }
        }

        public String ScriptName { get; set; }
        public long StartTime { get; set; }

        public ulong IDENT;
        public bool m_enabled = false;
        private int m_interval = 500;

        [JSFunction(Name = "start")]
        public bool Start()
        {
            return TimerList.AddTimer(this);
        }

        [JSFunction(Name = "stop")]
        public bool Stop()
        {
            return TimerList.RemoveTimer(this);
        }

        [JSProperty(Name = "interval")]
        public int Interval
        {
            get { return this.m_interval; }
            set
            {
                if (value >= 500)
                    this.m_interval = value;
            }
        }

        [JSProperty(Name = "oncomplete")]
        public UserDefinedFunction Callback { get; set; }
    }
}
