﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography;
using Jurassic;
using Jurassic.Library;
using ICallisto;

namespace callisto.Scripting.Objects
{
    class User : ObjectInstance
    {
        private String ScriptName { get; set; }
        private String guid { get; set; }

        public User(ObjectInstance prototype, IUser user, String script)
            : base(prototype)
        {
            this.PopulateFunctions();
            this.parent = user;
            this.ScriptName = script;
            this.guid = null;

            this.DefineProperty("nextUser", new PropertyDescriptor(this["nextUser"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("ban", new PropertyDescriptor(this["ban"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("disconnect", new PropertyDescriptor(this["disconnect"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("nudge", new PropertyDescriptor(this["nudge"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("redirect", new PropertyDescriptor(this["redirect"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("restoreAvatar", new PropertyDescriptor(this["restoreAvatar"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("sendEmote", new PropertyDescriptor(this["sendEmote"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("sendText", new PropertyDescriptor(this["sendText"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("setTopic", new PropertyDescriptor(this["setTopic"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("setUrl", new PropertyDescriptor(this["setUrl"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("scribble", new PropertyDescriptor(this["scribble"], PropertyAttributes.Sealed), throwOnError: true);
        }

        internal IUser parent;

        [JSProperty(Name = "age")]
        public int Age
        {
            get { return this.parent.Age; }
            set
            {
                if (value < 0 || value > 99)
                    return;

                this.parent.Age = (byte)value;
            }
        }

        [JSProperty(Name = "avatar")]
        public AvatarImage Avatar
        {
            get { return new AvatarImage(this.Engine.Object.InstancePrototype) { Data = this.parent.Avatar }; }
            set
            {
                if (value is AvatarImage)
                    this.parent.Avatar = value.Data;
                else
                    this.parent.Avatar = null;
            }
        }

        [JSProperty(Name = "browsable")]
        public bool Browsable
        {
            get { return this.parent.Browsable; }
            set { }
        }

        [JSProperty(Name = "cloaked")]
        public bool Cloaked
        {
            get { return this.parent.Cloaked; }
            set { this.parent.Cloaked = value; }
        }

        [JSProperty(Name = "country")]
        public String Country
        {
            get { return Helpers.CountryCodeToString(this.parent.Country); }
            set { }
        }

        [JSProperty(Name = "customName")]
        public String CustomName
        {
            get { return this.parent.CustomName; }
            set { this.parent.CustomName = value; }
        }

        [JSProperty(Name = "externalIp")]
        public String ExternalIp
        {
            get { return this.parent.ExternalIP.ToString(); }
            set { }
        }

        [JSProperty(Name = "dns")]
        public String DNS
        {
            get { return DNSManager.GetDns(this.parent.ExternalIP); }
            set { }
        }

        [JSProperty(Name = "dnsReceived")]
        public bool DNSReceived
        {
            get { return DNSManager.DnsResolved(this.parent.ExternalIP); }
            set { }
        }

        [JSProperty(Name = "fastPing")]
        public bool FastPing
        {
            get { return this.parent.FastPing; }
            set { }
        }

        [JSProperty(Name = "fileCount")]
        public int FileCount
        {
            get { return this.parent.FileCount; }
            set { }
        }

        [JSProperty(Name = "gender")]
        public String Gender
        {
            get
            {
                switch (this.parent.Sex)
                {
                    case 1:
                        return "Male";

                    case 2:
                        return "Female";

                    default:
                        return "Unknown";
                }
            }
            set
            {
                switch (value.ToUpper())
                {
                    case "MALE":
                        this.parent.Sex = 1;
                        break;

                    case "FEMALE":
                        this.parent.Sex = 2;
                        break;

                    default:
                        this.parent.Sex = 0;
                        break;
                }
            }
        }

        [JSProperty(Name = "guid")]
        public String Guid
        {
            get
            {
                if (this.guid == null)
                {
                    byte[] buf = Convert.FromBase64String(this.parent.Guid);
                    buf = SHA1.Create().ComputeHash(buf);
                    this.guid = Convert.ToBase64String(buf);
                }

                return this.guid;
            }
            set { }
        }

        [JSProperty(Name = "id")]
        public int Id
        {
            get { return this.parent.Id; }
            set { }
        }

        [JSProperty(Name = "ignores")]
        public IgnoreCollection Ignores
        {
            get { return new IgnoreCollection(this.Engine.Object.InstancePrototype, this.parent.IgnoreList, this.ScriptName); }
            set { }
        }

        [JSProperty(Name = "level")]
        public int Level
        {
            get { return (int)((byte)this.parent.Level); }
            set
            {
                if (value < 0 || value > 3)
                    return;

                this.parent.Level = (AdminLevel)((byte)value);
            }
        }

        [JSProperty(Name = "linked")]
        public bool Linked
        {
            get { return this.parent.IsLinked; }
            set { }
        }

        [JSProperty(Name = "localHost")]
        public bool LocalHost
        {
            get { return this.parent.IsLocalHost; }
            set { }
        }

        [JSProperty(Name = "localIp")]
        public String LocalIP
        {
            get { return this.parent.LocalIP.ToString(); }
            set { }
        }

        [JSProperty(Name = "muzzled")]
        public bool Muzzled
        {
            get { return this.parent.Muzzled; }
            set { this.parent.Muzzled = value; }
        }

        [JSProperty(Name = "name")]
        public String Name
        {
            get { return this.parent.Name; }
            set { this.parent.Name = value; }
        }

        [JSProperty(Name = "orgName")]
        public String OrgName
        {
            get { return this.parent.OrgName; }
            set { }
        }

        [JSProperty(Name = "personalMessage")]
        public String PersonalMessage
        {
            get { return this.parent.PersonalMessage; }
            set { this.parent.PersonalMessage = value; }
        }

        [JSProperty(Name = "port")]
        public int Port
        {
            get { return this.parent.Port; }
            set { }
        }

        [JSProperty(Name = "region")]
        public String Region
        {
            get { return this.parent.Region; }
            set { }
        }

        [JSProperty(Name = "version")]
        public String Version
        {
            get { return this.parent.Version; }
            set { }
        }

        [JSProperty(Name = "vroom")]
        public int Vroom
        {
            get { return this.parent.Vroom; }
            set
            {
                if (value < 0 || value > 65535)
                    return;

                this.parent.Vroom = (ushort)value;
            }
        }

        [JSFunction(Name = "nextUser")]
        public User NextUser()
        {
            CallistoScript script = ScriptManager.Scripts.Find(x => x.ScriptName == this.ScriptName);

            if (script != null)
            {
                if (this.parent.IsLinked)
                {
                    int index = script.linked_users.FindIndex(x => x.Name == this.Name);

                    if (index > -1)
                        if ((index + 1) < script.linked_users.Count)
                            return script.linked_users[index + 1];
                }
                else
                {
                    var linq = from x in script.local_users
                               where x.Id > this.Id
                               orderby x.Id
                               select x;

                    if (linq.Count() > 0)
                        return linq.ElementAt(0);
                }
            }

            return null;
        }

        [JSFunction(Name = "ban")]
        public void Ban()
        {
            this.parent.Ban();
        }

        [JSFunction(Name = "disconnect")]
        public void Disconnect()
        {
            this.parent.Disconnect();
        }

        [JSFunction(Name = "nudge")]
        public void Nudge()
        {
            if (this.Version.Contains("cb0t"))
            {
                byte[] temp = Encoding.UTF8.GetBytes("0Callisto");
                temp = HashlinkHelpers.e67(temp, 1488);
                temp = Encoding.Default.GetBytes(Convert.ToBase64String(temp));
                this.parent.SendCustomClientData("cb0t_nudge", "Callisto", temp);
            }
        }

        [JSFunction(Name = "redirect")]
        public void Redirect(object a)
        {
            if (a is String || a is ConcatenatedString)
                this.parent.Redirect(a.ToString());
        }

        [JSFunction(Name = "restoreAvatar")]
        public void RestoreAvatar()
        {
            this.parent.RestoreAvatar();
        }

        [JSFunction(Name = "sendEmote")]
        public void SendEmote(object a)
        {
            if (!(a is Undefined) && a != null)
                this.parent.SendEmote(a.ToString());
        }

        [JSFunction(Name = "sendText")]
        public void SendText(object a)
        {
            if (!(a is Undefined) && a != null)
                this.parent.SendText(a.ToString());
        }

        [JSFunction(Name = "setTopic")]
        public void SetTopic(object a)
        {
            if (!(a is Undefined))
                this.parent.SetTopic(a.ToString());
            else if (a != null)
                this.parent.SetTopic(Settings.ServerTopic);
        }

        [JSFunction(Name = "setUrl")]
        public void SetUrl(object a, object b)
        {
            if (a is Undefined && b is Undefined)
                this.parent.UpdateURL(String.Empty, String.Empty);
            else if (!(a is Undefined) && !(b is Undefined))
                if (a != null && b != null)
                    this.parent.UpdateURL(a.ToString(), b.ToString());
        }

        [JSFunction(Name = "scribble")]
        public void Scribble(object a)
        {
            if (!this.Version.Contains("cb0t"))
                return;

            if (a is ScribbleResult)
            {
                ScribbleResult scr = (ScribbleResult)a;
                scr.SendScribble(this.parent);
            }
        }

        public override string ToString()
        {
            return "[object User]";
        }
    }
}
