﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Reflection;
using ICallisto;
using Microsoft.Win32;

namespace callisto.Scripting
{
    class ScriptManager
    {
        public static List<CallistoScript> Scripts;
        public static List<Assembly> Extensions;
        public static int PrintCounter { get; set; }

        static ScriptManager()
        {
            PrintCounter = 0;
        }

        public static void Autorun()
        {
            Scripts = new List<CallistoScript>();
            Extensions = new List<Assembly>();

            String path = AppDomain.CurrentDomain.BaseDirectory + "Extensions\\";

            if (Directory.Exists(path))
            {
                foreach (FileInfo f in new DirectoryInfo(path).GetFiles())
                {
                    if (f.Extension == ".dll")
                    {
                        try
                        {
                            byte[] buf = File.ReadAllBytes(f.FullName);
                            Assembly asm = Assembly.Load(buf);
                            Extensions.Add(asm);
                        }
                        catch { }
                    }
                }
            }

            try // update the extensions folder reference for external installer use
            {
                RegistryKey key = Registry.CurrentUser.OpenSubKey("Software\\callisto", true);

                if (key == null)
                {
                    key = Registry.CurrentUser.CreateSubKey("Software\\callisto");
                    key.Close();
                    key = Registry.CurrentUser.OpenSubKey("Software\\callisto", true);
                }

                key.SetValue("ext_path", path);
                key.Close();
            }
            catch { }

            path = Settings.Path + "scriptloader";

            try
            {
                if (File.Exists(path))
                {
                    String[] files = File.ReadAllText(path).Split(new String[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);

                    if (files.Length > 0)
                    {
                        Statics.Autorun.AddScripts(files);

                        foreach (String str in files)
                            Load(new FileInfo(Settings.Path + "Scripts\\" + str + "\\" + str));
                    }
                }
            }
            catch { }
        }

        public static void ListScripts(IUser userobj)
        {
            foreach (CallistoScript s in Scripts)
                userobj.SendPrint(s.ScriptName);
        }

        public static bool Load(FileInfo file)
        {
            int index = Scripts.FindIndex(x => x.ScriptName == file.Name);

            if (index > -1)
            {
                Scripts[index].KillScript();
                Scripts.RemoveAt(index);
            }

            CallistoScript script = new CallistoScript(file.Name);

            if (File.Exists(file.FullName))
            {
                foreach (UserObject u in UserPool.Pool)
                    if (u.Connected)
                        if (u.logged_in)
                            script.AddUser(u);

                foreach (WebUser u in WebPool.Pool)
                    if (u.InUse)
                        if (u.LoggedIn)
                            script.AddUser(u);

                if (CallistoServer.Linker.IsLinking)
                    foreach (LinkedUserObject u in CallistoServer.Linker.linked_users)
                        script.AddUser(u);

                Scripts.Add(script);

                if (script.LoadScript(file.FullName))
                {
                    try
                    {
                        script.JS.CallGlobalFunction("onLoad");
                    }
                    catch (Jurassic.JavaScriptException e)
                    {
                        OnError(script.ScriptName, e.Message, e.LineNumber);
                    }
                    catch { }

                    return true;
                }
            }

            return false;
        }

        public static bool Unload(String name)
        {
            int index = Scripts.FindIndex(x => x.ScriptName == name);

            if (index > -1)
            {
                try
                {
                    Scripts[index].JS.CallGlobalFunction("onUnload");
                }
                catch (Jurassic.JavaScriptException e)
                {
                    OnError(Scripts[index].ScriptName, e.Message, e.LineNumber);
                }
                catch { }

                Scripts[index].KillScript();
                Scripts.RemoveAt(index);

                return true;
            }

            return false;
        }

        public static void Destroy()
        {
            if (Scripts != null)
            {
                Scripts.ForEach(x => x.KillScript());
                Scripts.Clear();
            }
        }

        public static void AddUser(IUser userobj)
        {
            if (!userobj.IsLinked)
                DNSManager.AddDns(userobj.ExternalIP);

            Scripts.ForEach(x => x.AddUser(userobj));
        }

        public static void RemoveUser(IUser userobj)
        {
            Scripts.ForEach(x => x.RemoveUser(userobj));
        }

        public static void OnError(String script, String msg, int line)
        {
            Scripts.ForEach(x =>
            {
                try { x.JS.CallGlobalFunction("onError", script, line, msg); }
                catch { }
            });
        }

    }
}
