﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using ICallisto;
using Jurassic;

namespace callisto.Scripting
{
    class ServerEvents
    {
        public static void OneSecondTick()
        {
            if (Settings.js_enabled)
            {
                lock (SyncList.List)
                {
                    foreach (ISyncListItem obj in SyncList.List)
                    {
                        if (ScriptManager.Scripts.Find(x => x.ScriptName == obj.ScriptName) != null)
                        {
                            if (obj is Objects.ScribbleResult)
                            {
                                Objects.ScribbleResult scribble = (Objects.ScribbleResult)obj;

                                try
                                {
                                    ScriptManager.PrintCounter = 0;

                                    if (scribble.Data == null)
                                        scribble.Callback.Call(scribble, false);
                                    else
                                        scribble.Callback.Call(scribble, true);
                                }
                                catch (JavaScriptException e)
                                {
                                    ScriptManager.OnError(scribble.ScriptName, e.Message, e.LineNumber);
                                }
                                catch { }
                            }
                            else if (obj is Objects.HttpRequestResult)
                            {
                                Objects.HttpRequestResult http = (Objects.HttpRequestResult)obj;

                                try
                                {
                                    ScriptManager.PrintCounter = 0;

                                    if (String.IsNullOrEmpty(http.Data))
                                        http.Callback.Call(http, false);
                                    else
                                        http.Callback.Call(http, true);
                                }
                                catch (JavaScriptException e)
                                {
                                    ScriptManager.OnError(http.ScriptName, e.Message, e.LineNumber);
                                }
                                catch { }
                            }
                            else if (obj is Objects.AvatarImage)
                            {
                                Objects.AvatarImage av = (Objects.AvatarImage)obj;

                                try
                                {
                                    ScriptManager.PrintCounter = 0;

                                    if (av.Data == null)
                                        av.Callback.Call(av, false);
                                    else
                                        av.Callback.Call(av, true);
                                }
                                catch (JavaScriptException e)
                                {
                                    ScriptManager.OnError(av.ScriptName, e.Message, e.LineNumber);
                                }
                                catch { }
                            }
                        }
                    }

                    SyncList.List.Clear();
                }

                CallistoScript[] s = ScriptManager.Scripts.ToArray();

                foreach (CallistoScript script in s)
                {
                    ScriptManager.PrintCounter = 0;

                    try
                    {
                        script.JS.CallGlobalFunction("onTimer");
                    }
                    catch (JavaScriptException e)
                    {
                        ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                    }
                    catch { }
                }
            }
        }

        public static bool OnJoinCheck(IUser userobj)
        {
            ScriptManager.AddUser(userobj);

            if (DNSManager.DnsResolved(userobj.ExternalIP))
            {
                String dns = DNSManager.GetDns(userobj.ExternalIP);

                if (DNSManager.BadDns(dns))
                {
                    BanManager.AddBan(userobj);
                    return false;
                }
            }

            if (Settings.js_enabled)
            {
                CallistoScript[] s = ScriptManager.Scripts.ToArray();

                foreach (CallistoScript script in s)
                {
                    ScriptManager.PrintCounter = 0;
                    Objects.User u = script.GetUser(userobj);

                    if (u != null)
                    {
                        try
                        {
                            bool result = script.JS.CallGlobalFunction<bool>("onJoinCheck", u);

                            if (!result)
                                return false;
                        }
                        catch (JavaScriptException e)
                        {
                            ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                        }
                        catch { }
                    }
                }
            }

            return true;
        }

        public static void OnJoin(IUser userobj)
        {
            if (userobj.IsLinked)
                ScriptManager.AddUser(userobj);

            if (Settings.js_enabled)
            {
                CallistoScript[] s = ScriptManager.Scripts.ToArray();

                foreach (CallistoScript script in s)
                {
                    ScriptManager.PrintCounter = 0;
                    Objects.User u = script.GetUser(userobj);

                    if (u != null)
                    {
                        try
                        {
                            script.JS.CallGlobalFunction("onJoin", u);
                        }
                        catch (JavaScriptException e)
                        {
                            ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                        }
                        catch { }
                    }
                }
            }
        }

        public static void OnDnsReceived(DNSItem item)
        {
            CallistoScript[] s = ScriptManager.Scripts.ToArray();
            String ip = item.IP.ToString();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;

                for (int i = (script.local_users.Count - 1); i > -1; i--)
                {
                    if (script.local_users[i].ExternalIp == ip)
                    {
                        if (DNSManager.BadDns(script.local_users[i].DNS))
                        {
                            try
                            {
                                BanManager.AddBan(script.local_users[i].parent);
                                script.local_users[i].Disconnect();
                            }
                            catch { }

                            return;
                        }

                        try
                        {
                            script.JS.CallGlobalFunction("onDnsReceived", script.local_users[i]);
                        }
                        catch (JavaScriptException e)
                        {
                            ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                        }
                        catch { }
                    }
                }
            }
        }

        public static void OnPart(IUser userobj)
        {
            if (Settings.js_enabled)
            {
                CallistoScript[] s = ScriptManager.Scripts.ToArray();

                foreach (CallistoScript script in s)
                {
                    ScriptManager.PrintCounter = 0;
                    Objects.User u = script.GetUser(userobj);

                    if (u != null)
                    {
                        try
                        {
                            script.JS.CallGlobalFunction("onPart", u);
                        }
                        catch (JavaScriptException e)
                        {
                            ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                        }
                        catch { }
                    }
                }
            }

            ScriptManager.RemoveUser(userobj);
        }

        public static void OnTextReceived(IUser userobj, String text)
        {
            if (!Settings.js_enabled)
                return;

            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;
                Objects.User u = script.GetUser(userobj);

                if (u != null)
                {
                    try
                    {
                        script.JS.CallGlobalFunction("onTextReceived", u, text);
                    }
                    catch (JavaScriptException e)
                    {
                        ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                    }
                    catch { }
                }
            }
        }

        public static String OnTextBefore(IUser userobj, String text)
        {
            if (!Settings.js_enabled)
                return text;

            String result = text;
            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;
                Objects.User u = script.GetUser(userobj);

                if (u != null)
                {
                    try
                    {
                        result = script.JS.CallGlobalFunction<String>("onTextBefore", u, result);

                        if (String.IsNullOrEmpty(result))
                            return String.Empty;
                    }
                    catch (JavaScriptException e)
                    {
                        ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                    }
                    catch { }
                }
            }

            return result;
        }

        public static void OnTextAfter(IUser userobj, String text)
        {
            if (!Settings.js_enabled)
                return;

            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;
                Objects.User u = script.GetUser(userobj);

                if (u != null)
                {
                    try
                    {
                        script.JS.CallGlobalFunction("onTextAfter", u, text);
                    }
                    catch (JavaScriptException e)
                    {
                        ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                    }
                    catch { }
                }
            }
        }

        public static void OnEmoteReceived(IUser userobj, String text)
        {
            if (!Settings.js_enabled)
                return;

            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;
                Objects.User u = script.GetUser(userobj);

                if (u != null)
                {
                    try
                    {
                        script.JS.CallGlobalFunction("onEmoteReceived", u, text);
                    }
                    catch (JavaScriptException e)
                    {
                        ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                    }
                    catch { }
                }
            }
        }

        public static String OnEmoteBefore(IUser userobj, String text)
        {
            if (!Settings.js_enabled)
                return text;

            String result = text;
            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;
                Objects.User u = script.GetUser(userobj);

                if (u != null)
                {
                    try
                    {
                        result = script.JS.CallGlobalFunction<String>("onEmoteBefore", u, result);

                        if (String.IsNullOrEmpty(result))
                            return String.Empty;
                    }
                    catch (JavaScriptException e)
                    {
                        ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                    }
                    catch { }
                }
            }

            return result;
        }

        public static void OnEmoteAfter(IUser userobj, String text)
        {
            if (!Settings.js_enabled)
                return;

            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;
                Objects.User u = script.GetUser(userobj);

                if (u != null)
                {
                    try
                    {
                        script.JS.CallGlobalFunction("onEmoteAfter", u, text);
                    }
                    catch (JavaScriptException e)
                    {
                        ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                    }
                    catch { }
                }
            }
        }

        private static String[] bad_chars = new String[]
        {
            "..",
            "/",
            "\\",
            " ",
        };

        public static void OnCommand(IUser userobj, String command_text, IUser target, String args)
        {
            if (!Settings.js_enabled)
                return;

            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;
                Objects.User u = script.GetUser(userobj);
                Objects.User t = script.GetUser(target);

                if (u != null)
                {
                    try
                    {
                        script.JS.CallGlobalFunction("onCommand", u, command_text, t, args);
                    }
                    catch (JavaScriptException e)
                    {
                        ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                    }
                    catch { }
                }
            }

            if (CanScript(userobj))
            {
                if (command_text.StartsWith("loadscript "))
                {
                    String filename = command_text.Substring(11);

                    if (filename.Length > 3 && filename.EndsWith(".js"))
                        if (bad_chars.Count<String>(x => filename.Contains(x)) == 0)
                            ScriptManager.Load(new FileInfo(Settings.Path + "Scripts\\" + filename + "\\" + filename));
                }
                else if (command_text.StartsWith("killscript "))
                    ScriptManager.Unload(command_text.Substring(11));
                else if (command_text == "listscripts")
                    ScriptManager.ListScripts(userobj);
            }
        }

        public static void OnHelp(IUser userobj)
        {
            if (!Settings.js_enabled)
                return;

            if (CanScript(userobj))
                if (userobj is UserObject)
                {
                    ((UserObject)userobj).SendPacket(ChatPacketHelpers.NoSuch("#loadscript <filename>"));
                    ((UserObject)userobj).SendPacket(ChatPacketHelpers.NoSuch("#killscript <filename>"));
                    ((UserObject)userobj).SendPacket(ChatPacketHelpers.NoSuch("#listscripts"));
                }

            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;
                Objects.User u = script.GetUser(userobj);

                if (u != null)
                {
                    try
                    {
                        script.JS.CallGlobalFunction("onHelp", u);
                    }
                    catch (JavaScriptException e)
                    {
                        ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                    }
                    catch { }
                }
            }
        }

        public static bool OnAvatarReceived(IUser userobj)
        {
            if (Settings.js_enabled)
            {
                CallistoScript[] s = ScriptManager.Scripts.ToArray();

                foreach (CallistoScript script in s)
                {
                    ScriptManager.PrintCounter = 0;
                    Objects.User u = script.GetUser(userobj);

                    if (u != null)
                    {
                        try
                        {
                            bool result = script.JS.CallGlobalFunction<bool>("onAvatar", u);

                            if (!result)
                                return false;
                        }
                        catch (JavaScriptException e)
                        {
                            ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                        }
                        catch { }
                    }
                }
            }

            return true;
        }

        public static bool OnPersonalReceived(IUser userobj, String _msg)
        {
            if (Settings.js_enabled)
            {
                CallistoScript[] s = ScriptManager.Scripts.ToArray();

                foreach (CallistoScript script in s)
                {
                    ScriptManager.PrintCounter = 0;
                    Objects.User u = script.GetUser(userobj);

                    if (u != null)
                    {
                        try
                        {
                            bool result = script.JS.CallGlobalFunction<bool>("onPersonalMessage", u, _msg);

                            if (!result)
                                return false;
                        }
                        catch (JavaScriptException e)
                        {
                            ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                        }
                        catch { }
                    }
                }
            }

            return true;
        }

        public static void OnRejected(IUser userobj)
        {
            if (!Settings.js_enabled)
                return;

            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;
                Objects.User u = script.GetUser(userobj);

                if (u != null)
                {
                    try
                    {
                        script.JS.CallGlobalFunction("onRejected", u);
                    }
                    catch (JavaScriptException e)
                    {
                        ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                    }
                    catch { }
                }
            }
        }

        public static bool OnVroomJoinCheck(IUser userobj, ushort vroom)
        {
            if (Settings.js_enabled)
            {
                CallistoScript[] s = ScriptManager.Scripts.ToArray();

                foreach (CallistoScript script in s)
                {
                    ScriptManager.PrintCounter = 0;
                    Objects.User u = script.GetUser(userobj);

                    if (u != null)
                    {
                        try
                        {
                            bool result = script.JS.CallGlobalFunction<bool>("onVroomJoinCheck", u, (int)vroom);

                            if (!result)
                                return false;
                        }
                        catch (JavaScriptException e)
                        {
                            ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                        }
                        catch { }
                    }
                }
            }

            return true;
        }

        public static void OnVroomJoin(IUser userobj)
        {
            if (!Settings.js_enabled)
                return;

            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;
                Objects.User u = script.GetUser(userobj);

                if (u != null)
                {
                    try
                    {
                        script.JS.CallGlobalFunction("onVroomJoin", u);
                    }
                    catch (JavaScriptException e)
                    {
                        ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                    }
                    catch { }
                }
            }
        }

        public static void OnVroomPart(IUser userobj)
        {
            if (!Settings.js_enabled)
                return;

            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;
                Objects.User u = script.GetUser(userobj);

                if (u != null)
                {
                    try
                    {
                        script.JS.CallGlobalFunction("onVroomPart", u);
                    }
                    catch (JavaScriptException e)
                    {
                        ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                    }
                    catch { }
                }
            }
        }

        public static void OnLoginRequested(IUser userobj)
        {
            if (!Settings.js_enabled)
                return;

            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;
                Objects.User u = script.GetUser(userobj);

                if (u != null)
                {
                    try
                    {
                        script.JS.CallGlobalFunction("onLoginAttempted", u, userobj.Level != AdminLevel.Regular);
                    }
                    catch (JavaScriptException e)
                    {
                        ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                    }
                    catch { }
                }
            }
        }

        public static void OnFileReceived(IUser userobj, String filename, MimeMessage type)
        {
            if (!Settings.js_enabled)
                return;

            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;
                Objects.User u = script.GetUser(userobj);

                if (u != null)
                {
                    try
                    {
                        script.JS.CallGlobalFunction("onFileReceived", u, filename);
                    }
                    catch (JavaScriptException e)
                    {
                        ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                    }
                    catch { }
                }
            }
        }

        public static bool OnFlood(IUser userobj, ProtoMessage msg)
        {
            if (Settings.js_enabled)
            {
                CallistoScript[] s = ScriptManager.Scripts.ToArray();

                foreach (CallistoScript script in s)
                {
                    ScriptManager.PrintCounter = 0;
                    Objects.User u = script.GetUser(userobj);

                    if (u != null)
                    {
                        try
                        {
                            bool result = script.JS.CallGlobalFunction<bool>("onFlood", u);

                            if (!result)
                                return false;
                        }
                        catch (JavaScriptException e)
                        {
                            ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                        }
                        catch { }
                    }
                }
            }

            return true;
        }

        public static bool OnPM(IUser userobj, IUser target, PMObject text)
        {
            if (Settings.js_enabled)
            {
                CallistoScript[] s = ScriptManager.Scripts.ToArray();

                foreach (CallistoScript script in s)
                {
                    ScriptManager.PrintCounter = 0;
                    Objects.User u = script.GetUser(userobj);
                    Objects.User t = script.GetUser(target);

                    if (u != null && t != null)
                    {
                        try
                        {
                            bool result = script.JS.CallGlobalFunction<bool>("onPM", u, t,
                                new Objects.PM(script.JS.Object.InstancePrototype, text));

                            if (!result || String.IsNullOrEmpty(text.GetText()))
                                return false;
                        }
                        catch (JavaScriptException e)
                        {
                            ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                        }
                        catch { }
                    }
                }
            }

            return true;
        }

        public static void OnBotPM(IUser userobj, String text)
        {
            if (!Settings.js_enabled)
                return;

            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;
                Objects.User u = script.GetUser(userobj);

                if (u != null)
                {
                    try
                    {
                        script.JS.CallGlobalFunction("onBotPM", u, text);
                    }
                    catch (JavaScriptException e)
                    {
                        ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                    }
                    catch { }
                }
            }
        }

        public static void OnLinkJoin(ILink room)
        {
            if (!Settings.js_enabled)
                return;

            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;

                try
                {
                    script.JS.CallGlobalFunction("onLinkConnected",
                        new Objects.LinkedRoom(script.JS.Object.InstancePrototype, room));
                }
                catch (JavaScriptException e)
                {
                    ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                }
                catch { }
            }
        }

        public static void OnLinkPart(ILink room)
        {
            if (!Settings.js_enabled)
                return;

            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;

                try
                {
                    script.JS.CallGlobalFunction("onLinkDisconnected",
                        new Objects.LinkedRoom(script.JS.Object.InstancePrototype, room));
                }
                catch (JavaScriptException e)
                {
                    ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                }
                catch { }
            }
        }

        public static void OnLinkError(ILink room, LinkError code)
        {
            if (!Settings.js_enabled)
                return;

            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;

                try
                {
                    script.JS.CallGlobalFunction("onLinkError",
                        new Objects.LinkedRoom(script.JS.Object.InstancePrototype, room), code.ToString());
                }
                catch (JavaScriptException e)
                {
                    ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                }
                catch { }
            }
        }

        public static void OnLinkCommand(IUser userobj, String command, IUser target, String args)
        {
            if (!Settings.js_enabled)
                return;

            CallistoScript[] s = ScriptManager.Scripts.ToArray();

            foreach (CallistoScript script in s)
            {
                ScriptManager.PrintCounter = 0;
                Objects.User u = script.GetUser(userobj, true);
                Objects.User t = script.GetUser(target);

                if (u != null)
                {
                    try
                    {
                        script.JS.CallGlobalFunction("onLinkCommand", u, command, t, args);
                    }
                    catch (JavaScriptException e)
                    {
                        ScriptManager.OnError(script.ScriptName, e.Message, e.LineNumber);
                    }
                    catch { }
                }
            }
        }

        private static bool CanScript(IUser userobj)
        {
            switch (Settings.script_level)
            {
                case 1:
                    return userobj.Level >= AdminLevel.Moderator;

                case 2:
                    return userobj.Level >= AdminLevel.Administrator;

                case 3:
                    return userobj.Level >= AdminLevel.Host;

                case 4:
                    return userobj.IsLocalHost;

                default:
                    return false;
            }
        }


    }
}
