﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Jurassic;
using Jurassic.Library;

namespace callisto.Scripting.Statics
{
    class Autorun : ObjectInstance
    {
        private static List<String> _scripts = new List<String>();

        internal static void AddScripts(String[] items)
        {
            _scripts.Clear();
            _scripts.AddRange(items);
        }

        public Autorun(ScriptEngine engine)
            : base(engine)
        {
            this.PopulateFunctions();
            this.DefineProperty("add", new PropertyDescriptor(this["add"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("remove", new PropertyDescriptor(this["remove"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("load", new PropertyDescriptor(this["load"], PropertyAttributes.Sealed), throwOnError: true);
        }

        private static String[] bad_chars = new String[]
        {
            "..",
            "/",
            "\\",
            " ",
        };

        [JSFunction(Name = "add")]
        public static bool Add(String filename)
        {
            if (filename.Length > 3)
                if (filename.EndsWith(".js"))
                    if (bad_chars.Count<String>(x => filename.Contains(x)) == 0)
                        if (!_scripts.Contains(filename))
                        {
                            _scripts.Add(filename);

                            String path = Settings.Path + "scriptloader";
                            String result = String.Join("\r\n", _scripts);

                            try
                            {
                                System.IO.File.WriteAllText(path, result);
                                return true;
                            }
                            catch { }
                        }

            return false;
        }

        [JSFunction(Name = "remove")]
        public static bool Remove(String filename)
        {
            if (_scripts.Contains(filename))
            {
                _scripts.Remove(filename);

                String path = Settings.Path + "scriptloader";
                String result = String.Join("\r\n", _scripts);

                try
                {
                    System.IO.File.WriteAllText(path, result);
                    return true;
                }
                catch { }
            }

            return false;
        }

        [JSFunction(Name = "load")]
        public static void Load()
        {
            ScriptManager.Autorun();
        }

        [JSProperty(Name = "scripts")]
        public String Scripts
        {
            get
            {
                String result = String.Empty;

                if (_scripts.Count > 0)
                    result = String.Join("\r\n", _scripts);

                return result;
            }
            set { }
        }
    }
}
