﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Jurassic;
using Jurassic.Library;

namespace callisto.Scripting.Statics
{
    class File : ObjectInstance
    {
        public File(ScriptEngine engine)
            : base(engine)
        {
            this.PopulateFunctions();
            this.DefineProperty("load", new PropertyDescriptor(this["load"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("exists", new PropertyDescriptor(this["exists"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("save", new PropertyDescriptor(this["save"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("append", new PropertyDescriptor(this["append"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("kill", new PropertyDescriptor(this["kill"], PropertyAttributes.Sealed), throwOnError: true);
        }

        private static String[] bad_chars_script = new String[]
        {
            "..",
            "/",
            "\\",
            " ",
        };

        [JSFunction(Name = "load", Flags = JSFunctionFlags.HasEngineParameter)]
        public static String JSLoad(ScriptEngine eng, object a)
        {
            if (a is String || a is ConcatenatedString)
            {
                String filename = a.ToString();

                if (filename.Length > 1)
                    if (bad_chars_script.Count<String>(x => filename.Contains(x)) == 0)
                    {
                        String path = Settings.Path + "Scripts\\" + eng.ScriptName + "\\data\\" + filename;

                        try
                        {
                            return System.IO.File.ReadAllText(path);
                        }
                        catch { }
                    }
            }

            return null;
        }

        [JSFunction(Name = "exists", Flags = JSFunctionFlags.HasEngineParameter)]
        public static bool JSExists(ScriptEngine eng, object a)
        {
            if (a is String || a is ConcatenatedString)
            {
                String filename = a.ToString();

                if (filename.Length > 1)
                    if (bad_chars_script.Count<String>(x => filename.Contains(x)) == 0)
                    {
                        String path = Settings.Path + "Scripts\\" + eng.ScriptName + "\\data\\" + filename;

                        try
                        {
                            return System.IO.File.Exists(path);
                        }
                        catch { }
                    }
            }

            return false;
        }

        [JSFunction(Name = "save", Flags = JSFunctionFlags.HasEngineParameter)]
        public static bool JSSave(ScriptEngine eng, object a, object b, object c) // file, content || file, script, content
        {
            if (!(c is Undefined))
            {
                if (!(a is String || a is ConcatenatedString) || !(b is String || b is ConcatenatedString) || !(c is String || c is ConcatenatedString))
                    return false;

                String file = a.ToString();
                String script = b.ToString();
                String content = c.ToString();

                if (file.Length > 1)
                    if (bad_chars_script.Count<String>(x => file.Contains(x)) == 0)
                        if (script.Length > 3 && script.EndsWith(".js"))
                            if (bad_chars_script.Count<String>(x => script.Contains(x)) == 0)
                                if (bad_chars_script.Count<String>(x => file.Contains(x)) == 0)
                                {
                                    try
                                    {
                                        String path = Settings.Path + "Scripts\\" + script + "\\data";

                                        if (!Directory.Exists(path))
                                            Directory.CreateDirectory(path);

                                        path += "\\" + file;
                                        System.IO.File.WriteAllText(path, content);
                                        return true;
                                    }
                                    catch { }
                                }
            }
            else
            {
                if (!(a is String || a is ConcatenatedString) || !(b is String || b is ConcatenatedString))
                    return false;

                String file = a.ToString();
                String script = eng.ScriptName;
                String content = b.ToString();

                if (file.Length > 1)
                    if (bad_chars_script.Count<String>(x => file.Contains(x)) == 0)
                    {
                        try
                        {
                            String path = Settings.Path + "Scripts\\" + script + "\\data";

                            if (!Directory.Exists(path))
                                Directory.CreateDirectory(path);

                            path += "\\" + file;
                            System.IO.File.WriteAllText(path, content);
                            return true;
                        }
                        catch { }
                    }
            }

            return false;
        }

        [JSFunction(Name = "append", Flags = JSFunctionFlags.HasEngineParameter)]
        public static bool JSAppend(ScriptEngine eng, object a, object b) // file, content
        {
            if (!(a is String || a is ConcatenatedString) || !(b is String || b is ConcatenatedString))
                return false;

            String file = a.ToString();
            String script = eng.ScriptName;
            String content = b.ToString();

            if (file.Length > 1)
                if (bad_chars_script.Count<String>(x => file.Contains(x)) == 0)
                {
                    try
                    {
                        String path = Settings.Path + "Scripts\\" + script + "\\data";

                        if (!Directory.Exists(path))
                            Directory.CreateDirectory(path);

                        path += "\\" + file;

                        using (StreamWriter stream = System.IO.File.Exists(path)
                            ? System.IO.File.AppendText(path) : System.IO.File.CreateText(path))
                            stream.Write(content);

                        return true;
                    }
                    catch { }
                }

            return false;
        }

        [JSFunction(Name = "kill", Flags = JSFunctionFlags.HasEngineParameter)]
        public static bool JDDelete(ScriptEngine eng, object a)
        {
            if (a is String || a is ConcatenatedString)
            {
                String file = a.ToString();
                String script = eng.ScriptName;

                if (file.Length > 1)
                    if (bad_chars_script.Count<String>(x => file.Contains(x)) == 0)
                    {
                        String path = Settings.Path + "Scripts\\" + script + "\\data\\" + file;

                        try
                        {
                            if (System.IO.File.Exists(path))
                            {
                                System.IO.File.Delete(path);
                                return true;
                            }
                        }
                        catch { }
                    }
            }

            return false;
        }
    }
}
