﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Jurassic;
using Jurassic.Library;
using ICallisto;

namespace callisto.Scripting.Statics
{
    class Global
    {
        public CallistoScript parent { get; set; }

        public Global(CallistoScript script)
        {
            this.parent = script;
        }

        public int ByteLength(object a)
        {
            if (a == null)
                return -1;

            return Encoding.UTF8.GetByteCount(a.ToString());
        }

        public String ClrName(object a)
        {
            if (a == null)
                return "null";

            return a.GetType().ToString();
        }

        public bool CanScript(object a)
        {
            if (!Settings.js_inroom_enabled)
                return false;

            if (!(a is Objects.User))
                return false;
            
            Objects.User u = (Objects.User)a;

            if (u.Level >= Settings.script_level)
                return true;

            return u.LocalHost;
        }

        public Objects.User FirstUser()
        {
            if (this.parent.local_users.Count == 0)
                return null;

            if (this.parent.local_users.Count == 1)
                return this.parent.local_users[0];

            return this.parent.local_users.OrderBy(a => a.Id).ToList()[0];
        }

        public Objects.User FirstLinked()
        {
            if (this.parent.linked_users.Count == 0)
                return null;

            return this.parent.linked_users[0];
        }

        public Objects.BannedUser FirstBanned()
        {
            IBan b = BanManager.GetFirstBanned;

            if (b != null)
                return new Objects.BannedUser(this.parent.JS.Object.InstancePrototype, b, this.parent.ScriptName);

            return null;
        }

        public Objects.User User(object a, object b) // name/id, link_only
        {
            if (a is Null)
                return null;

            bool link_only = false;

            if (b is bool)
                link_only = (bool)b;

            if (link_only)
            {
                if (!(a is String || a is ConcatenatedString))
                    return null;

                if (a.ToString().Length < 2)
                    return null;

                return this.parent.linked_users.Find(x => x.Name.StartsWith((String)a));
            }

            Objects.User result = null;

            if (a is String || a is ConcatenatedString)
            {
                if (a.ToString().Length < 2)
                    return null;

                result = this.parent.local_users.Find(x => x.Name.StartsWith((String)a));
            }
            else if (a is int || a is double)
            {
                int _i;

                if (int.TryParse(a.ToString(), out _i))
                    result = this.parent.local_users.Find(x => x.Id == _i);
            }

            if (result != null)
                return result;
            else if (a is String || a is ConcatenatedString)
                if (a.ToString().Length >= 2)
                    return this.parent.linked_users.Find(x => x.Name.StartsWith((String)a));

            return null;
        }

        public void Print(object a, object b)
        {
            if (ScriptManager.PrintCounter++ >= 500)
                return;

            if (b is Undefined) // to all local users
            {
                String result = null;

                if (!(a is Null || a is Undefined))
                    if (a is bool)
                        result = ((bool)a).ToString().ToLower();
                    else if (a != null)
                        result = a.ToString();

                if (result != null)
                    foreach (Objects.User u in this.parent.local_users)
                        u.parent.SendPrint(result);
            }
            else if (a is int || a is double) // to vroom
            {
                int _i;

                if (int.TryParse(a.ToString(), out _i))
                {
                    if (_i >= 0 && _i <= 65535)
                    {
                        String result = null;

                        if (!(b is Null || b is Undefined))
                            if (b is bool)
                                result = ((bool)b).ToString().ToLower();
                            else if (b != null)
                                result = b.ToString();

                        if (result != null)
                            foreach (Objects.User u in this.parent.local_users)
                                if (u.Vroom == _i)
                                    u.parent.SendPrint(result);
                    }
                }
            }
            else if (a is Objects.User) // to user
            {
                Objects.User u = (Objects.User)a;

                if (u != null)
                {
                    String result = null;

                    if (!(b is Null || b is Undefined))
                        if (b is bool)
                            result = ((bool)b).ToString().ToLower();
                        else if (b != null)
                            result = b.ToString();

                    if (result != null)
                        u.parent.SendPrint(result);
                }
            }
            else if (b is bool) // to all users
            {
                String result = null;

                if (!(a is Null || a is Undefined))
                    if (a is bool)
                        result = ((bool)a).ToString().ToLower();
                    else if (a != null)
                        result = a.ToString();

                if (result != null)
                {
                    foreach (Objects.User u in this.parent.local_users)
                        u.parent.SendPrint(result);

                    if ((bool)b)
                        foreach (Objects.User u in this.parent.linked_users)
                            u.parent.SendPrint(result);
                }
            }
        }

        public void SendPM(object a, object b, object c)
        {
            if (ScriptManager.PrintCounter++ >= 500)
                return;

            if (!(a is Objects.User))
                return;

            if (b is Null || c is Null)
                return;

            if (b != null && c != null)
            {
                String sender = b.ToString();
                String text = c.ToString();

                if (String.IsNullOrEmpty(sender) || String.IsNullOrEmpty(text))
                    return;

                ((Objects.User)a).parent.SendPM(sender, text);
            }
        }
    }
}
