﻿using System;
using System.Net;
using Jurassic;
using Jurassic.Library;

namespace callisto.Scripting.Statics
{
    class Hashlink : ObjectInstance
    {
        public Hashlink(ScriptEngine engine)
            : base(engine)
        {
            this.PopulateFunctions();
            this.DefineProperty("encode", new PropertyDescriptor(this["encode"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("decode", new PropertyDescriptor(this["decode"], PropertyAttributes.Sealed), throwOnError: true);
        }

        [JSFunction(Name = "encode")]
        public static String Encode(object a)
        {
            if (a is ObjectInstance)
            {
                try
                {
                    ObjectInstance obj = (ObjectInstance)a;
                    ChannelObject chan = new ChannelObject
                    {
                        ExternalIP = IPAddress.Parse(obj.GetPropertyValue("ip").ToString()),
                        LocalIP = IPAddress.Parse(obj.GetPropertyValue("ip").ToString()),
                        Port = ushort.Parse(obj.GetPropertyValue("port").ToString()),
                        Name = obj.GetPropertyValue("name").ToString(),
                        Topic = String.Empty
                    };

                    String str = HashlinkHelpers.EncodeHashlink(chan);

                    if (str != null)
                        return str;
                }
                catch { }
            }

            return null;
        }

        [JSFunction(Name = "decode", Flags = JSFunctionFlags.HasEngineParameter)]
        public static Objects.HashlinkResult Decode(ScriptEngine eng, object a)
        {
            if (a is String || a is ConcatenatedString)
            {
                String str = a.ToString();

                try
                {
                    ChannelObject chan = HashlinkHelpers.DecodeHashlink(a.ToString());

                    if (chan != null)
                        return new Objects.HashlinkResult(eng.Object.InstancePrototype, chan);
                }
                catch { }
            }

            return null;
        }

    }
}
