﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Jurassic;
using Jurassic.Library;

namespace callisto.Scripting.Statics
{
    class Linker : ObjectInstance
    {
        public Linker(ScriptEngine engine)
            : base(engine)
        {
            this.PopulateFunctions();
            this.DefineProperty("connect", new PropertyDescriptor(this["connect"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("disconnect", new PropertyDescriptor(this["disconnect"], PropertyAttributes.Sealed), throwOnError: true);
        }

        [JSProperty(Name = "canLink")]
        public static bool CanLink
        {
            get { return Settings.RoomLinking; }
            set { }
        }

        [JSProperty(Name = "linkedAdmins")]
        public static bool CanLinkAdmins
        {
            get { return Settings.AdminLinking; }
            set { }
        }

        [JSProperty(Name = "externalIp")]
        public static String ExternalIP
        {
            get { return IsLinked ? CallistoServer.Linker.ExternalIP.ToString() : "0.0.0.0"; }
            set { }
        }

        [JSProperty(Name = "hashlink")]
        public static String Hashlink
        {
            get { return IsLinked ? CallistoServer.Linker.Hashlink : String.Empty; }
            set { }
        }

        [JSProperty(Name = "isLinked")]
        public static bool IsLinked
        {
            get { return CallistoServer.Linker.IsLinking; }
            set { }
        }

        [JSProperty(Name = "name")]
        public static String Name
        {
            get { return IsLinked ? CallistoServer.Linker.Name : String.Empty; }
            set { }
        }

        [JSProperty(Name = "port")]
        public static int Port
        {
            get { return IsLinked ? CallistoServer.Linker.Port : 0; }
            set { }
        }

        [JSProperty(Name = "userCount")]
        public static int UserCount
        {
            get { return IsLinked ? CallistoServer.Linker.UserCount : 0; }
            set { }
        }

        [JSFunction(Name = "connect")]
        public static void Connect(object a)
        {
            if (a is String || a is ConcatenatedString)
            {
                String str = a.ToString();

                if (!String.IsNullOrEmpty(str))
                    CallistoServer.Linker.Connect(str, Helpers.UnixTime);
            }
        }

        [JSFunction(Name = "disconnect")]
        public static void Disconnect()
        {
            if (CallistoServer.Linker.IsLinking)
                CallistoServer.Linker.PartAndDisconnect();
            else if (CallistoServer.Linker.IsActive)
                CallistoServer.Linker.Disconnect();
        }
    }
}
