﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Jurassic;
using Jurassic.Library;
using Microsoft.Win32;

namespace callisto.Scripting.Statics
{
    class Reg : ObjectInstance
    {
        public Reg(ScriptEngine engine)
            : base(engine)
        {
            this.PopulateFunctions();
            this.DefineProperty("exists", new PropertyDescriptor(this["exists"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("getValue", new PropertyDescriptor(this["getValue"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("setValue", new PropertyDescriptor(this["setValue"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("deleteValue", new PropertyDescriptor(this["deleteValue"], PropertyAttributes.Sealed), throwOnError: true);
            this.DefineProperty("clear", new PropertyDescriptor(this["clear"], PropertyAttributes.Sealed), throwOnError: true);
        }

        [JSFunction(Name = "exists", Flags = JSFunctionFlags.HasEngineParameter)]
        public static bool Exists(ScriptEngine eng, String key)
        {
            String script = eng.ScriptName;
            RegistryKey rk = Registry.CurrentUser.OpenSubKey("Software\\callisto", false);

            if (rk == null)
                return false;

            rk.Close();
            rk = Registry.CurrentUser.OpenSubKey("Software\\callisto\\" + script, false);

            if (rk == null)
                return false;

            object value = rk.GetValue(key);
            rk.Close();

            if (value == null)
                return false;

            return true;
        }

        [JSFunction(Name = "getValue", Flags = JSFunctionFlags.HasEngineParameter)]
        public static String GetValue(ScriptEngine eng, String key)
        {
            String script = eng.ScriptName;
            RegistryKey rk = Registry.CurrentUser.OpenSubKey("Software\\callisto", false);

            if (rk == null)
                return null;

            rk.Close();
            rk = Registry.CurrentUser.OpenSubKey("Software\\callisto\\" + script, false);

            if (rk == null)
                return null;

            object value = rk.GetValue(key);
            rk.Close();

            if (value == null)
                return null;

            String str = (String)value;

            if (str.Length > 1)
                return str.Substring(2);

            return null;
        }

        [JSFunction(Name = "setValue", Flags = JSFunctionFlags.HasEngineParameter)]
        public static bool SetValue(ScriptEngine eng, String key, object value)
        {
            String script = eng.ScriptName;
            RegistryKey rk = Registry.CurrentUser.OpenSubKey("Software\\callisto", false);

            if (rk == null)
                Registry.CurrentUser.CreateSubKey("Software\\callisto");
            else
                rk.Close();

            rk = Registry.CurrentUser.OpenSubKey("Software\\callisto\\" + script, true);

            if (rk == null)
            {
                Registry.CurrentUser.CreateSubKey("Software\\callisto\\" + script);
                rk = Registry.CurrentUser.OpenSubKey("Software\\callisto\\" + script, true);
            }

            if (value is int || value is String || value is double || value is ConcatenatedString)
            {
                rk.SetValue(key, (String)("ST" + value.ToString()));
                rk.Close();
                return true;
            }

            rk.Close();
            return false;
        }

        [JSFunction(Name = "deleteValue", Flags = JSFunctionFlags.HasEngineParameter)]
        public static bool DeleteValue(ScriptEngine eng, String key)
        {
            String script = eng.ScriptName;
            RegistryKey rk = Registry.CurrentUser.OpenSubKey("Software\\callisto", false);

            if (rk == null)
                return false;
            else
                rk.Close();

            rk = Registry.CurrentUser.OpenSubKey("Software\\callisto\\" + script, true);

            if (rk == null)
                return false;

            try
            {
                rk.DeleteValue(key, true);
            }
            catch  // value doesn't exist
            {
                rk.Close();
                return false;
            }

            rk.Close();
            return true;
        }

        [JSFunction(Name = "clear", Flags = JSFunctionFlags.HasEngineParameter)]
        public static bool Clear(ScriptEngine eng)
        {
            String script = eng.ScriptName;
            RegistryKey rk = Registry.CurrentUser.OpenSubKey("Software\\callisto", true);

            if (rk == null)
                return false;

            try
            {
                rk.DeleteSubKey(script, true);
            }
            catch // script subkey doesn't exist
            {
                rk.Close();
                return false;
            }

            rk.Close();
            return true;
        }

    }
}
