﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Jurassic;
using Jurassic.Library;

namespace callisto.Scripting.Statics
{
    class Room : ObjectInstance
    {
        public Room(ScriptEngine engine)
            : base(engine)
        {
            this.PopulateFunctions();
            this.DefineProperty("setUrl", new PropertyDescriptor(this["setUrl"], PropertyAttributes.Sealed), throwOnError: true);
        }

        [JSProperty(Name = "version")]
        public static int Version
        {
            get
            {
                String str = About.CALLISTO_VERSION;
                str = str.Replace(".", "");
                int i;

                if (int.TryParse(str, out i))
                    return i;

                return 1;
            }
            set { }
        }

        [JSProperty(Name = "botName")]
        public static String BotName
        {
            get { return Settings.BotName; }
            set { }
        }

        [JSProperty(Name = "customNames")]
        public static bool CustomNames
        {
            get { return Settings.CAN_CUSTOM_NAME; }
            set { Settings.CAN_CUSTOM_NAME = value; }
        }

        [JSProperty(Name = "externalIp")]
        public static String ExternalIP
        {
            get { return Settings.ServerIP.ToString(); }
            set { }
        }

        [JSProperty(Name = "hashlink")]
        public static String Hashlink
        {
            get
            {
                ChannelObject chan = new ChannelObject();
                chan.external_ip = Settings.ServerIP;
                chan.local_ip = Settings.ServerIP;
                chan.name = Settings.ServerName;
                chan.port = (ushort)Settings.ServerPort;
                chan.topic = String.Empty;
                String str = HashlinkHelpers.EncodeHashlink(chan);
                return str;
            }
            set { }
        }

        [JSProperty(Name = "name")]
        public static String Name
        {
            get { return Settings.ServerName; }
            set { }
        }

        [JSProperty(Name = "port")]
        public static int Port
        {
            get { return Settings.ServerPort; }
            set { }
        }

        [JSProperty(Name = "startTime")]
        public static double StartTime
        {
            get { return Settings.StartTime; }
            set { }
        }

        [JSProperty(Name = "topic")]
        public static String Topic
        {
            get { return Settings.ServerTopic; }
            set { Settings.ServerTopic = value; }
        }

        [JSFunction(Name = "setUrl")]
        public static void SetUrl(object a, object b)
        {
            if (a == null || b == null)
                return;

            String addr = String.Empty;
            String text = String.Empty;

            if (!(a is Undefined || a is Null))
                addr = a.ToString();

            if (!(b is Undefined || b is Null))
                text = b.ToString();

            if (String.IsNullOrEmpty(addr) && String.IsNullOrEmpty(text))
                Settings.Url = new String[] { String.Empty, String.Empty };
            else if (!String.IsNullOrEmpty(addr))
                if (!String.IsNullOrEmpty(text))
                    Settings.Url = new String[] { addr, text };
        }
    }
}
