﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;
using System.Xml;
using ICallisto;

namespace callisto
{
    class BanManager
    {
        private static List<BanItem> items = new List<BanItem>();

        public static void AddBan(IUser userobj)
        {
            if (items.Find(x => x.ExternalIP.Equals(userobj.ExternalIP) || x.Guid == userobj.Guid) == null) // user doesn't exist
            {
                items.Add(new BanItem
                {
                    guid = userobj.Guid,
                    externalip = userobj.ExternalIP,
                    localip = userobj.LocalIP,
                    name = userobj.Name,
                    version = userobj.Version,
                    port = userobj.Port,
                    banned = true
                });

                Save();
            }
            else // user exists but maybe not flagged as banned?
            {
                int id = items.FindIndex(x => (x.ExternalIP.Equals(userobj.ExternalIP) || x.Guid == userobj.Guid) && !x.banned);

                if (id > -1)
                {
                    items[id].banned = true;
                    Save();
                }
            } // user exists already and flagged as banned, forget it!
        }

        public static IBan GetFirstBanned
        {
            get { return items.Find(x => x.banned); }
        }

        public static IBan GetNextBanned(IBan userobj)
        {
            int id = items.FindIndex(x => x.ExternalIP.Equals(userobj.ExternalIP) || x.Guid == userobj.Guid);

            if (id > -1)
            {
                for (int i = (id + 1); i < items.Count; i++)
                    if (items[i].banned)
                        return items[i];
            }

            return null;
        }

        public static bool IsBanned(IUser userobj)
        {
            return items.Find(x => (x.externalip.Equals(userobj.ExternalIP) || x.guid == userobj.Guid) && x.banned) != null;
        }

        public static void Clear()
        {
            items.Clear();
            Save();
        }

        public static void Load()
        {
            try
            {
                FileStream f = new FileStream(Settings.Path + "bans.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("bans");

                while (xml.ReadToFollowing("item"))
                {
                    xml.ReadSubtree().ReadToFollowing("guid");
                    String guid = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));

                    xml.ReadToFollowing("externalip");
                    IPAddress externalip = new IPAddress(Convert.FromBase64String(xml.ReadElementContentAsString()));

                    xml.ReadToFollowing("localip");
                    IPAddress localip = new IPAddress(Convert.FromBase64String(xml.ReadElementContentAsString()));

                    xml.ReadToFollowing("name");
                    String name = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));

                    xml.ReadToFollowing("version");
                    String version = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));

                    xml.ReadToFollowing("port");
                    ushort port = ushort.Parse(xml.ReadElementContentAsString());

                    items.Add(new BanItem
                    {
                        guid = guid,
                        externalip = externalip,
                        localip = localip,
                        name = name,
                        version = version,
                        port = port,
                        banned = true
                    });
                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }
        }

        public static void Save()
        {
            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(Settings.Path + "bans.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("bans");

                foreach (BanItem a in items)
                {
                    if (a.banned)
                    {
                        xml.WriteStartElement("item");
                        xml.WriteElementString("guid", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.guid)));
                        xml.WriteElementString("externalip", Convert.ToBase64String(a.externalip.GetAddressBytes()));
                        xml.WriteElementString("localip", Convert.ToBase64String(a.localip.GetAddressBytes()));
                        xml.WriteElementString("name", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.name)));
                        xml.WriteElementString("version", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.version)));
                        xml.WriteElementString("port", a.port.ToString());
                        xml.WriteEndElement();
                    }
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }
        }
    }

    class BanItem : IBan
    {
        public String guid;
        public IPAddress externalip;
        public IPAddress localip;
        public String name;
        public String version;
        public ushort port;
        public bool banned;

        public String Guid
        {
            get { return this.guid; }
        }

        public IPAddress ExternalIP
        {
            get { return this.externalip; }
        }

        public IPAddress LocalIP
        {
            get { return this.localip; }
        }

        public String Name
        {
            get { return this.name; }
        }

        public String Version
        {
            get { return this.version; }
        }

        public ushort Port
        {
            get { return this.port; }
        }

        public bool Banned
        {
            get { return true; }
            set
            {
                if (!value)
                {
                    this.banned = false;
                    BanManager.Save();
                }
            }
        }

        public IBan NextUser
        {
            get { return BanManager.GetNextBanned(this); }
        }
    }
}
