﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Windows.Forms;
using ICallisto;

namespace callisto
{
    class CallistoServer
    {
        public static LinkClient Linker;

        private TcpListener tserver;
        private UdpListener userver;
        private WebServer wserver;

        private Thread thread;
        private bool terminate = false;

        public delegate void UdpReportDelegate();
        public event UdpReportDelegate OnUdpReport;

        public void Start()
        {
            this.thread = new Thread(new ThreadStart(this.BackgroundWorker));
            this.thread.Start();
        }

        public void Stop()
        {
            this.terminate = true;

            try { this.tserver.Stop(); }
            catch { }
            try { this.userver.Stop(); }
            catch { }

            UserPool.KillUserPool();

            if (Linker.IsLinking)
                Linker.Disconnect();

            if (this.wserver != null)
                this.wserver.Stop();

            Scripting.ScriptManager.Destroy();
        }

        public void Stop(bool x)
        {
            this.terminate = true;
            Scripting.ScriptManager.Destroy();
        }

        private void BackgroundWorker()
        {
            CallistoPluginStats.ResetStats();
            UserPool.CreateUserPool();
            WebPool.CreateUserPool();
            Captcha.LoadGuids();

            Linker = new LinkClient();

            this.terminate = false;

            this.tserver = new TcpListener(IPAddress.Any, Settings.ServerPort);

            try { this.tserver.Start(); }
            catch            
            {
                MessageBox.Show("This port is already in use by a different application",
                    "Callisto Server", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            this.userver = new UdpListener();

            try { this.userver.Start(); }
            catch
            {
                MessageBox.Show("This port is already in use by a different application",
                    "Callisto Server", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            this.wserver = new WebServer(Helpers.UnixTime);

            uint text_flood_timer = Helpers.UnixTime;
            uint plugin_timer = Helpers.UnixTime;
            uint proxy_timer = Helpers.UnixTime - 21600;
            uint cap_timer = Helpers.UnixTime;

            Scripting.TimerList.NextIdent = 0;
            Scripting.ScriptManager.Autorun();

            while (true) // main server loop
            {
                if (this.terminate) return;

                uint time = Helpers.UnixTime;

                if ((text_flood_timer + 60) < time)
                {
                    text_flood_timer = time;
                    Helpers.last_typer.Clear();
                }

                if (plugin_timer < time)
                {
                    plugin_timer = time;
                    ServerEvents.OnTimer();
                    Scripting.ServerEvents.OneSecondTick();
                    UserPool.UpdateFastPingers(time);
                    this.OnUdpReport();
                }

                if ((proxy_timer + 21600) < time)
                {
                    proxy_timer = time;
                    Proxies.Update();
                }

                if ((cap_timer + 1800) < time)
                {
                    cap_timer = time;
                    Captcha.SaveGuids();
                }

                this.ListenForNewConnections(time);
                this.ServiceConnectedUsers(time);
                this.wserver.DoTasks(time);
                this.ServiceLink(time);
                this.userver.DoTasks(time);
                ServerEvents.STT();
                Scripting.TimerList.UpdateTimers(Helpers.UnixTimeMS);
                Scripting.DNSManager.DequeuePending();

                Thread.Sleep(50);
            }
        }

        private void ListenForNewConnections(uint time)
        {
            while (this.tserver.Pending()) // (accept if pending) == non blocking
            {
                Socket sock = this.tserver.AcceptSocket();

                if (!this.userver.IsFirewall(sock))
                {
                    int id = UserPool.NextAvailable;

                    if (id > -1)
                        UserPool.Pool[id] = new UserObject(sock, (ushort)id, time);
                    else
                        sock.Disconnect(false); // pool is full!
                }
            }
        }

        private void ServiceConnectedUsers(uint time)
        {
            foreach (UserObject userobj in UserPool.Pool)
            {
                if (userobj.Connected)
                {
                    userobj.DispatchOutbound(); // send pending outbound
                    userobj.ReceiveInbound(); // receive pending inbound

                    if (!userobj.logged_in) // no login packet yet
                    {
                        if (userobj.IsHTML5) // WEBSOCKETS USER
                        {
                            int id = WebPool.NextAvailable;

                            if (id > -1 && Settings.allow_silverlight)
                            {
                                WebPool.Pool[id] = new WebUser(userobj, (id + 700), time);
                                userobj.socket = null;
                            }
                            else userobj.Disconnect();

                            continue;
                        }

                        if (time > (userobj.spawn_time + 10)) // 10 seconds pass and user still fails to send login
                        {
                            userobj.Disconnect();
                            continue;
                        }
                    }
                    else // sent update packet lately?
                    {
                        if (time > (userobj.last_update_time + 300)) // 5 minutes pass and user still fails to send update
                        {
                            userobj.Disconnect();
                            continue;
                        }
                    }

                    if (userobj.receive_fails > 3) // remote user has disconnected from the server
                    {
                        userobj.Disconnect(); // tidy up
                        continue;
                    }

                    AresPacket p;

                    while ((p = userobj.NextPacket) != null) // process pending packets
                    {
                        if (userobj.Connected)
                        {
                            if (userobj.EncMode == EncryptionMode.Encrypted)
                                p.Packet = ChatPacketHelpers.Decrypt(p.Packet, userobj.EncByte, userobj.EncWord, p.Msg);

                            try
                            {
                                if ((byte)p.Msg < 200)
                                    if (p.Packet.Length > 4040)
                                        throw new Exception();

                                ChatPacketProcessor.Eval(userobj, p.Msg, new AresPacketReader(p.Packet), time);
                            }
                            catch
                            {
                                userobj.Disconnect();
                                break;
                            }
                        }
                    }
                }
            }
        }

        private void ServiceLink(uint time)
        {
            if (!Linker.IsActive)
                return; // not linking right now

            if (Linker.Mode == LinkMode.Client)
                if (!Linker.ClientConnected(time))
                    return; // we are client but still not connected yet

            Linker.DispatchOutbound();
            Linker.ReceiveInbound();

            if (time > (Linker.last_update_received_time + 180)) // 3 minutes pass and user still fails to send update
            {
                Linker.PartAndDisconnect();
                ServerEvents.OnLinkError(Linker, LinkError.PingTimeout);
                return;
            }

            if (time > (Linker.last_update_sent_time + 60)) // have we updated lately?
            {
                Linker.last_update_sent_time = time;
                Linker.SendPacket(LinkPacketHelpers.Pong());
            }

            if (Linker.receive_fails > 3) // remote server has disconnected from us
            {
                Linker.PartAndDisconnect();
                ServerEvents.OnLinkError(Linker, LinkError.RemoteConnectionLost);
                return;
            }

            AresPacket p;

            while ((p = Linker.NextPacket) != null) // process pending packets
            {
                try
                {
                    if (p.Msg != ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL)
                        if (p.Packet.Length > 4000)
                            throw new Exception();

                    LinkPacketProcessor.Eval(p.Msg, new AresPacketReader(p.Packet), time);
                }
                catch
                {
                    Linker.PartAndDisconnect();
                    ServerEvents.OnLinkError(Linker, LinkError.MaliciousPacket);
                    return;
                }
            }
        }
    }
}
