﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using ICallisto;

namespace callisto
{
    class ChatPacketHelpers
    {
        public static byte[] Error(String text)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(text, false);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_ERROR);
        }

        public static byte[] LoginAck(UserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.name);
            packet.WriteString(Settings.ServerName);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_LOGIN_ACK);
        }

        public static byte[] UpdateUserStatus(UserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.name);
            packet.WriteInt16(userobj.file_count);
            packet.WriteByte(userobj.browsable ? (byte)1 : (byte)0);
            packet.WriteIP(userobj.node_ip);
            packet.WriteInt16(userobj.node_port);
            packet.WriteIP(Settings.allow_dc ? userobj.external_ip : IPAddress.Parse("0.0.0.0"));
            packet.WriteByte((byte)userobj.level);
            packet.WriteByte(userobj.age);
            packet.WriteByte(userobj.sex);
            packet.WriteByte(userobj.country);
            packet.WriteString(userobj.region);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_UPDATE_USER_STATUS);
        }

        public static byte[] Redirect(ChannelObject cobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteIP(cobj.external_ip);
            packet.WriteInt16(cobj.port);
            packet.WriteIP(cobj.local_ip);
            packet.WriteString(cobj.name);
            packet.WriteString("Redirecting you to " + cobj.name + "...");
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_REDIRECT);
        }

        public static byte[] Public(String username, String text)
        {
            if (text.Length > 300)
                text = text.Substring(0, 300);

            while (Encoding.UTF8.GetByteCount(text) > 300)
                text = text.Substring(0, text.Length - 1);

            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(username);
            packet.WriteString(text, false);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_PUBLIC);
        }

        public static byte[] Emote(String username, String text)
        {
            if (text.Length > 300)
                text = text.Substring(0, 300);

            while (Encoding.UTF8.GetByteCount(text) > 300)
                text = text.Substring(0, text.Length - 1);

            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(username);
            packet.WriteString(text, false);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_EMOTE);
        }

        public static byte[] Join(UserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteInt16(userobj.file_count);
            packet.WriteInt32(0);
            packet.WriteIP(Settings.allow_dc ? userobj.external_ip : IPAddress.Parse("0.0.0.0"));
            packet.WriteInt16(Settings.allow_dc ? userobj.dc_port : (ushort)0);
            packet.WriteIP(userobj.node_ip);
            packet.WriteInt16(userobj.node_port);
            packet.WriteByte(0);
            packet.WriteString(userobj.name);
            packet.WriteIP(Settings.allow_dc ? userobj.local_ip : IPAddress.Parse("0.0.0.0"));
            packet.WriteByte(userobj.browsable ? (byte)1 : (byte)0);
            packet.WriteByte((byte)userobj.level);
            packet.WriteByte(userobj.age);
            packet.WriteByte(userobj.sex);
            packet.WriteByte(userobj.country);
            packet.WriteString(userobj.region);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_JOIN);
        }

        public static byte[] Join(UserObject userobj, bool include_version)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteInt16(userobj.file_count);
            packet.WriteInt32(0);
            packet.WriteIP(Settings.allow_dc ? userobj.external_ip : IPAddress.Parse("0.0.0.0"));
            packet.WriteInt16(Settings.allow_dc ? userobj.dc_port : (ushort)0);
            packet.WriteIP(userobj.node_ip);
            packet.WriteInt16(userobj.node_port);
            packet.WriteByte(0);
            packet.WriteString(userobj.name);
            packet.WriteIP(Settings.allow_dc ? userobj.local_ip : IPAddress.Parse("0.0.0.0"));
            packet.WriteByte(userobj.browsable ? (byte)1 : (byte)0);
            packet.WriteByte((byte)userobj.level);

            if (include_version)
            {
                packet.WriteString(userobj.version);
                packet.WriteGuid(userobj.guid);
                packet.WriteByte(userobj.Muzzled ? (byte)1 : (byte)00);
            }

            packet.WriteByte(userobj.age);
            packet.WriteByte(userobj.sex);
            packet.WriteByte(userobj.country);
            packet.WriteString(userobj.region);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_JOIN);
        }

        public static byte[] FastPing()
        {
            return new AresPacketWriter().ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_FASTPING);
        }

        public static byte[] Private(String username, String text)
        {
            if (text.Length > 300)
                text = text.Substring(0, 300);

            while (Encoding.UTF8.GetByteCount(text) > 300)
                text = text.Substring(0, text.Length - 1);

            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(username);
            packet.WriteString(text, false);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_PVT);
        }

        public static byte[] IsIgnoringYou(String name)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(name);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_ISIGNORINGYOU);
        }

        public static byte[] OfflineUser(String name)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(name);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_OFFLINEUSER);
        }

        public static byte[] Part(UserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.name);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_PART);
        }

        public static byte[] UserListItem(UserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteInt16(userobj.file_count);
            packet.WriteInt32(0);
            packet.WriteIP(Settings.allow_dc ? userobj.external_ip : IPAddress.Parse("0.0.0.0"));
            packet.WriteInt16(Settings.allow_dc ? userobj.dc_port : (ushort)0);
            packet.WriteIP(userobj.node_ip);
            packet.WriteInt16(userobj.node_port);
            packet.WriteByte(0);
            packet.WriteString(userobj.name);
            packet.WriteIP(Settings.allow_dc ? userobj.local_ip : IPAddress.Parse("0.0.0.0"));
            packet.WriteByte(userobj.browsable ? (byte)1 : (byte)0);
            packet.WriteByte((byte)userobj.level);
            packet.WriteByte(userobj.age);
            packet.WriteByte(userobj.sex);
            packet.WriteByte(userobj.country);
            packet.WriteString(userobj.region);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_CHANNEL_USER_LIST);
        }

        public static byte[] UserListBotItem()
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteInt16(0);
            packet.WriteInt32(0);
            packet.WriteIP(IPAddress.Parse("0.0.0.0"));
            packet.WriteInt16(0);
            packet.WriteIP(IPAddress.Parse("0.0.0.0"));
            packet.WriteInt16(0);
            packet.WriteByte(0);
            packet.WriteString(Settings.BotName);
            packet.WriteIP(IPAddress.Parse("0.0.0.0"));
            packet.WriteByte(0);
            packet.WriteByte(3);
            packet.WriteByte(0);
            packet.WriteByte(0);
            packet.WriteByte(0);
            packet.WriteString(String.Empty);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_CHANNEL_USER_LIST);
        }

        public static byte[] UserListItem(UserObject userobj, bool include_version)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteInt16(userobj.file_count);
            packet.WriteInt32(0);
            packet.WriteIP(Settings.allow_dc ? userobj.external_ip : IPAddress.Parse("0.0.0.0"));
            packet.WriteInt16(Settings.allow_dc ? userobj.dc_port : (ushort)0);
            packet.WriteIP(userobj.node_ip);
            packet.WriteInt16(userobj.node_port);
            packet.WriteByte(0);
            packet.WriteString(userobj.name);
            packet.WriteIP(Settings.allow_dc ? userobj.local_ip : IPAddress.Parse("0.0.0.0"));
            packet.WriteByte(userobj.browsable ? (byte)1 : (byte)0);
            packet.WriteByte((byte)userobj.level);

            if (include_version)
            {
                packet.WriteString(userobj.version);
                packet.WriteGuid(userobj.guid);
                packet.WriteByte(userobj.Muzzled ? (byte)1 : (byte)00);
            }

            packet.WriteByte(userobj.age);
            packet.WriteByte(userobj.sex);
            packet.WriteByte(userobj.country);
            packet.WriteString(userobj.region);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_CHANNEL_USER_LIST);
        }

        public static byte[] Topic(String text)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(text, false);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_TOPIC);
        }

        public static byte[] TopicFirst()
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(Settings.ServerTopic, false);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_TOPIC_FIRST);
        }

        public static byte[] TopicFirst(String text)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(text, false);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_TOPIC_FIRST);
        }

        public static byte[] UserListEnd()
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteByte(0);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_CHANNEL_USER_LIST_END);
        }

        public static byte[] NoSuch(String text)
        {
            AresPacketWriter packet = new AresPacketWriter();

            if (text.Length > 1024)
                text = text.Substring(0, 1024);

            packet.WriteString(text, false);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_NOSUCH);
        }

        public static byte[] MyFeatures(UserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(Settings.SERVER_VERSION + " [" + About.CALLISTO_VERSION + "]");
            packet.WriteByte(userobj.supports_compression ? (byte)7 : (byte)3);
            packet.WriteByte(63);
            packet.WriteByte(Helpers.ComboBoxLanguageToAresLanguage(Settings.ServerLanguage));
            packet.WriteInt32(userobj.cookie);
            packet.WriteByte(1);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_MYFEATURES);
        }

        public static byte[] Url(String addr, String tag)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(addr);
            packet.WriteString(tag);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_URL);
        }

        public static byte[] OpChange(UserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteByte(userobj.level > 0 ? (byte)1 : (byte)0);
            packet.WriteByte(0);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_OPCHANGE);
        }

        public static byte[] PersonalMessage(UserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.name);
            packet.WriteString(userobj.personal_message, false);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_PERSONAL_MESSAGE);
        }

        public static byte[] Avatar(UserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.name);
            packet.WriteBytes(userobj.avatar);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_AVATAR);
        }

        public static byte[] BotAvatar()
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(Settings.BotName);
            packet.WriteBytes(Settings.botav);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_AVATAR);
        }

        public static byte[] AvatarCleared(UserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.name);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_AVATAR);
        }

        public static byte[] EndOfBrowse(ushort id)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteInt16(id);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_ENDOFBROWSE);
        }

        public static byte[] BrowseError(ushort id)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteInt16(id);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_BROWSEERROR);
        }

        public static byte[] BrowseItem(ushort id, SharedFileObject file)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteInt16(id);
            packet.WriteByte((byte)file.type);
            packet.WriteInt32(file.size);
            packet.WriteBytes(file.content);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_BROWSEITEM);
        }

        public static byte[] StartOfBrowse(ushort id, ushort count)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteInt16(id);
            packet.WriteInt16(count);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_STARTOFBROWSE);
        }

        public static byte[] SearchHit(ushort id, UserObject userobj, SharedFileObject file)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteInt16(id);
            packet.WriteByte((byte)file.type);
            packet.WriteInt32(file.size);
            packet.WriteBytes(file.content);
            packet.WriteString(userobj.name);
            packet.WriteIP(userobj.external_ip);
            packet.WriteInt16(userobj.dc_port);
            packet.WriteIP(userobj.node_ip);
            packet.WriteInt16(userobj.node_port);
            packet.WriteIP(userobj.local_ip);
            packet.WriteByte(userobj.current_uploads);
            packet.WriteByte(userobj.max_uploads);
            packet.WriteByte(userobj.current_queued);
            packet.WriteByte(1);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_SEARCHHIT);
        }

        public static byte[] EndOfSearch(ushort id)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteInt16(id);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_ENDOFSEARCH);
        }

        public static byte[] ClientCompressed(byte[] data)
        {
            try
            {
                AresPacketWriter packet = new AresPacketWriter();
                packet.WriteBytes(Zip.Compress(data));
                return packet.ToAresPacket(ProtoMessage.MSG_CHAT_CLIENTCOMPRESSED);
            }
            catch { }

            return null;
        }

        public static byte[] SuperNodes(byte[] nodes)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteBytes(nodes);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_HERE_SUPERNODES);
        }

        public static byte[] DirectChatPush(UserObject userobj, byte[] text_synch)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.name);
            packet.WriteIP(userobj.external_ip);
            packet.WriteInt16(userobj.dc_port);
            packet.WriteIP(userobj.local_ip);
            packet.WriteBytes(text_synch);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_CLIENT_DIRCHATPUSH);
        }

        public static byte[] CustomData(UserObject userobj, String ident, byte[] data)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(ident);
            packet.WriteString(userobj.name);
            packet.WriteBytes(data);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_CUSTOM_DATA);
        }

        public static byte[] CustomData(String sender, String ident, byte[] data)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(ident);
            packet.WriteString(sender);
            packet.WriteBytes(data);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_CUSTOM_DATA);
        }

        public static byte[] CustomFont(UserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.name); // user's name + null
            packet.WriteByte((byte)userobj.font.Size); // limited to between 8 to 16
            packet.WriteString(userobj.font.Name); // null terminated
            packet.WriteByte(userobj.font.NameColor);
            packet.WriteByte(userobj.font.TextColor);

            if (userobj.font.NameColorEx != null && userobj.font.TextColorEx != null)
            {
                packet.WriteBytes(userobj.font.NameColorEx);
                packet.WriteBytes(userobj.font.TextColorEx);
            }

            byte[] buf = packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_CUSTOM_FONT); // id = 204
            packet = new AresPacketWriter();
            packet.WriteBytes(buf);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL);
        }

        public static byte[] CustomFont(AresFont font)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(String.Empty); // user's name + null
            packet.WriteByte((byte)font.Size); // limited to between 8 to 16
            packet.WriteString(font.Name); // null terminated
            byte[] buf = packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_CUSTOM_FONT); // id = 204
            packet = new AresPacketWriter();
            packet.WriteBytes(buf);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL);
        }

        public static byte[] SupportsVoiceClips()
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteByte(Settings.enable_vc ? (byte)1 : (byte)0);
            packet.WriteByte(Settings.enable_vc_hq ? (byte)1 : (byte)0);
            byte[] buf = packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_VC_SUPPORTED);
            packet = new AresPacketWriter();
            packet.WriteBytes(buf);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL);
        }

        public static byte[] VoiceChatFirst(String sender, byte[] buffer)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(sender);
            packet.WriteBytes(buffer);
            byte[] buf = packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_VC_FIRST);
            packet = new AresPacketWriter();
            packet.WriteBytes(buf);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL);
        }

        public static byte[] VoiceChatFirstTo(String sender, byte[] buffer)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(sender);
            packet.WriteBytes(buffer);
            byte[] buf = packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_VC_FIRST_FROM);
            packet = new AresPacketWriter();
            packet.WriteBytes(buf);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL);
        }

        public static byte[] VoiceChatChunk(String sender, byte[] buffer)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(sender);
            packet.WriteBytes(buffer);
            byte[] buf = packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_VC_CHUNK);
            packet = new AresPacketWriter();
            packet.WriteBytes(buf);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL);
        }

        public static byte[] VoiceChatChunkTo(String sender, byte[] buffer)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(sender);
            packet.WriteBytes(buffer);
            byte[] buf = packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_VC_CHUNK_FROM);
            packet = new AresPacketWriter();
            packet.WriteBytes(buf);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL);
        }

        public static byte[] VoiceChatIgnored(String sender)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(sender);
            byte[] buf = packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_VC_IGNORE);
            packet = new AresPacketWriter();
            packet.WriteBytes(buf);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL);
        }

        public static byte[] VoiceChatNoPrivate(String sender)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(sender);
            byte[] buf = packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_VC_NOPVT);
            packet = new AresPacketWriter();
            packet.WriteBytes(buf);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL);
        }

        public static byte[] VoiceChatUserSupport(UserObject userobj)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.name);
            packet.WriteByte(userobj.voice_chat ? (byte)1 : (byte)0);
            packet.WriteByte(userobj.voice_private_chat ? (byte)1 : (byte)0);
            byte[] buf = packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_VC_USER_SUPPORTED);
            packet = new AresPacketWriter();
            packet.WriteBytes(buf);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL);
        }

        public static byte[] SupportsCustomEmotes()
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteByte(16);
            byte[] buf = packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_SUPPORTS_CUSTOM_EMOTES);
            packet = new AresPacketWriter();
            packet.WriteBytes(buf);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL);
        }

        public static byte[] CustomEmoteItem(UserObject userobj, CustomEmoteItem item)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.name);
            packet.WriteString(item.Shortcut);
            packet.WriteByte(item.Size);
            packet.WriteBytes(item.Image);
            byte[] buf = packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_CUSTOM_EMOTES_ITEM);
            packet = new AresPacketWriter();
            packet.WriteBytes(buf);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL);
        }

        public static byte[] CustomEmoteDelete(UserObject userobj, String shortcut)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteString(userobj.name);
            packet.WriteString(shortcut);
            byte[] buf = packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_CUSTOM_EMOTE_DELETE);
            packet = new AresPacketWriter();
            packet.WriteBytes(buf);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL);
        }

        public static byte[] CanRoomScribble()
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteByte(Settings.allow_room_scribbles ? (byte)1 : (byte)0);
            byte[] buf = packet.ToAresPacket(ProtoMessage.MSG_CHAT_SERVER_ROOM_SCRIBBLE);
            packet = new AresPacketWriter();
            packet.WriteBytes(buf);
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL);
        }


        // encryption

        private static ProtoMessage[] encs = new ProtoMessage[]
        {
            ProtoMessage.MSG_CHAT_SERVER_PUBLIC,
            ProtoMessage.MSG_CHAT_SERVER_EMOTE,
            ProtoMessage.MSG_CHAT_SERVER_PVT,
            ProtoMessage.MSG_CHAT_SERVER_PERSONAL_MESSAGE,
            ProtoMessage.MSG_CHAT_SERVER_LOGIN_ACK,
            ProtoMessage.MSG_CHAT_SERVER_MYFEATURES,
            ProtoMessage.MSG_CHAT_SERVER_CHANNEL_USER_LIST,
            ProtoMessage.MSG_CHAT_SERVER_JOIN,
            ProtoMessage.MSG_CHAT_SERVER_PART,
            ProtoMessage.MSG_CHAT_SERVER_URL,
            ProtoMessage.MSG_CHAT_SERVER_UPDATE_USER_STATUS,
            ProtoMessage.MSG_CHAT_CLIENT_PUBLIC,
            ProtoMessage.MSG_CHAT_CLIENT_EMOTE,
            ProtoMessage.MSG_CHAT_CLIENT_PVT,
            ProtoMessage.MSG_CHAT_CLIENT_PERSONAL_MESSAGE,
            ProtoMessage.MSG_CHAT_CLIENT_UPDATE_STATUS,
            ProtoMessage.MSG_CHAT_CLIENT_COMMAND
        };

        public static byte[] Encrypt(byte[] data, byte fc, ushort sc)
        {
            if (encs.Contains((ProtoMessage)data[2]))
            {
                byte[] buf = new byte[data.Length - 3];
                Array.Copy(data, 3, buf, 0, buf.Length);
                buf = Crypto.Encrypt(fc, sc, buf);
                AresPacketWriter packet = new AresPacketWriter();
                packet.WriteBytes(buf);
                return packet.ToAresPacket((ProtoMessage)data[2]);
            }

            return data;
        }

        public static byte[] Decrypt(byte[] data, byte fc, ushort sc, ProtoMessage msg)
        {
            if (encs.Contains(msg))
                return Crypto.Decrypt(fc, sc, data);

            return data;
        }

        public static byte[] AckEnc()
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteBytes(new byte[] { 0, 0, 255 });
            return packet.ToAresPacket(ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL);
        }
    }
}
