﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using ICallisto;

namespace callisto
{
    class HashlinkHelpers
    {
        public static String EncodeHashlink(IChannelObject item)
        {
            AresPacketWriter packet = new AresPacketWriter();
            packet.WriteBytes(new byte[20]);
            packet.WriteString("CHATCHANNEL");
            packet.WriteIP(item.ExternalIP);
            packet.WriteInt16(item.Port);
            packet.WriteIP(item.LocalIP);
            packet.WriteString(item.Name);
            packet.WriteByte(0);
            byte[] bytes = packet.ToByteArray();
            bytes = Zip.Compress(bytes);
            bytes = e67(bytes, 28435);
            return Convert.ToBase64String(bytes);
        }

        public static ChannelObject DecodeHashlink(String item)
        {
            ChannelObject chitem = new ChannelObject();

            item = item.Trim();

            if (item.StartsWith("arlnk://"))
                item = item.Substring(8);

            try
            {
                if (item.ToUpper().StartsWith("CHATROOM:")) // not encrypted
                {
                    item = item.Substring(9);
                    int split = item.IndexOf(":");
                    chitem.external_ip = IPAddress.Parse(item.Substring(0, split));
                    item = item.Substring(split + 1);
                    split = item.IndexOf("|");
                    chitem.port = ushort.Parse(item.Substring(0, split));
                    chitem.name = item.Substring(split + 1);
                    chitem.topic = String.Empty;
                    return chitem;
                }
                else // encrypted
                {
                    byte[] bytes = Convert.FromBase64String(item);
                    bytes = d67(bytes, 28435);
                    bytes = Zip.Decompress(bytes);
                    AresPacketReader packet = new AresPacketReader(bytes);
                    packet.SkipBytes(32);
                    chitem.external_ip = packet.ReadIP();
                    chitem.port = packet.ReadInt16();
                    chitem.local_ip = packet.ReadIP();
                    chitem.name = packet.ReadString();
                    chitem.topic = String.Empty;
                    return chitem;
                }
            }
            catch // badly formed hashlink
            {
                return null;
            }
        }

        private static byte[] d67(byte[] data, int b)
        {
            byte[] buffer = new byte[data.Length];
            Array.Copy(data, buffer, data.Length);

            for (int i = 0; i < data.Length; i++)
            {
                buffer[i] = (byte)(data[i] ^ b >> 8 & 255);
                b = (b + data[i]) * 23219 + 36126 & 65535;
            }
            return buffer;
        }

        public static byte[] e67(byte[] data, int b)
        {
            byte[] buffer = new byte[data.Length];

            for (int i = 0; i < data.Length; i++)
            {
                buffer[i] = (byte)((data[i] ^ (b >> 8)) & 255);
                b = ((buffer[i] + b) * 23219 + 36126) & 65535;
            }

            return buffer;
        }
    }

    class ChannelObject : IChannelObject
    {
        public String name;
        public String topic;
        public IPAddress external_ip;
        public IPAddress local_ip;
        public ushort port;

        public String Name
        {
            get { return this.name; }
            set { this.name = value; }
        }

        public String Topic
        {
            get { return this.topic; }
            set { this.topic = value; }
        }

        public IPAddress ExternalIP
        {
            get { return this.external_ip; }
            set { this.external_ip = value; }
        }

        public IPAddress LocalIP
        {
            get { return this.local_ip; }
            set { this.local_ip = value; }
        }

        public ushort Port
        {
            get { return this.port; }
            set { this.port = value; }
        }
    }
}
