﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ICallisto;

namespace callisto
{
    class ServerEvents
    {
        public static CallistoPluginManager Plugins;

        public static bool OnJoinCheck(IUser userobj)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    bool result = p.Plugin.OnJoinCheck(userobj);

                    if (!result)
                    {
                        CallistoPluginManager.CurrentPlugin = cur_plugin;
                        return false;
                    }
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;

            return Scripting.ServerEvents.OnJoinCheck(userobj);
        }

        public static bool OnAvatarReceived(IUser userobj)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    bool result = p.Plugin.OnAvatarReceived(userobj);

                    if (!result)
                    {
                        CallistoPluginManager.CurrentPlugin = cur_plugin;
                        return false;
                    }
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;

            return Scripting.ServerEvents.OnAvatarReceived(userobj);
        }

        public static bool OnPersonalReceived(IUser userobj, String msg)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    bool result = p.Plugin.OnPersonalReceived(userobj, msg);

                    if (!result)
                    {
                        CallistoPluginManager.CurrentPlugin = cur_plugin;
                        return false;
                    }
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;

            return Scripting.ServerEvents.OnPersonalReceived(userobj, msg);
        }

        public static void OnRejected(IUser userobj, NativeRejectionType type)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;
            String msg = String.Empty;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnRejected(userobj, type, ref msg);
                }
                catch (Exception e) { OnError(p, e); }
            }

            Scripting.ServerEvents.OnRejected(userobj);

            if (!String.IsNullOrEmpty(msg))
                userobj.SendPrint(msg);

            CallistoPluginManager.CurrentPlugin = cur_plugin;
        }

        public static void OnJoin(IUser userobj)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnJoin(userobj);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            Scripting.ServerEvents.OnJoin(userobj);
        }

        public static void STT()
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.STT();
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
        }

        public static void OnPart(IUser userobj)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnPart(userobj);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            Scripting.ServerEvents.OnPart(userobj);
        }

        public static void OnTextReceived(IUser userobj, String text)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnTextReceived(userobj, text);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            Scripting.ServerEvents.OnTextReceived(userobj, text);
        }

        public static String OnTextBefore(IUser userobj, String text)
        {
            if (text.StartsWith("#login"))
                return String.Empty; // login will always be ignored

            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    text = p.Plugin.OnTextBefore(userobj, text);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            return Scripting.ServerEvents.OnTextBefore(userobj, text);
        }

        public static void OnTextAfter(IUser userobj, String text)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnTextAfter(userobj, text);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            Scripting.ServerEvents.OnTextAfter(userobj, text);
        }

        public static void OnEmoteReceived(IUser userobj, String text)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnEmoteReceived(userobj, text);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            Scripting.ServerEvents.OnEmoteReceived(userobj, text);
        }

        public static String OnEmoteBefore(IUser userobj, String text)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    text = p.Plugin.OnEmoteBefore(userobj, text);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            return Scripting.ServerEvents.OnEmoteBefore(userobj, text);
        }

        public static void OnEmoteAfter(IUser userobj, String text)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnEmoteAfter(userobj, text);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            Scripting.ServerEvents.OnEmoteAfter(userobj, text);
        }

        public static bool OnVroomJoinCheck(IUser userobj, ushort vroom)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    bool result = p.Plugin.OnVroomJoinCheck(userobj, vroom);

                    if (!result)
                    {
                        CallistoPluginManager.CurrentPlugin = cur_plugin;
                        return false;
                    }
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            return Scripting.ServerEvents.OnVroomJoinCheck(userobj, vroom);
        }

        public static void OnVroomJoin(IUser userobj)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnVroomJoin(userobj);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            Scripting.ServerEvents.OnVroomJoin(userobj);
        }

        public static void OnVroomPart(IUser userobj)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnVroomPart(userobj);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            Scripting.ServerEvents.OnVroomPart(userobj);
        }

        public static void OnCommand(IUser userobj, String command_text, IUser target, String args)
        {
            if (command_text == "help")
            {
                OnHelp(userobj);
                return;
            }

            if (command_text.StartsWith("login "))
            {
                String str = command_text.Substring(6);

                if (str.Length > 0)
                {
                    userobj.Level = PasswordManager.GetLevel(userobj, str); // AdminLevel.Regular if invalid password
                    OnLoginRequested(userobj);
                }

                return;
            }

            if (command_text == "version")
            {
                userobj.SendPrint("\x000314- Callisto Ares Chat Server");
                userobj.SendPrint("\x000314- Version " + About.CALLISTO_VERSION);
                return;
            }

            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnCommand(userobj, command_text, target, args);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            Scripting.ServerEvents.OnCommand(userobj, command_text, target, args);
        }

        public static void OnLoginRequested(IUser userobj)
        {
            if (userobj.Level == AdminLevel.Regular)
                CallistoPluginStats.invalid_login_attempts++;

            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnLoginRequested(userobj);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            Scripting.ServerEvents.OnLoginRequested(userobj);
        }

        public static void OnHelp(IUser userobj)
        {
            if (userobj is UserObject)
                ((UserObject)userobj).SendPacket(ChatPacketHelpers.NoSuch("#login <password>"));

            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                if (userobj is WebUser)
                    if (!((WebUser)userobj).InUse)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnHelp(userobj);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            Scripting.ServerEvents.OnHelp(userobj);
        }

        public static void OnFileReceived(IUser userobj, String filename, MimeMessage type)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnFileReceived(userobj, filename, type);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            Scripting.ServerEvents.OnFileReceived(userobj, filename, type);
        }

        public static bool OnFlood(IUser userobj, ProtoMessage msg)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    bool result = p.Plugin.OnFlood(userobj, msg);

                    if (!result)
                    {
                        CallistoPluginManager.CurrentPlugin = cur_plugin;
                        return false; // plugin says don't flood out!
                    }
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            return Scripting.ServerEvents.OnFlood(userobj, msg);
        }

        public static String OnPM(IUser userobj, IUser target, String text)
        {
            PMObject pm = new PMObject(text);
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    bool result = p.Plugin.OnPM(userobj, target, pm);

                    if (!result)
                    {
                        CallistoPluginManager.CurrentPlugin = cur_plugin;
                        return String.Empty;
                    }
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            
            bool script_result = Scripting.ServerEvents.OnPM(userobj, target, pm);

            if (!script_result)
                return String.Empty;

            return pm.GetText();
        }

        public static void OnBotPM(IUser userobj, String text)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                if (userobj is UserObject)
                    if (!((UserObject)userobj).Connected)
                        break;

                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnBotPM(userobj, text);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            Scripting.ServerEvents.OnBotPM(userobj, text);
        }

        public static void OnLinkJoin(ILink room)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnLinkJoin(room);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            Scripting.ServerEvents.OnLinkJoin(room);
        }

        public static void OnLinkPart(ILink room)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnLinkPart(room);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            Scripting.ServerEvents.OnLinkPart(room);
        }

        public static void OnLinkError(ILink room, LinkError code)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnLinkError(room, code);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            Scripting.ServerEvents.OnLinkError(room, code);
        }

        public static void OnLinkCommand(IUser userobj, String command, IUser target, String args)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnLinkCommand(userobj, command, target, args);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
            Scripting.ServerEvents.OnLinkCommand(userobj, command, target, args);
        }

        public static void OnTimer()
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnTimer();
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
        }

        public static void OnCaptcha(IUser userobj, CaptchaState state)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnCaptcha(userobj, state);
                }
                catch (Exception e) { OnError(p, e); }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
        }

        public static void OnError(CallistoPlugin plugin, Exception e)
        {
            CallistoPlugin[] plugins = Plugins.GetPlugins;
            String cur_plugin = CallistoPluginManager.CurrentPlugin;

            foreach (CallistoPlugin p in plugins)
            {
                CallistoPluginManager.CurrentPlugin = p.Name;

                try
                {
                    p.Plugin.OnError(plugin.Name, e);
                }
                catch { }
            }

            CallistoPluginManager.CurrentPlugin = cur_plugin;
        }

    }
}
