﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.IO;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using ICallisto;

namespace callisto
{
    internal class UserObject : IUser
    {
        internal int captcha_attempts = 0;
        internal int captcha_answer = 0;
        internal bool has_captched = false;


        internal Socket socket;
        internal uint spawn_time;
        internal uint last_update_time;
        internal int receive_fails;
        internal List<SharedFileObject> files = new List<SharedFileObject>();
        internal List<byte> link_data_in = new List<byte>();
        internal List<byte> data_in = new List<byte>();
        internal Queue<byte[]> data_out = new Queue<byte[]>();
        private List<AresPacket> formatted_packets = new List<AresPacket>();
        private List<String> recent_posts = new List<String>();
        private uint last_packet_time = 0;
        private uint packet_counter_main = 0;
        private uint packet_counter_pm = 0;
        private uint packet_counter_misc = 0;

        internal IPAddress external_ip;
        internal ushort id;
        internal uint cookie;
        internal List<String> ignores = new List<String>();
        internal CallistoCustomDataManager custom_data_manager = new CallistoCustomDataManager();
        internal bool local_host;
        internal bool browsable;
        internal AdminLevel level;
        internal byte country;
        internal byte sex;
        internal byte age;
        internal ushort dc_port;
        internal ushort file_count;
        internal ushort vroom;
        internal IPAddress local_ip;
        internal Guid guid;
        internal String region;
        internal String personal_message;
        internal String version;
        internal String name;
        internal String org_name;
        internal byte[] avatar;
        internal byte[] orgAvatar;
        internal byte current_uploads;
        internal byte max_uploads;
        internal byte current_queued;
        internal bool supports_compression;
        internal bool logged_in;
        internal IPAddress node_ip;
        internal ushort node_port;
        internal List<String> custom_tags = new List<String>();
        internal bool cloaked;
        internal AresFont font;
        internal bool fastping;
        internal uint last_fastping = 0;
        internal bool voice_chat = false;
        internal bool voice_private_chat = false;
        internal List<String> voice_chat_ignores = new List<String>();

        internal bool supports_custom_emotes = false;
        internal List<CustomEmoteItem> custom_emotes = new List<CustomEmoteItem>();
        internal String custom_name = String.Empty;
        internal bool muz = false;

        #region INTERFACE_CONFORM
        public bool FastPing { get { return this.fastping; } }

        public bool Muzzled
        {
            get { return this.muz; }
            set
            {
                this.muz = value;

                if (CallistoServer.Linker.IsLinking)
                    CallistoServer.Linker.SendPacket(LinkPacketHelpers.SendMuzzleUpdate(this));
            }
        }

        public String CustomName
        {
            get { return this.custom_name; }
            set
            {
                this.custom_name = value;

                if (this.custom_name == null)
                    this.custom_name = String.Empty;

                if (CallistoServer.Linker.IsLinking)
                    CallistoServer.Linker.SendPacket(LinkPacketHelpers.SendCustomNameText(this));
            }
        }

        public IAresFont Font
        {
            get { return this.font; }
            set
            {
                this.font = new AresFont();
                this.font.Name = value.Name;
                this.font.Size = value.Size;
                UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.CustomFont(this));
            }
        }

        public bool Cloaked
        {
            get { return this.cloaked; }
            set
            {
                if (this.cloaked == value || !this.logged_in)
                    return;

                if (value)
                {
                    if (this.level == AdminLevel.Regular)
                        return;

                    if (CallistoServer.Linker.IsLinking)
                        return;

                    this.cloaked = true;
                    UserPool.BroadcastToLower(this, ChatPacketHelpers.Part(this));
                    WebPool.BroadcastAresPart(this);
                }
                else
                {
                    this.cloaked = false;
                    UserPool.BroadcastToLower(this, ChatPacketHelpers.Join(this));
                    WebPool.BroadcastAresJoin(this);

                    if (this.avatar != null)
                        UserPool.BroadcastToLower(this, ChatPacketHelpers.Avatar(this));

                    if (!String.IsNullOrEmpty(this.personal_message))
                        UserPool.BroadcastToLower(this, ChatPacketHelpers.PersonalMessage(this));

                    if (this.font != null)
                    {
                        UserPool.BroadcastToLower(this, ChatPacketHelpers.CustomFont(this));
                        WebPool.BroadcastAresFont(this);
                    }

                    if (this.supports_custom_emotes)
                        foreach (CustomEmoteItem c in this.custom_emotes)
                            UserPool.BroadcastToLowerCustomEmotes(this, ChatPacketHelpers.CustomEmoteItem(this, c));
                }
            }
        }

        public String[] CustomClientTags { get { return this.custom_tags.ToArray(); } }

        public String[] IgnoreList { get { return this.ignores.ToArray(); } }

        public Dictionary<String, object> CustomData { get { return this.custom_data_manager.GetData; } }

        public bool IsLocalHost { get { return this.local_host; } }

        public bool Browsable { get { return this.browsable; } }

        public void SendCustomClientData(String ident, String sender, byte[] data)
        {
            this.SendPacket(ChatPacketHelpers.CustomData(sender, ident, data));
        }

        public AdminLevel Level
        {
            get { return this.level; }
            set
            {
                if (this.level == value)
                    return;

                if (this.IsLocalHost && value != AdminLevel.Host)
                    return;

                if (this.logged_in)
                {
                    if (this.cloaked) // is there anyone who can/can't see me now my level has changed?
                    {
                        switch (value)
                        {
                            case AdminLevel.Host: // want to be host
                                if (this.level == AdminLevel.Moderator) // hide from other levels
                                {
                                    UserPool.BroadcastToLevel(this, ChatPacketHelpers.Part(this), AdminLevel.Moderator);
                                    UserPool.BroadcastToLevel(this, ChatPacketHelpers.Part(this), AdminLevel.Administrator);
                                }
                                else // hide from admins
                                    UserPool.BroadcastToLevel(this, ChatPacketHelpers.Part(this), AdminLevel.Administrator);
                                break;

                            case AdminLevel.Administrator: // want to be admin
                                if (this.level == AdminLevel.Moderator) // hide from mods
                                    UserPool.BroadcastToLevel(this, ChatPacketHelpers.Part(this), AdminLevel.Moderator);
                                else // show to admins
                                    this.SendJoinPacketsToLevel(AdminLevel.Administrator);
                                break;

                            case AdminLevel.Moderator:
                                if (this.level == AdminLevel.Administrator) // hide from mods
                                {
                                    UserPool.BroadcastToLevel(this, ChatPacketHelpers.Join(this), AdminLevel.Moderator);

                                    if (this.font != null)
                                        UserPool.BroadcastToLevel(this, ChatPacketHelpers.CustomFont(this), AdminLevel.Moderator);

                                    if (this.voice_chat || this.voice_private_chat)
                                        UserPool.BroadcastToLevel(this, ChatPacketHelpers.VoiceChatUserSupport(this), AdminLevel.Moderator);

                                    if (this.supports_custom_emotes)
                                        foreach (CustomEmoteItem c in this.custom_emotes)
                                            UserPool.BroadcastToLevelCustomEmotes(this, ChatPacketHelpers.CustomEmoteItem(this, c), AdminLevel.Moderator);
                                }
                                else // hide from mods and admin
                                {
                                    this.SendJoinPacketsToLevel(AdminLevel.Moderator);
                                    this.SendJoinPacketsToLevel(AdminLevel.Administrator);
                                }
                                break;

                            case AdminLevel.Regular:
                                this.Cloaked = false;
                                break;
                        }
                    }

                    switch (this.level) // is there anyone i can/can't see now that my level changed?
                    {
                        case AdminLevel.Regular:
                            if (value == AdminLevel.Host)
                                this.ShowMeCloakedJoinPacketsOfLevels(AdminLevel.Moderator, AdminLevel.Host);
                            else if (value == AdminLevel.Administrator)
                                this.ShowMeCloakedJoinPacketsOfLevels(AdminLevel.Moderator, AdminLevel.Administrator);
                            else this.ShowMeCloakedJoinPacketsOfLevels(AdminLevel.Moderator, AdminLevel.Moderator);
                            break;

                        case AdminLevel.Moderator:
                            if (value == AdminLevel.Regular)
                                this.ShowMeCloakedPartPacketOfLevels(AdminLevel.Moderator, AdminLevel.Moderator);
                            else if (value == AdminLevel.Administrator)
                                this.ShowMeCloakedJoinPacketsOfLevels(AdminLevel.Administrator, AdminLevel.Administrator);
                            else this.ShowMeCloakedJoinPacketsOfLevels(AdminLevel.Administrator, AdminLevel.Host);
                            break;

                        case AdminLevel.Administrator:
                            if (value == AdminLevel.Regular)
                                this.ShowMeCloakedPartPacketOfLevels(AdminLevel.Moderator, AdminLevel.Administrator);
                            else if (value == AdminLevel.Moderator)
                                this.ShowMeCloakedPartPacketOfLevels(AdminLevel.Administrator, AdminLevel.Administrator);
                            else this.ShowMeCloakedJoinPacketsOfLevels(AdminLevel.Host, AdminLevel.Host);
                            break;

                        case AdminLevel.Host:
                            if (value == AdminLevel.Administrator)
                                this.ShowMeCloakedPartPacketOfLevels(AdminLevel.Host, AdminLevel.Host);
                            else if (value == AdminLevel.Moderator)
                                this.ShowMeCloakedPartPacketOfLevels(AdminLevel.Administrator, AdminLevel.Host);
                            else this.ShowMeCloakedPartPacketOfLevels(AdminLevel.Moderator, AdminLevel.Host);
                            break;
                    }
                }

                this.level = value;

                if (this.logged_in)
                {
                    if (this.cloaked)
                        UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.UpdateUserStatus(this));
                    else
                    {
                        UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.UpdateUserStatus(this));
                        WebPool.BroadcastAresUpdate(this);
                    }

                    this.SendPacket(ChatPacketHelpers.OpChange(this));

                    if (CallistoServer.Linker.IsLinking && this.vroom == 0)
                        CallistoServer.Linker.SendPacket(ChatPacketHelpers.UpdateUserStatus(this));
                }
            }
        }

        private void ShowMeCloakedJoinPacketsOfLevels(AdminLevel from, AdminLevel to)
        {
            foreach (UserObject u in UserPool.Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == this.vroom)
                            if (u.level >= from && u.level <= to)
                                if (u.id != this.id)
                                    if (u.cloaked)
                                        this.ShowMeCloakedJoinPacketsOfUser(u);
        }

        private void ShowMeCloakedPartPacketOfLevels(AdminLevel from, AdminLevel to)
        {
            foreach (UserObject u in UserPool.Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == this.vroom)
                            if (u.level >= from && u.level <= to)
                                if (u.id != this.id)
                                    if (u.cloaked)
                                        this.SendPacket(ChatPacketHelpers.Part(u));
        }

        private void ShowMeCloakedJoinPacketsOfUser(UserObject u)
        {
            this.SendPacket(ChatPacketHelpers.Join(u));

            if (u.font != null)
                this.SendPacket(ChatPacketHelpers.CustomFont(u));

            if (u.avatar != null)
                this.SendPacket(ChatPacketHelpers.Avatar(u));

            if (!String.IsNullOrEmpty(u.personal_message))
                this.SendPacket(ChatPacketHelpers.PersonalMessage(u));

            if (u.voice_chat || u.voice_private_chat)
                this.SendPacket(ChatPacketHelpers.VoiceChatUserSupport(u));

            if (u.supports_custom_emotes)
                foreach (CustomEmoteItem c in u.custom_emotes)
                    this.SendPacket(ChatPacketHelpers.CustomEmoteItem(u, c));
        }

        private void SendJoinPacketsToLevel(AdminLevel level)
        {
            UserPool.BroadcastToLevel(this, ChatPacketHelpers.Join(this), level);

            if (this.font != null)
                UserPool.BroadcastToLevel(this, ChatPacketHelpers.CustomFont(this), level);

            if (this.avatar != null)
                UserPool.BroadcastToLevel(this, ChatPacketHelpers.Avatar(this), level);

            if (!String.IsNullOrEmpty(this.personal_message))
                UserPool.BroadcastToLevel(this, ChatPacketHelpers.PersonalMessage(this), level);

            if (this.voice_chat || this.voice_private_chat)
                UserPool.BroadcastToLevel(this, ChatPacketHelpers.VoiceChatUserSupport(this), level);

            if (this.supports_custom_emotes)
                foreach (CustomEmoteItem c in this.custom_emotes)
                    UserPool.BroadcastToLevelCustomEmotes(this, ChatPacketHelpers.CustomEmoteItem(this, c), level);
        }

        public bool IsLinked { get { return false; } }

        public byte Country { get { return this.country; } }

        public byte Sex
        {
            get { return this.sex; }
            set
            {
                if (value > 99)
                    return;

                if (!this.logged_in) // not logged in yet (changing vroom at OnJoinCheck)
                {
                    this.sex = value;
                    return;
                }

                this.logged_in = false;
                LinkedUserObject lCheck = null;

                if (CallistoServer.Linker.IsLinking)
                {
                    lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == this.name);

                    if (this.vroom == 0) // tell linked room that user has parted
                        CallistoServer.Linker.SendPacket(ChatPacketHelpers.Part(this));
                }

                if (lCheck != null && this.vroom == 0)
                {
                    lCheck.visible = true;
                    UserPool.BroadcastToVroom(0, LinkPacketHelpers.Update(lCheck));
                    WebPool.BroadcastAresUpdate(lCheck);
                }
                else
                {
                    if (this.cloaked)
                        UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.Part(this));
                    else
                    {
                        UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Part(this));
                        WebPool.BroadcastAresPart(this);
                    }
                }

                this.sex = value;

                lCheck = null;

                if (CallistoServer.Linker.IsLinking)
                {
                    lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == this.name);

                    if (this.vroom == 0)
                    {
                        CallistoServer.Linker.SendPacket(ChatPacketHelpers.Join(this, true));

                        if (this.CustomName.Length > 0)
                            CallistoServer.Linker.SendPacket(LinkPacketHelpers.SendCustomNameText(this));
                    }
                }

                if (lCheck != null && this.vroom == 0) // linked user exists with new name
                {
                    lCheck.visible = false;
                    UserPool.BroadcastToVroom(0, ChatPacketHelpers.UpdateUserStatus(this));
                    WebPool.BroadcastAresUpdate(this);
                }
                else
                {
                    if (this.cloaked)
                    {
                        UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.Join(this));

                        if (this.font != null)
                            UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.CustomFont(this));

                        if (this.voice_chat || this.voice_private_chat)
                            UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.VoiceChatUserSupport(this));
                    }
                    else
                    {
                        UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Join(this));
                        WebPool.BroadcastAresJoin(this);

                        if (this.font != null)
                        {
                            UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.CustomFont(this));
                            WebPool.BroadcastAresFont(this);
                        }

                        if (this.voice_chat || this.voice_private_chat)
                            UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.VoiceChatUserSupport(this));
                    }
                }

                this.logged_in = true;
                this.SendPacket(ChatPacketHelpers.LoginAck(this));
                this.supports_custom_emotes = false;

                if (Settings.allow_custom_emotes)
                    this.SendPacket(ChatPacketHelpers.SupportsCustomEmotes());

                UserPool.SendUserlistToUser(this);
                UserPool.SendAvatars(this);
                UserPool.SendCustomFonts(this);
                UserPool.ShowWhoSupportsVC(this);

                if (this.avatar != null)
                {
                    if (this.cloaked)
                        UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.Avatar(this));
                    else
                        UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Avatar(this));

                    if (CallistoServer.Linker.IsLinking)
                        CallistoServer.Linker.SendPacket(ChatPacketHelpers.Avatar(this));
                }
                if (!String.IsNullOrEmpty(this.personal_message))
                {
                    if (this.cloaked)
                        UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.PersonalMessage(this));
                    else
                        UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.PersonalMessage(this));

                    if (CallistoServer.Linker.IsLinking)
                        CallistoServer.Linker.SendPacket(ChatPacketHelpers.Avatar(this));
                }

                if (CallistoServer.Linker.IsLinking)
                    if (this.vroom == 0 && this.custom_tags.Count > 0)
                        CallistoServer.Linker.SendPacket(LinkPacketHelpers.SetCustomTags(this));
            }
        }

        public byte Age
        {
            get { return this.age; }
            set
            {
                if (value > 99)
                    return;

                if (!this.logged_in) // not logged in yet (changing vroom at OnJoinCheck)
                {
                    this.age = value;
                    return;
                }

                this.logged_in = false;
                LinkedUserObject lCheck = null;

                if (CallistoServer.Linker.IsLinking)
                {
                    lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == this.name);

                    if (this.vroom == 0) // tell linked room that user has parted
                        CallistoServer.Linker.SendPacket(ChatPacketHelpers.Part(this));
                }

                if (lCheck != null && this.vroom == 0)
                {
                    lCheck.visible = true;
                    UserPool.BroadcastToVroom(0, LinkPacketHelpers.Update(lCheck));
                    WebPool.BroadcastAresUpdate(lCheck);
                }
                else
                {
                    if (this.cloaked)
                        UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.Part(this));
                    else
                    {
                        UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Part(this));
                        WebPool.BroadcastAresPart(this);
                    }
                }

                this.age = value;

                lCheck = null;

                if (CallistoServer.Linker.IsLinking)
                {
                    lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == this.name);

                    if (this.vroom == 0)
                    {
                        CallistoServer.Linker.SendPacket(ChatPacketHelpers.Join(this, true));

                        if (this.CustomName.Length > 0)
                            CallistoServer.Linker.SendPacket(LinkPacketHelpers.SendCustomNameText(this));
                    }
                }

                if (lCheck != null && this.vroom == 0) // linked user exists with new name
                {
                    lCheck.visible = false;
                    UserPool.BroadcastToVroom(0, ChatPacketHelpers.UpdateUserStatus(this));
                    WebPool.BroadcastAresUpdate(this);
                }
                else
                {
                    if (this.cloaked)
                    {
                        UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.Join(this));

                        if (this.font != null)
                            UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.CustomFont(this));

                        if (this.voice_chat || this.voice_private_chat)
                            UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.VoiceChatUserSupport(this));
                    }
                    else
                    {
                        UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Join(this));
                        WebPool.BroadcastAresJoin(this);

                        if (this.font != null)
                        {
                            UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.CustomFont(this));
                            WebPool.BroadcastAresFont(this);
                        }

                        if (this.voice_chat || this.voice_private_chat)
                            UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.VoiceChatUserSupport(this));
                    }
                }

                this.logged_in = true;
                this.SendPacket(ChatPacketHelpers.LoginAck(this));
                this.supports_custom_emotes = false;

                if (Settings.allow_custom_emotes)
                    this.SendPacket(ChatPacketHelpers.SupportsCustomEmotes());

                UserPool.SendUserlistToUser(this);
                UserPool.SendAvatars(this);
                UserPool.SendCustomFonts(this);
                UserPool.ShowWhoSupportsVC(this);

                if (this.avatar != null)
                {
                    if (this.cloaked)
                        UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.Avatar(this));
                    else
                        UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Avatar(this));

                    if (CallistoServer.Linker.IsLinking)
                        CallistoServer.Linker.SendPacket(ChatPacketHelpers.Avatar(this));
                }
                if (!String.IsNullOrEmpty(this.personal_message))
                {
                    if (this.cloaked)
                        UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.PersonalMessage(this));
                    else
                        UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.PersonalMessage(this));

                    if (CallistoServer.Linker.IsLinking)
                        CallistoServer.Linker.SendPacket(ChatPacketHelpers.Avatar(this));
                }

                if (CallistoServer.Linker.IsLinking)
                    if (this.vroom == 0 && this.custom_tags.Count > 0)
                        CallistoServer.Linker.SendPacket(LinkPacketHelpers.SetCustomTags(this));
            }
        }

        public ushort Port { get { return this.dc_port; } }

        public ushort FileCount { get { return this.file_count; } }

        public int Id { get { return this.id; } }

        public ushort Vroom
        {
            get { return this.vroom; }
            set
            {
                if (!this.logged_in) // not logged in yet (changing vroom at OnJoinCheck)
                {
                    this.vroom = value;
                    return;
                }

                if (ServerEvents.OnVroomJoinCheck(this, value))
                {
                    ServerEvents.OnVroomPart(this);
                    this.logged_in = false;
                    LinkedUserObject lCheck = null;

                    if (CallistoServer.Linker.IsLinking)
                    {
                        lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == this.name);

                        if (this.vroom == 0) // tell linked room that user has parted
                            CallistoServer.Linker.SendPacket(ChatPacketHelpers.Part(this));
                    }

                    if (lCheck != null && this.vroom == 0)
                    {
                        lCheck.visible = true;
                        UserPool.BroadcastToVroom(0, LinkPacketHelpers.Update(lCheck));
                        WebPool.BroadcastAresUpdate(lCheck);
                    }
                    else
                    {
                        if (this.cloaked)
                            UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.Part(this));
                        else
                        {
                            UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Part(this));
                            WebPool.BroadcastAresPart(this);
                        }
                    }

                    this.vroom = value;

                    lCheck = null;

                    if (CallistoServer.Linker.IsLinking)
                    {
                        lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == this.name);

                        if (this.vroom == 0)
                        {
                            CallistoServer.Linker.SendPacket(ChatPacketHelpers.Join(this, true));

                            if (this.CustomName.Length > 0)
                                CallistoServer.Linker.SendPacket(LinkPacketHelpers.SendCustomNameText(this));
                        }
                    }

                    if (lCheck != null && this.vroom == 0) // linked user exists with new name
                    {
                        lCheck.visible = false;
                        UserPool.BroadcastToVroom(0, ChatPacketHelpers.UpdateUserStatus(this));
                        WebPool.BroadcastAresUpdate(this);
                    }
                    else
                    {
                        if (this.cloaked)
                        {
                            UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.Join(this));

                            if (this.font != null)
                                UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.CustomFont(this));

                            if (this.voice_chat || this.voice_private_chat)
                                UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.VoiceChatUserSupport(this));
                        }
                        else
                        {
                            UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Join(this));
                            WebPool.BroadcastAresJoin(this);

                            if (this.font != null)
                            {
                                UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.CustomFont(this));
                                WebPool.BroadcastAresFont(this);
                            }

                            if (this.voice_chat || this.voice_private_chat)
                                UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.VoiceChatUserSupport(this));
                        }
                    }

                    this.logged_in = true;
                    this.SendPacket(ChatPacketHelpers.LoginAck(this));
                    this.supports_custom_emotes = false;

                    if (Settings.allow_custom_emotes)
                        this.SendPacket(ChatPacketHelpers.SupportsCustomEmotes());

                    UserPool.SendUserlistToUser(this);
                    UserPool.SendAvatars(this);
                    UserPool.SendCustomFonts(this);
                    UserPool.ShowWhoSupportsVC(this);

                    if (this.avatar != null)
                    {
                        if (this.cloaked)
                            UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.Avatar(this));
                        else
                            UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Avatar(this));

                        if (CallistoServer.Linker.IsLinking)
                            CallistoServer.Linker.SendPacket(ChatPacketHelpers.Avatar(this));
                    }
                    if (!String.IsNullOrEmpty(this.personal_message))
                    {
                        if (this.cloaked)
                            UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.PersonalMessage(this));
                        else
                            UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.PersonalMessage(this));

                        if (CallistoServer.Linker.IsLinking)
                            CallistoServer.Linker.SendPacket(ChatPacketHelpers.Avatar(this));
                    }

                    if (CallistoServer.Linker.IsLinking)
                        if (this.vroom == 0 && this.custom_tags.Count > 0)
                            CallistoServer.Linker.SendPacket(LinkPacketHelpers.SetCustomTags(this));

                    ServerEvents.OnVroomJoin(this);
                }
            }
        }

        public IPAddress LocalIP { get { return this.local_ip; } }

        public IPAddress ExternalIP { get { return this.external_ip; } }

        public String Guid { get { return Convert.ToBase64String(this.guid.ToByteArray()); } }

        public String Region { get { return this.region; } }

        public String PersonalMessage
        {
            get { return this.personal_message; }
            set
            {
                this.personal_message = value;

                while (Encoding.UTF8.GetByteCount(this.personal_message) > 50)
                    this.personal_message = this.personal_message.Substring(0, this.personal_message.Length - 1);

                if (this.cloaked)
                    UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.PersonalMessage(this));
                else
                    UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.PersonalMessage(this));

                if (CallistoServer.Linker.IsLinking && this.vroom == 0)
                    CallistoServer.Linker.SendPacket(ChatPacketHelpers.PersonalMessage(this));
            }
        }

        public byte[] Avatar
        {
            get { return this.avatar; }
            set
            {
                this.avatar = value;

                if (this.avatar != null)
                {
                    if (this.cloaked)
                        UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.Avatar(this));
                    else
                        UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Avatar(this));

                    if (CallistoServer.Linker.IsLinking && this.vroom == 0)
                        CallistoServer.Linker.SendPacket(ChatPacketHelpers.Avatar(this));
                }
                else
                {
                    if (this.cloaked)
                        UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.AvatarCleared(this));
                    else
                        UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.AvatarCleared(this));

                    if (CallistoServer.Linker.IsLinking && this.vroom == 0)
                        CallistoServer.Linker.SendPacket(ChatPacketHelpers.AvatarCleared(this));
                }
            }
        }

        public void DownloadAvatar(String avatar_address)
        {
            if (this.downloading_avatar)
                return;

            Thread thread = new Thread(new ParameterizedThreadStart(this.DownloadAvatarWorker));
            thread.Start(avatar_address);
        }

        private bool downloading_avatar = false;

        private void DownloadAvatarWorker(object args)
        {
            if (this.downloading_avatar)
                return;

            this.downloading_avatar = true;
            String avatar_address = (String)args;

            try
            {
                WebRequest request = WebRequest.Create(avatar_address);
                request.Timeout = 5000;
                WebResponse response = request.GetResponse();
                List<byte> avatar_in = new List<byte>();
                int avatar_received = 0;
                byte[] avatar_buffer = new byte[1024];

                using (Stream stream = response.GetResponseStream())
                    while ((avatar_received = stream.Read(avatar_buffer, 0, 1024)) > 0)
                        avatar_in.AddRange(avatar_buffer.Take(avatar_received));

                response.Close();

                // format picture to 48x48 ares avatar dimensions

                using (Bitmap avatar_raw = new Bitmap(new MemoryStream(avatar_in.ToArray())))
                {
                    using (Bitmap avatar_sized = new Bitmap(48, 48))
                    {
                        using (Graphics g = Graphics.FromImage(avatar_sized))
                        {
                            g.InterpolationMode = InterpolationMode.HighQualityBicubic;
                            g.DrawImage(avatar_raw, new RectangleF(0, 0, 48, 48));
                            
                            using (MemoryStream ms = new MemoryStream())
                            {
                                avatar_sized.Save(ms, ImageFormat.Jpeg);
                                avatar_buffer = ms.ToArray();
                            }
                        }
                    }
                }
                // clean up

                avatar_in.Clear();

                if (this.Connected) // maybe user disconnected whilst the avatar was downloading
                    if (this.logged_in)
                        this.Avatar = avatar_buffer;
            }
            catch { }

            this.downloading_avatar = false;
        }

        public void RestoreAvatar()
        {
            this.Avatar = this.orgAvatar;
        }

        public String Version { get { return this.version; } }

        public IUser NextUser
        {
            get
            {
                foreach (UserObject u in UserPool.Pool)
                    if (u.Connected)
                        if (u.logged_in)
                            if (u.id > this.id)
                                return u;

                return WebPool.FirstUser;
            }
        }

        public String Name
        {
            get { return this.name; }
            set
            {
                if (!this.logged_in) // not logged in yet (changing name at OnJoinCheck)
                {
                    Helpers.FormatUsername(this, value);
                    return;
                }

                this.logged_in = false;
                LinkedUserObject lCheck = null;

                if (CallistoServer.Linker.IsLinking)
                {
                    lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == this.name);

                    if (this.vroom == 0)
                        CallistoServer.Linker.SendPacket(ChatPacketHelpers.Part(this));
                }

                if (lCheck != null && this.vroom == 0) // linked user exists with old name
                {
                    lCheck.visible = true;
                    UserPool.BroadcastToVroom(0, LinkPacketHelpers.Update(lCheck));
                    WebPool.BroadcastAresUpdate(lCheck);
                }
                else
                {
                    if (this.cloaked)
                        UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.Part(this));
                    else
                    {
                        UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Part(this));
                        WebPool.BroadcastAresPart(this);
                    }
                }

                Helpers.FormatUsername(this, value);

                lCheck = null;

                if (CallistoServer.Linker.IsLinking)
                {
                    lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == this.name);

                    if (this.vroom == 0)
                    {
                        CallistoServer.Linker.SendPacket(ChatPacketHelpers.Join(this, true));

                        if (this.CustomName.Length > 0)
                            CallistoServer.Linker.SendPacket(LinkPacketHelpers.SendCustomNameText(this));
                    }
                }

                if (lCheck != null && this.vroom == 0) // linked user exists with new name
                {
                    lCheck.visible = false;
                    UserPool.BroadcastToVroom(0, ChatPacketHelpers.UpdateUserStatus(this));
                    WebPool.BroadcastAresUpdate(this);
                }
                else
                {
                    if (this.cloaked)
                    {
                        UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.Join(this));

                        if (this.font != null)
                            UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.CustomFont(this));

                        if (this.voice_chat || this.voice_private_chat)
                            UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.VoiceChatUserSupport(this));
                    }
                    else
                    {
                        UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Join(this));
                        WebPool.BroadcastAresJoin(this);

                        if (this.font != null)
                        {
                            UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.CustomFont(this));
                            WebPool.BroadcastAresFont(this);
                        }

                        if (this.voice_chat || this.voice_private_chat)
                            UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.VoiceChatUserSupport(this));
                    }
                }

                this.logged_in = true;
                this.SendPacket(ChatPacketHelpers.LoginAck(this));
                this.supports_custom_emotes = false;

                if (Settings.allow_custom_emotes)
                    this.SendPacket(ChatPacketHelpers.SupportsCustomEmotes());

                UserPool.SendUserlistToUser(this);
                UserPool.SendAvatars(this);
                UserPool.SendCustomFonts(this);
                UserPool.ShowWhoSupportsVC(this);

                if (this.avatar != null)
                {
                    if (this.cloaked)
                        UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.Avatar(this));
                    else
                        UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Avatar(this));

                    if (CallistoServer.Linker.IsLinking)
                        CallistoServer.Linker.SendPacket(ChatPacketHelpers.Avatar(this));
                }

                if (!String.IsNullOrEmpty(this.personal_message))
                {
                    if (this.cloaked)
                        UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.PersonalMessage(this));
                    else
                        UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.PersonalMessage(this));

                    if (CallistoServer.Linker.IsLinking)
                        CallistoServer.Linker.SendPacket(ChatPacketHelpers.PersonalMessage(this));
                }

                if (CallistoServer.Linker.IsLinking)
                    if (this.vroom == 0 && this.custom_tags.Count > 0)
                        CallistoServer.Linker.SendPacket(LinkPacketHelpers.SetCustomTags(this));
            }
        }

        public String OrgName { get { return this.org_name; } }

        public void SendPM(String name, String text)
        {
            this.SendPacket(ChatPacketHelpers.Private(name, text));
        }

        public void SendPluginPM(String sender, String text) // forced PM does not account for admin sharing being disabled
        {
            this.SendPacket(ChatPacketHelpers.Private(sender, text));
        }

        public void SendEmote(String text)
        {
            UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Emote(this.name, text));
            WebPool.BroadcastEmote(this, text);

            if (this.vroom == 0)
                if (CallistoServer.Linker.IsLinking)
                    CallistoServer.Linker.SendPacket(ChatPacketHelpers.Emote(this.name, text));
        }

        public void SendText(String text)
        {
            if (Settings.CAN_CUSTOM_NAME && this.custom_name.Length > 0)
            {
                UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.NoSuch(this.custom_name + text));
                WebPool.BroadcastCustomNamePublic(this, text);
            }
            else
            {
                UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Public(this.name, text));
                WebPool.BroadcastPublic(this, text);
            }

            if (this.vroom == 0)
                if (CallistoServer.Linker.IsLinking)
                    CallistoServer.Linker.SendPacket(ChatPacketHelpers.Public(this.name, text));
        }

        public void SendPrint(String text)
        {
            this.SendPacket(ChatPacketHelpers.NoSuch(text));
        }

        public void Redirect(String hashlink)
        {
            ChannelObject chan = HashlinkHelpers.DecodeHashlink(hashlink);

            if (chan != null)
            {
                this.SendPacket(ChatPacketHelpers.Redirect(chan));
                this.Disconnect();
            }
        }

        public void UpdateURL(String address, String text)
        {
            this.SendPacket(ChatPacketHelpers.Url(address, text));
        }

        public void Ban()
        {
            if (!this.local_host)
            {
                BanManager.AddBan(this);
                this.Disconnect();
            }
        }

        public void Disconnect()
        {
            this.Disconnect(false);
        }

        public void Disconnect(bool ghost)
        {
            if (ghost)
                if (this.cloaked)
                    this.Cloaked = false;

            if (this.socket != null)
            {
                this.DispatchOutbound();
                try { this.socket.Disconnect(false); }
                catch { }
                try { this.socket.Close(); }
                catch { }
            }

            this.socket = null;

            if (this.logged_in && !ghost)
            {
                CallistoPluginStats.part_count++;
                LinkedUserObject lCheck = null;

                if (CallistoServer.Linker.IsLinking)
                    lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == this.name);

                this.logged_in = false;
                ServerEvents.OnPart(this);

                if (lCheck != null && this.vroom == 0)
                {
                    lCheck.visible = true;
                    UserPool.BroadcastToVroom(this.vroom, LinkPacketHelpers.Update(lCheck));
                    WebPool.BroadcastAresUpdate(lCheck);
                }
                else
                {
                    if (this.cloaked)
                        UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.Part(this));
                    else
                    {
                        UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Part(this));
                        WebPool.BroadcastAresPart(this);
                    }
                }

                UserPool.RemoveIgnoredUser(this);

                if (this.vroom == 0 && CallistoServer.Linker.IsLinking)
                    CallistoServer.Linker.SendPacket(ChatPacketHelpers.Part(this));
            }
            else Scripting.ScriptManager.RemoveUser(this);

            this.data_in.Clear();
            this.data_out.Clear();
            this.formatted_packets.Clear();
            this.link_data_in.Clear();
            this.ignores.Clear();
            this.files.Clear();
        }

        public void ShowUserParting()
        {
            if (this.logged_in)
            {
                CallistoPluginStats.part_count++;
                LinkedUserObject lCheck = null;

                if (CallistoServer.Linker.IsLinking)
                    lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == this.name);

                this.logged_in = false;

                if (lCheck != null && this.vroom == 0)
                {
                    lCheck.visible = true;
                    UserPool.BroadcastToVroom(this.vroom, LinkPacketHelpers.UserListItem(lCheck));
                    WebPool.BroadcastAresUpdate(lCheck);
                }
                else
                {
                    if (this.cloaked)
                        UserPool.BroadcastToEqualOrHigher(this, ChatPacketHelpers.Part(this));
                    else
                    {
                        UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Part(this));
                        WebPool.BroadcastAresPart(this);
                    }
                }

                ServerEvents.OnPart(this);
                UserPool.RemoveIgnoredUser(this);

                if (this.vroom == 0 && CallistoServer.Linker.IsLinking)
                    CallistoServer.Linker.SendPacket(ChatPacketHelpers.Part(this));
            }
        }

        public void Destroy()
        {
            if (this.socket != null)
            {
                this.DispatchOutbound();
                try { this.socket.Disconnect(false); }
                catch { }
                try { this.socket.Close(); }
                catch { }
            }

            this.socket = null;
            this.data_in.Clear();
            this.data_out.Clear();
            this.formatted_packets.Clear();
            this.link_data_in.Clear();
            this.ignores.Clear();
            this.files.Clear();
            this.custom_emotes.Clear();
        }

        public void SetTopic(String text)
        {
            this.SendPacket(ChatPacketHelpers.TopicFirst(text));
        }
        #endregion

        internal UserObject() { this.IsHTML5 = false; }

        internal UserObject(Socket socket, ushort id, uint time)
        {
            this.id = id;
            this.socket = socket;
            this.socket.Blocking = false;
            this.external_ip = ((IPEndPoint)socket.RemoteEndPoint).Address;
            this.cookie = time;
            this.spawn_time = time;
            this.last_update_time = time;
            this.last_packet_time = time;

            if (Settings.local_auto_login)
                this.local_host = Helpers.IsLocalHost(this.external_ip);
            else
                this.local_host = false;

            if (this.local_host)
                this.external_ip = Settings.ServerIP;

            this.logged_in = false;
            this.receive_fails = 0;
            this.vroom = 0;
            this.personal_message = String.Empty;
            this.level = this.local_host ? AdminLevel.Host : AdminLevel.Regular;
            this.avatar = null;
            this.fastping = false;
            this.IsHTML5 = false;
        }

        /// <summary>Get the user object occupation status</summary>
        internal bool Connected
        {
            get { return this.socket != null; }
        }

        /// <summary>Add a packet for this user to receive</summary>
        internal void SendPacket(byte[] data)
        {
            this.data_out.Enqueue(data);
        }

        /// <summary>Send all pending outbound data for this user</summary>
        internal void DispatchOutbound()
        {
            while (this.data_out.Count > 0)
            {
                try
                {
                    byte[] buffer = this.data_out.Peek();

                    if (this.EncMode == EncryptionMode.Encrypted)
                        buffer = ChatPacketHelpers.Encrypt(buffer, this.EncByte, this.EncWord);

                    this.socket.Send(buffer);
                    CallistoPluginStats.data_sent += (ulong)this.data_out.Dequeue().Length;
                }
                catch { return; } // would block
            }
        }

        internal bool IsHTML5 { get; set; }

        /// <summary>receive all pending inbound data for this user</summary>
        internal void ReceiveInbound()
        {
            int receive_count = 0;
            byte[] buffer1 = new byte[8192];
            SocketError e = SocketError.Success;

            try
            {
                int avail = this.socket.Available;

                if (avail > 8192)
                    avail = 8192;

                receive_count = this.socket.Receive(buffer1, 0, avail, SocketFlags.None, out e);
            }
            catch { } // would block

            if (receive_count == 0)
            {
                if (e == SocketError.WouldBlock)
                    this.receive_fails = 0;
                else
                    this.receive_fails++;
            }
            else
            {
                CallistoPluginStats.data_received += (ulong)receive_count;
                byte[] buffer2 = new byte[receive_count];
                Array.Copy(buffer1, buffer2, receive_count);
                this.data_in.AddRange(buffer2);
                this.receive_fails = 0;
            }

            while (this.data_in.Count >= 3)
            {
                if (!this.logged_in)
                {
                    String test = Encoding.Default.GetString(this.data_in.ToArray(), 0, 3);

                    if (test.ToUpper() == "GET")
                    {
                        this.IsHTML5 = true;
                        break;
                    }
                }

                AresPacket packet = new AresPacket();
                packet.Size = BitConverter.ToUInt16(this.data_in.ToArray(), 0);
                packet.Msg = (ProtoMessage)this.data_in[2];

                if (this.data_in.Count >= (packet.Size + 3))
                {
                    this.data_in.RemoveRange(0, 3); // no longer need the header
                    packet.Packet = this.data_in.GetRange(0, packet.Size).ToArray();
                    this.data_in.RemoveRange(0, packet.Size);

                    if (packet.Msg == ProtoMessage.MSG_CHAT_CLIENTCOMPRESSED) // convert into packet array
                    {
                        try
                        {
                            AresPacket[] zip_packets = Helpers.GetPacketsFromZip(packet.Packet);
                            this.formatted_packets.AddRange(zip_packets);
                        }
                        catch { }

                        break;
                    }
                    else { this.formatted_packets.Add(packet); } // add to the heap
                }
                else { break; }
            }
        }

        /// <summary>Get the next available packet received from this user</summary>
        internal AresPacket NextPacket
        {
            get
            {
                if (this.formatted_packets.Count == 0)
                    return null;

                AresPacket packet = this.formatted_packets[0];
                this.formatted_packets.RemoveAt(0);
                return packet;
            }
        }

        public EncryptionMode EncMode = EncryptionMode.Unencrypted;
        public byte EncByte { get; set; }
        public ushort EncWord { get; set; }

        internal void PopulateCredentials(AresPacketReader packet)
        {
            this.guid = packet.ReadGuid();
            this.file_count = packet.ReadInt16();
            this.EncMode = packet.ReadByte() == 255 ? EncryptionMode.Encrypted : EncryptionMode.Unencrypted;

            if (this.EncMode == EncryptionMode.Encrypted)
            {
                byte[] buf = packet.ToArray();
                this.EncByte = (byte)buf.Length;
                this.EncWord = 0;

                for (int i = 0; i < buf.Length; i++)
                    this.EncWord += buf[i];
            }

            this.dc_port = packet.ReadInt16();
            this.node_ip = packet.ReadIP();
            this.node_port = packet.ReadInt16();
            packet.SkipBytes(4); // don't use line speed anymore
            this.name = packet.ReadString();
            Helpers.FormatUsername(this);
            this.org_name = this.name;
            this.version = packet.ReadString();
            this.local_ip = packet.ReadIP();
            packet.SkipBytes(4); // don't need external ip coz we got the REAL one from the socket
            byte browse_flag = packet.ReadByte();
            this.browsable = browse_flag > 2;
            this.supports_compression = browse_flag > 3;

            if (!this.browsable)
                this.file_count = 0; // no point pretending you have files if you're not browsable

            this.current_uploads = packet.ReadByte();
            this.max_uploads = packet.ReadByte();
            this.current_queued = packet.ReadByte();

            if (packet.Remaining() > 3)
            {
                this.age = packet.ReadByte();
                this.sex = packet.ReadByte();
                this.country = packet.ReadByte();
                this.region = packet.ReadString();

                if (this.region.Length > 30)
                    this.region = this.region.Substring(0, 30);
            }
            else // user has not updated to a recent ares
            {
                this.age = 0;
                this.sex = 0;
                this.country = 0;
                this.region = String.Empty;
            }

            if (!Settings.allow_filesharing)
                this.browsable = false;
        }

        internal void Login(bool ghost, bool fastpinger)
        {
            if (!ghost) // anti-bounce for ghosts
            {
                CallistoPluginStats.join_count++;

                if (UserPool.UserCount > CallistoPluginStats.peak_user_count)
                    CallistoPluginStats.peak_user_count = UserPool.UserCount;

                LinkedUserObject lCheck = null;

                if (CallistoServer.Linker.IsLinking)
                    lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == this.name);

                if (lCheck != null && this.vroom == 0) // user already exists as a linked user
                {
                    lCheck.visible = false;
                    UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.UpdateUserStatus(this));
                    WebPool.BroadcastAresUpdate(this);
                }
                else
                {
                    UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Join(this));
                    WebPool.BroadcastAresJoin(this);
                }
            }
            else this.custom_tags.Add("CALLISTO_GHOST");

            this.logged_in = true;
            this.fastping = fastpinger;

            if (this.EncMode == EncryptionMode.Encrypted)
                this.SendPacket(ChatPacketHelpers.AckEnc());

            this.SendPacket(ChatPacketHelpers.LoginAck(this));
            this.SendPacket(ChatPacketHelpers.MyFeatures(this));
            this.SendPacket(ChatPacketHelpers.CanRoomScribble());

            if (!Settings.allow_dc)
                this.SendPacket(new byte[] { 0, 0, 215 }); // no ips

            this.SendPacket(ChatPacketHelpers.SupportsVoiceClips());
            this.SendPacket(ChatPacketHelpers.TopicFirst());
            UserPool.SendUserlistToUser(this);
            this.SendPacket(ChatPacketHelpers.OpChange(this));

            if (!String.IsNullOrEmpty(Settings.Url[0]) && !String.IsNullOrEmpty(Settings.Url[1]))
                this.SendPacket(ChatPacketHelpers.Url(Settings.Url[0], Settings.Url[1]));

            this.supports_custom_emotes = false;

            if (Settings.allow_custom_emotes)
                this.SendPacket(ChatPacketHelpers.SupportsCustomEmotes());

            UserPool.SendAvatars(this);
            UserPool.SendCustomFonts(this);
            UserPool.ShowWhoSupportsVC(this);
        }

        internal bool IsFlooding(ProtoMessage msg, byte[] data, uint time)
        {
            if (msg == ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL)
                return false;

            if (msg == ProtoMessage.MSG_CHAT_CLIENT_PUBLIC || msg == ProtoMessage.MSG_CHAT_CLIENT_EMOTE)
            {
                if (Helpers.IsTextFlood(this)) return true;
                if (this.level > 0 || this.local_host) return false;

                this.recent_posts.Insert(0, Encoding.UTF8.GetString(data));

                if (this.recent_posts.Count == 5)
                {
                    this.recent_posts.RemoveAt(4);
                    return this.recent_posts.TrueForAll(str => str == this.recent_posts[0]);
                }
            }

            if (this.level > 0 || this.local_host) return false;

            if (msg == ProtoMessage.MSG_CHAT_CLIENT_ADDSHARE ||
                msg == ProtoMessage.MSG_CHAT_CLIENTCOMPRESSED ||
                msg == ProtoMessage.MSG_CHAT_CLIENT_FASTPING)
                return false;

            if (time > (this.last_packet_time + 1))
            {
                this.last_packet_time = time;
                this.packet_counter_main = 0;
                this.packet_counter_misc = 0;
                this.packet_counter_pm = 0;
                return false;
            }
            else
            {
                if (msg == ProtoMessage.MSG_CHAT_CLIENT_PUBLIC || msg == ProtoMessage.MSG_CHAT_CLIENT_EMOTE) return ++this.packet_counter_main > 3;
                if (msg == ProtoMessage.MSG_CHAT_CLIENT_PVT) return ++this.packet_counter_pm > 7;
                return ++this.packet_counter_misc > 8;
            }
        }

        public override bool Equals(object obj)
        {
            IUser user = (IUser)obj;
            return this.Guid.Equals(user.Guid) || this.ExternalIP.Equals(user.ExternalIP);
        }

        public override int GetHashCode() { return base.GetHashCode(); }
    }

    class CustomEmoteItem
    {
        public String Shortcut;
        public byte Size;
        public byte[] Image;
    }
}
