﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using ICallisto;

namespace callisto
{
    class UserPool
    {
        public static UserObject[] Pool;

        /// <summary>Initialize the user pool</summary>
        public static void CreateUserPool()
        {
            UserRecordManager.items.Clear();

            Pool = new UserObject[700];

            for (int i = 0; i < Pool.Length; i++)
                Pool[i] = new UserObject();
        }

        /// <summary>Get next available pool index</summary>
        public static int NextAvailable
        {
            get
            {
                for (int i = 0; i < Pool.Length; i++)
                    if (!Pool[i].Connected)
                        return i;

                return -1;
            }
        }

        /// <summary>Get number of users with an ip address</summary>
        public static int ClientCountByIP(IPAddress ip)
        {
            int counter = 0;

            foreach (UserObject u in UserPool.Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.external_ip.Equals(ip))
                            counter++;

            return counter;
        }

        /// <summary>Get status of desired name</summary>
        public static bool NameExists(IUser userobj)
        {
            if (userobj.Name.ToUpper() == "CALLISTO") // reserve that name
                return true;

            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.id != userobj.Id)
                            if (u.name == userobj.Name)
                                return true;

            foreach (WebUser u in WebPool.Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.name == userobj.Name)
                            return true;

            return false;
        }

        /// <summary>Get ghosted object of this user</summary>
        public static UserObject FindGhost(IUser userobj)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.id != userobj.Id)
                            if (u.name == userobj.Name)
                                if (u.Guid.Equals(userobj.Guid) || u.external_ip.Equals(userobj.ExternalIP))
                                    return u;

            return null;
        }

        public static void UpdateFastPingers(uint time)
        {
            foreach (UserObject u in Pool)
            {
                if (u.Connected)
                {
                    if (u.logged_in)
                    {
                        if (u.fastping)
                        {
                            if ((u.last_fastping + 10) < time)
                            {
                                u.last_fastping = time;
                                u.SendPacket(ChatPacketHelpers.FastPing());
                            }
                        }
                    }
                }
            }
        }

        public static void SendCustomEmotes(UserObject userobj)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.vroom == userobj.vroom)
                        if (u.supports_custom_emotes)
                            foreach (CustomEmoteItem c in u.custom_emotes)
                                userobj.SendPacket(ChatPacketHelpers.CustomEmoteItem(u, c));
        }

        /// <summary>Send this user all the avatars received from users in a vroom</summary>
        public static void SendAvatars(UserObject userobj)
        {
            if (Settings.botav != null)
                userobj.SendPacket(ChatPacketHelpers.BotAvatar());

            foreach (UserObject u in Pool)
            {
                if (u.Connected)
                {
                    if (u.logged_in)
                    {
                        if (u.vroom == userobj.vroom)
                        {
                            if (!String.IsNullOrEmpty(u.personal_message))
                                userobj.SendPacket(ChatPacketHelpers.PersonalMessage(u));

                            if (u.avatar != null)
                                userobj.SendPacket(ChatPacketHelpers.Avatar(u));
                        }
                    }
                }
            }

            if (CallistoServer.Linker.IsLinking && userobj.vroom == 0)
            {
                foreach (LinkedUserObject u in CallistoServer.Linker.linked_users)
                {
                    if (u.visible)
                    {
                        if (!String.IsNullOrEmpty(u.personal_message))
                            userobj.SendPacket(LinkPacketHelpers.PersonalMessage(u));

                        if (u.avatar != null)
                            userobj.SendPacket(LinkPacketHelpers.Avatar(u));
                    }
                }
            }

            foreach (WebUser u in WebPool.Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Vroom == userobj.vroom)
                        {
                            userobj.SendPacket(WebPackets.AresAvatar(u));
                            userobj.SendPacket(WebPackets.AresPersonalMessage(u));
                        }
        }

        /// <summary>Send this user all the custom fonts submitted so far</summary>
        public static void SendCustomFonts(UserObject userobj)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == userobj.vroom)
                            if (u.font != null)
                                userobj.SendPacket(ChatPacketHelpers.CustomFont(u));
        }

        /// <summary>Send avatars to linked room</summary>
        public static void SendAvatarsToLink(LinkClient room)
        {
            foreach (UserObject u in Pool)
            {
                if (u.Connected)
                {
                    if (u.logged_in)
                    {
                        if (u.vroom == 0)
                        {
                            if (!String.IsNullOrEmpty(u.personal_message))
                                room.SendPacket(ChatPacketHelpers.PersonalMessage(u));

                            if (u.avatar != null)
                                room.SendPacket(ChatPacketHelpers.Avatar(u));
                        }
                    }
                }
            }

            foreach (WebUser u in WebPool.Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Vroom == 0)
                        {
                            room.SendPacket(WebPackets.AresAvatar(u));
                            room.SendPacket(WebPackets.AresPersonalMessage(u));
                        }
        }

        /// <summary>Send the userlist to this user</summary>
        public static void SendUserlistToUser(UserObject userobj)
        {
            userobj.SendPacket(ChatPacketHelpers.UserListBotItem());

            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == userobj.vroom)
                            if (!u.cloaked || userobj.level >= u.level)
                                userobj.SendPacket(ChatPacketHelpers.UserListItem(u));

            if (CallistoServer.Linker.IsLinking && userobj.vroom == 0)
                foreach (LinkedUserObject u in CallistoServer.Linker.linked_users)
                    if (u.visible)
                        userobj.SendPacket(LinkPacketHelpers.UserListItem(u));

            foreach (WebUser u in WebPool.Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Vroom == userobj.vroom)
                            userobj.SendPacket(WebPackets.AresUserListItem(u));

            userobj.SendPacket(ChatPacketHelpers.UserListEnd());
        }

        /// <summary>Send the userlist to link</summary>
        public static void SendUserlistToLink(LinkClient room)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == 0)
                            room.SendPacket(ChatPacketHelpers.UserListItem(u, true));

            foreach (WebUser u in WebPool.Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Vroom == 0)
                            room.SendPacket(WebPackets.AresUserListItem(u, true));

            room.SendPacket(ChatPacketHelpers.UserListEnd());
        }

        public static void SendCustomNamesToLink(LinkClient room)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == 0)
                            if (u.CustomName.Length > 0)
                                room.SendPacket(LinkPacketHelpers.SendCustomNameText(u));
        }

        /// <summary>Send the custom tags to link</summary>
        public static void SendCustomTagsToLink(LinkClient room)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == 0)
                            if (u.custom_tags.Count > 0)
                                room.SendPacket(LinkPacketHelpers.SetCustomTags(u));
        }

        /// <summary>Send packet to everyone</summary>
        public static void BroadcastToAll(byte[] packet)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        u.SendPacket(packet);
        }

        /// <summary>Send packet to admins</summary>
        public static void BroadcastToAdmins(byte[] packet)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.level > 0)
                            u.SendPacket(packet);
        }

        /// <summary>Send packet to admins</summary>
        public static void BroadcastToAdmins(ushort vroom, byte[] packet)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.level > 0)
                            if (u.vroom == vroom)
                                u.SendPacket(packet);
        }

        /// <summary>Send packet to regular users</summary>
        public static void BroadcastToRegs(ushort vroom, byte[] packet)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == vroom)
                            if (u.level == 0)
                                u.SendPacket(packet);
        }

        /// <summary>Send packet to equal or higher level users</summary>
        public static void BroadcastToEqualOrHigher(UserObject userobj, byte[] packet)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == userobj.vroom)
                            if (u.level >= userobj.level)
                                u.SendPacket(packet);
        }

        public static void BroadcastToEqualOrHigherCustomEmotes(UserObject userobj, byte[] packet)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == userobj.vroom)
                            if (u.level >= userobj.level)
                                if (u.supports_custom_emotes)
                                    u.SendPacket(packet);
        }

        /// <summary>Send packet to lower level users</summary>
        public static void BroadcastToLower(UserObject userobj, byte[] packet)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == userobj.vroom)
                            if (u.level < userobj.level)
                                u.SendPacket(packet);
        }

        public static void BroadcastToLowerCustomEmotes(UserObject userobj, byte[] packet)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == userobj.vroom)
                            if (u.level < userobj.level)
                                if (u.supports_custom_emotes)
                                    u.SendPacket(packet);
        }

        /// <summary>Send packet to level users</summary>
        public static void BroadcastToLevel(UserObject userobj, byte[] packet, AdminLevel level)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == userobj.vroom)
                            if (u.level == level)
                                u.SendPacket(packet);
        }

        public static void BroadcastToLevelCustomEmotes(UserObject userobj, byte[] packet, AdminLevel level)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == userobj.vroom)
                            if (u.level == level)
                                if (u.supports_custom_emotes)
                                    u.SendPacket(packet);
        }

        public static void UncloakAll()
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.cloaked)
                            u.Cloaked = false;
        }

        /// <summary>Send packet to everyone in a vroom</summary>
        public static void BroadcastToVroom(ushort vroom, byte[] packet)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == vroom)
                            u.SendPacket(packet);
        }

        public static void BroadcastToVroomCustomEmotes(ushort vroom, byte[] packet)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == vroom)
                            if (u.supports_custom_emotes)
                                u.SendPacket(packet);
        }

        /// <summary>Send packet to everyone who the sender is not ignored by in a vroom</summary>
        public static void BroadcastToUnignored(IUser sender, byte[] packet)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == sender.Vroom)
                            if (!u.ignores.Contains(sender.Name))
                                u.SendPacket(packet);
        }

        /// <summary>Send packet to everyone who the linked sender is not ignored by</summary>
        public static void BroadcastToUnignored(LinkedUserObject sender, byte[] packet)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == 0)
                            if (!u.ignores.Contains(sender.name))
                                u.SendPacket(packet);
        }

        /// <summary>Send packet to everyone apart from the sender</summary>
        public static void BroadcastCustomData(UserObject sender, byte[] packet)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.id != sender.id)
                            if (!u.version.StartsWith("Ares "))
                                u.SendPacket(packet);
        }

        /// <summary>Send voice clip packet to everyone apart from the sender</summary>
        public static void BroadcastVoiceClip(UserObject sender, byte[] packet)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.id != sender.id)
                            if (u.voice_chat)
                                if (!u.voice_chat_ignores.Contains(sender.name))
                                    u.SendPacket(packet);
        }

        public static void ShowWhoSupportsVC(UserObject userobj)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == userobj.vroom)
                            if (u.voice_chat || u.voice_private_chat)
                                userobj.SendPacket(ChatPacketHelpers.VoiceChatUserSupport(u));
        }

        /// <summary>Get a user object by user name</summary>
        public static UserObject FindUserByName(String name)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.name == name)
                            return u;

            return null;
        }

        /// <summary>Get a user object by user name in a specific vroom</summary>
        public static UserObject FindUserByName(String name, ushort vroom)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == vroom)
                            if (u.name == name)
                                return u;

            return null;
        }

        /// <summary>Get a user object by user name substring</summary>
        public static UserObject FindUserByNameStartsWith(String name)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.name.StartsWith(name))
                            return u;

            return null;
        }

        /// <summary>Get a user object by id</summary>
        public static UserObject FindUserByID(ushort id)
        {
            if (id >= Pool.Length)
                return null;

            if (Pool[id].Connected)
                if (Pool[id].logged_in)
                    return Pool[id];

            return null;
        }

        /// <summary>Remove this user from everyone's ignore lists</summary>
        public static void RemoveIgnoredUser(UserObject userobj)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.ignores.Contains(userobj.name))
                            u.ignores.Remove(userobj.name);
        }

        /// <summary>Remove this name from everyone's ignore lists</summary>
        public static void RemoveIgnoredUser(String name)
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.ignores.Contains(name))
                            u.ignores.Remove(name);
        }

        /// <summary>Disconnect every user currently connected to the server</summary>
        public static void KillUserPool()
        {
            foreach (UserObject u in Pool)
                if (u.Connected)
                    u.Destroy();
        }

        /// <summary>Get a collection of super nodes from various users</summary>
        public static byte[] GetSuperNodes()
        {
            List<ushort> available_ids = new List<ushort>();

            foreach (UserObject u in Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.version.Length == 15)
                            if (u.version.StartsWith("Ares") && u.version[10].ToString() == ".") // genuine ares client
                                if (!u.node_ip.Equals(System.Net.IPAddress.Parse("0.0.0.0")))
                                    if (u.node_port != 0)
                                        available_ids.Add((ushort)u.id);

            if (available_ids.Count == 0) // unlikely, but no nodes available :-(
                return new byte[] { };

            if (available_ids.Count > 20) // trim
            {
                Random rnd = new Random(Environment.TickCount);

                while (available_ids.Count > 20)
                    available_ids.RemoveAt((int)Math.Floor(rnd.NextDouble() * available_ids.Count));
            }

            List<byte> supernodes = new List<byte>();

            foreach (ushort i in available_ids)
            {
                supernodes.AddRange(Pool[i].node_ip.GetAddressBytes());
                supernodes.AddRange(BitConverter.GetBytes(Pool[i].node_port));
            }

            return supernodes.ToArray();
        }

        /// <summary>Get the user count</summary>
        public static ushort UserCount
        {
            get
            {
                ushort counter = 0;

                for (int i = 0; i < Pool.Length; i++)
                    if (Pool[i].Connected)
                        //if (Pool[i].logged_in)
                            counter++;

                foreach (WebUser u in WebPool.Pool)
                    if (u.InUse)
                        //if (u.LoggedIn)
                            counter++;

                return counter;
            }
        }

        /// <summary>Get the user count for linking</summary>
        public static ushort UserCountForLinking
        {
            get
            {
                ushort counter = 0;

                for (int i = 0; i < Pool.Length; i++)
                    if (Pool[i].Connected)
                        if (Pool[i].logged_in)
                            if (Pool[i].vroom == 0)
                                counter++;

                foreach (WebUser u in WebPool.Pool)
                    if (u.InUse)
                        if (u.LoggedIn)
                            if (u.Vroom == 0)
                                counter++;

                return counter;
            }
        }

        /// <summary>Update local hosts</summary>
        public static void UpdateLocalHosts()
        {
            if (!Settings.local_auto_login)
                return;

            foreach (UserObject u in Pool)
            {
                if (u.Connected)
                {
                    if (u.logged_in)
                    {
                        if (Helpers.IsLocalHost(u.external_ip))
                        {
                            u.external_ip = Settings.ServerIP;
                            u.local_host = true;
                            u.level = AdminLevel.Host;
                            BroadcastToVroom(u.vroom, ChatPacketHelpers.UpdateUserStatus(u));
                            u.SendPacket(ChatPacketHelpers.OpChange(u));

                            if (CallistoServer.Linker.IsLinking)
                                CallistoServer.Linker.SendPacket(ChatPacketHelpers.UpdateUserStatus(u));
                        }
                    }
                }
            }
        }

    }
}
