﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Net;
using System.Xml;

namespace callisto
{
    class TrustedServers
    {
        public static List<TrustedServerItems> Items;

        public static void Load()
        {
            Items = new List<TrustedServerItems>();

            try
            {
                FileStream f = new FileStream(Settings.Path + "trustedservers.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("trustedservers");

                while (xml.ReadToFollowing("item"))
                {
                    TrustedServerItems a = new TrustedServerItems();

                    xml.ReadSubtree().ReadToFollowing("Name");
                    a.Name = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("IP");
                    a.IP = new IPAddress(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("Port");
                    a.Port = xml.ReadElementContentAsInt();

                    Items.Add(a);
                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }
        }

        public static void Save()
        {
            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(Settings.Path + "trustedservers.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("trustedservers");

                foreach (TrustedServerItems a in Items)
                {
                    xml.WriteStartElement("item");
                    xml.WriteElementString("Name", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.Name)));
                    xml.WriteElementString("IP", Convert.ToBase64String(a.IP.GetAddressBytes()));
                    xml.WriteElementString("Port", a.Port.ToString());
                    xml.WriteEndElement();
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }
        }
    }

    class TrustedServerItems
    {
        public String Name;
        public IPAddress IP;
        public int Port;

        public TrustedServerItems() { }

        public TrustedServerItems(ChannelObject c)
        {
            this.Name = c.name;
            this.IP = c.external_ip;
            this.Port = c.port;
        }

        public override bool Equals(object obj)
        {
            return this.Name == ((TrustedServerItems)obj).Name;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }
}
