﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;

namespace callisto
{
    class UdpListener
    {
        private Queue<UdpDatagram> datagrams_out;

        private List<UdpFirewallObject> firewall_tests; // we test these people's firewalls
        private List<IPAddress> firewall_checkers; // people checking our firewall
        private int firewall_checkers_index; // position in server list to ask for next check request

        private Socket socket;

        private uint timer_one_second;
        private uint timer_one_minute;
        private uint timer_fifteen_minutes;

        private bool received_server_ip = false;
        private bool finished_testing_my_firewall = false;

        public void Start()
        {
            UdpStats.Reset();
            this.datagrams_out = new Queue<UdpDatagram>();
            this.firewall_tests = new List<UdpFirewallObject>();
            this.firewall_checkers = new List<IPAddress>();
            this.firewall_checkers_index = 0;
            uint time = Helpers.UnixTime;
            this.socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
            this.socket.Blocking = false;
            this.socket.Bind(new IPEndPoint(Settings.udp_ip, Settings.ServerPort));
            this.timer_one_second = time;
            this.timer_one_minute = time;
            this.timer_fifteen_minutes = time;
            this.received_server_ip = false;
            this.finished_testing_my_firewall = false;
        }

        public void Stop()
        {
            try { this.socket.Shutdown(SocketShutdown.Both); }
            catch { }
            try { this.socket.Close(); }
            catch { }
        }

        public void DoTasks(uint time)
        {
            this.DispatchOutbound();
            this.ReceiveInbound();

            if (time > this.timer_one_second)
            {
                this.timer_one_second = time;
                this.ExpireFirewallTests(time);                

                if (!this.finished_testing_my_firewall)
                    this.CheckMyFirewall();
                else
                    this.Push(time);
            }

            if (time > (this.timer_one_minute + 60))
            {
                this.timer_one_minute = time;
                UdpServerPool.ExpireList();

                if (!this.finished_testing_my_firewall) // a minute is long enough for the firewall test
                    this.finished_testing_my_firewall = true;
            }

            if (time > (this.timer_fifteen_minutes + 900)) // udp save and report
            {
                this.timer_fifteen_minutes = time;
                UdpServerPool.Save();
            }
        }

        public void ExpireFirewallTests(uint time)
        {
            UdpFirewallObject firewall;

            while ((firewall = this.firewall_tests.Find(f => (f.spawn + 10) < time)) != null)
            {
                firewall.Dispose();
                this.firewall_tests.Remove(firewall);
            }
        }

        public bool IsFirewall(Socket s)
        {
            if (this.finished_testing_my_firewall)
                return false;

            IPAddress ip = ((IPEndPoint)s.RemoteEndPoint).Address;

            if (this.firewall_checkers.Contains(ip))
            {
                try { s.Disconnect(false); }
                catch { }
                try { s.Shutdown(SocketShutdown.Both); }
                catch { }
                try { s.Close(); }
                catch { }

                this.finished_testing_my_firewall = true;
                this.firewall_checkers.Clear();
                return true;
            }
            
            return false;
        }

        private void CheckMyFirewall()
        {
            if (this.firewall_checkers_index < UdpServerPool.Pool.Count)
            {
                UdpServerObject server = UdpServerPool.Pool[this.firewall_checkers_index++];
                UdpDatagram datagram = new UdpDatagram();
                datagram.Address = new IPEndPoint(server.ip, server.port);
                datagram.Msg = UdpMessage.OP_SERVERLIST_WANTCHECKFIREWALL;
                datagram.Packet = UdpPacketHelpers.WantCheckFirewall();
                this.datagrams_out.Enqueue(datagram);
            }
        }

        private void Push(uint time)
        {
            if (!Settings.ShouldList)
                return;

            UdpServerPool.SortMethod = UdpSortMethod.SortOldestContactedFirst;
            UdpServerPool.Pool.Sort();

            if (UdpServerPool.Pool.Count > 0)
            {
                if ((UdpServerPool.Pool[0].last_sent_add_IPS + 900) < time)
                {
                    UdpServerPool.Pool[0].last_sent_add_IPS = (int)time;
                    UdpServerPool.Pool[0].try_count++;
                    UdpDatagram datagram = new UdpDatagram();
                    datagram.Address = new IPEndPoint(UdpServerPool.Pool[0].ip, UdpServerPool.Pool[0].port);
                    datagram.Msg = UdpMessage.OP_SERVERLIST_ADDIPS;
                    datagram.Packet = UdpPacketHelpers.AddIps(UdpServerPool.Pool[0].ip);
                    this.datagrams_out.Enqueue(datagram);
                    ChannelListUpdateArgs.ServerRequests++;
                }
            }
        }

        private void DispatchOutbound()
        {
            while (this.datagrams_out.Count > 0)
            {
                UdpDatagram datagram = this.datagrams_out.Peek();

                try
                {
                    int size = this.socket.SendTo(datagram.Packet, datagram.Address);
                    this.datagrams_out.Dequeue();

                    if (size == datagram.Packet.Length)
                    {
                        if (datagram.Msg == UdpMessage.OP_SERVERLIST_ACKINFO)
                        {
                            UdpStats.AckInfoHits++;
                            ChannelListUpdateArgs.ClientRequests++;
                        }
                    }
                }
                catch // would block
                {
                    if (datagram.tries++ > 2)
                        this.datagrams_out.Dequeue();

                    return;
                }
            }
        }

        private void ReceiveInbound()
        {
            EndPoint address = (EndPoint)new IPEndPoint(IPAddress.Any, 0);
            byte[] buffer = new byte[8192];

            while (this.socket.Available > 0)
            {
                try
                {
                    int size = this.socket.ReceiveFrom(buffer, ref address);

                    if (size > 0)
                    {
                        UdpMessage msg = (UdpMessage)buffer[0];
                        byte[] packet = new byte[size - 1];
                        Array.Copy(buffer, 1, packet, 0, packet.Length);

                        try
                        {
                            this.Eval((IPEndPoint)address, msg, new UdpPacketReader(packet));
                        }
                        catch { }
                    }
                }
                catch { break; } // would block
            }
        }

        private void Eval(IPEndPoint address, UdpMessage msg, UdpPacketReader packet)
        {
            switch (msg)
            {
                case UdpMessage.OP_SERVERLIST_SENDINFO: // client wants us on their channel list
                    UdpPacketProcessor.ProcessSendInfo(address, ref this.datagrams_out);
                    break;

                case UdpMessage.OP_SERVERLIST_ADDIPS: // server wants us on our server list
                    UdpPacketProcessor.ProcessAddIps(address, packet, ref this.datagrams_out);
                    break;

                case UdpMessage.OP_SERVERLIST_ACKIPS: // server has put us on their server list
                    UdpPacketProcessor.ProcessAckIps(address, packet);
                    break;

                case UdpMessage.OP_SERVERLIST_SENDNODES: // server wants us to send some supernodes
                    UdpPacketProcessor.ProcessSendNodes(address, ref this.datagrams_out);
                    break;

                case UdpMessage.OP_SERVERLIST_WANTCHECKFIREWALL: // server asks if we can check their firewall
                    UdpPacketProcessor.ProcessWantCheckFirewall(address, packet, ref this.datagrams_out, ref this.firewall_tests);
                    break;

                case UdpMessage.OP_SERVERLIST_READYTOCHECKFIREWALL: // server will check our firewall
                    if (UdpPacketProcessor.ProcessReadyCheckFirewall(address, packet, ref this.datagrams_out, ref this.received_server_ip))
                        if (!this.finished_testing_my_firewall)
                            this.firewall_checkers.Add(address.Address);
                    break;

                case UdpMessage.OP_SERVERLIST_PROCEEDCHECKFIREWALL: // server wants us to start checking their firewall
                    UdpPacketProcessor.ProcessProceedCheckFirewall(packet, ref this.firewall_tests);
                    break;

                case UdpMessage.OP_SERVERLIST_CHECKFIREWALLBUSY: // server is too busy to check our firewall
                    UdpPacketProcessor.ProcessCheckFirewallBusy(packet);
                    break;
            }
        }
    }
}
