﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;

namespace callisto
{
    class UdpPacketHelpers
    {
        public static byte[] AckInfo()
        {
            UdpPacketWriter packet = new UdpPacketWriter();
            packet.WriteInt16((ushort)Settings.ServerPort);
            packet.WriteInt16(UserPool.UserCount);
            packet.WriteString(Settings.ServerName);
            packet.WriteString(Settings.ServerTopic);
            packet.WriteByte((byte)Helpers.ComboBoxLanguageToAresLanguage(Settings.ServerLanguage));
            packet.WriteString(Settings.SERVER_VERSION);
            UdpServerObject[] servers = UdpServerPool.GetServers(6);
            packet.WriteByte((byte)servers.Length);

            foreach (UdpServerObject s in servers)
            {
                packet.WriteIP(s.ip);
                packet.WriteInt16(s.port);
            }

            return packet.ToAresPacket(UdpMessage.OP_SERVERLIST_ACKINFO);
        }

        public static byte[] AddIps(IPAddress target_ip)
        {
            UdpPacketWriter packet = new UdpPacketWriter();
            packet.WriteInt16((ushort)Settings.ServerPort);

            UdpServerObject[] servers = UdpServerPool.GetServers(target_ip, 6);

            foreach (UdpServerObject s in servers)
            {
                packet.WriteIP(s.ip);
                packet.WriteInt16(s.port);
            }

            return packet.ToAresPacket(UdpMessage.OP_SERVERLIST_ADDIPS);
        }

        public static byte[] AckIps(IPAddress target_ip)
        {
            UdpPacketWriter packet = new UdpPacketWriter();
            packet.WriteInt16((ushort)Settings.ServerPort);

            UdpServerObject[] servers = UdpServerPool.GetServers(target_ip, 6);

            foreach (UdpServerObject s in servers)
            {
                packet.WriteIP(s.ip);
                packet.WriteInt16(s.port);
            }

            return packet.ToAresPacket(UdpMessage.OP_SERVERLIST_ACKIPS);
        }

        public static byte[] AckNodes(byte[] nodes)
        {
            UdpPacketWriter packet = new UdpPacketWriter();
            packet.WriteInt16((ushort)Settings.ServerPort);
            packet.WriteBytes(nodes);
            return packet.ToAresPacket(UdpMessage.OP_SERVERLIST_ACKNODES);
        }

        public static byte[] WantCheckFirewall()
        {
            UdpPacketWriter packet = new UdpPacketWriter();
            packet.WriteInt16((ushort)Settings.ServerPort);
            return packet.ToAresPacket(UdpMessage.OP_SERVERLIST_WANTCHECKFIREWALL);
        }

        public static byte[] ReadyCheckFirewall(IPAddress target_ip, uint cookie)
        {
            UdpPacketWriter packet = new UdpPacketWriter();
            packet.WriteInt32(cookie);
            packet.WriteIP(target_ip);
            return packet.ToAresPacket(UdpMessage.OP_SERVERLIST_READYTOCHECKFIREWALL);
        }

        public static byte[] CheckFirewallBusy(IPAddress target_ip)
        {
            UdpPacketWriter packet = new UdpPacketWriter();
            packet.WriteInt16((ushort)Settings.ServerPort);

            UdpServerObject[] servers = UdpServerPool.GetServers(target_ip, 6);

            foreach (UdpServerObject s in servers)
            {
                packet.WriteIP(s.ip);
                packet.WriteInt16(s.port);
            }

            return packet.ToAresPacket(UdpMessage.OP_SERVERLIST_CHECKFIREWALLBUSY);
        }

        public static byte[] ProceedCheckFirewall(uint cookie)
        {
            UdpPacketWriter packet = new UdpPacketWriter();
            packet.WriteInt16((ushort)Settings.ServerPort);
            packet.WriteInt32(cookie);
            return packet.ToAresPacket(UdpMessage.OP_SERVERLIST_PROCEEDCHECKFIREWALL);
        }

    }
}
