﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Security.Cryptography;

namespace callisto
{
    class UdpPacketProcessor
    {
        public static void ProcessSendInfo(IPEndPoint address, ref Queue<UdpDatagram> datagrams_out)
        {
            if (!Settings.ShouldList)
                return;

            UdpStats.SendInfoHits++;
            UdpDatagram datagram = new UdpDatagram();
            datagram.Address = address;
            datagram.Msg = UdpMessage.OP_SERVERLIST_ACKINFO;
            datagram.Packet = UdpPacketHelpers.AckInfo();
            datagrams_out.Enqueue(datagram);
        }

        public static void ProcessAddIps(IPEndPoint address, UdpPacketReader packet, ref Queue<UdpDatagram> datagrams_out)
        {
            if (packet.Remaining() < 2)
                return;

            ushort port = packet.ReadInt16();
            int time = (int)Helpers.UnixTime;

            UdpServerObject server = UdpServerPool.Pool.Find(s => s.ip.Equals(address.Address));

            if (server != null)
            {
                server.port = port;
            }
            else
            {
                server = new UdpServerObject(address.Address, port);
                UdpServerPool.AddServer(server);
            }

            UdpServerPool.PurgeList();

            while (packet.Remaining() > 5)
            {
                IPAddress additional_ip = packet.ReadIP();
                ushort additional_port = packet.ReadInt16();
                UdpServerPool.AddServer(new UdpServerObject(additional_ip, additional_port));
            }

            UdpDatagram datagram = new UdpDatagram();
            datagram.Address = address;
            datagram.Msg = UdpMessage.OP_SERVERLIST_ACKIPS;
            datagram.Packet = UdpPacketHelpers.AckIps(address.Address);
            datagrams_out.Enqueue(datagram);
        }

        public static void ProcessAckIps(IPEndPoint address, UdpPacketReader packet)
        {
            if (packet.Remaining() < 2)
                return;

            UdpStats.IpsAckHits++;
            ChannelListUpdateArgs.ServerResponses++;
            ushort port = packet.ReadInt16();
            int time = (int)Helpers.UnixTime;
            UdpServerObject server = UdpServerPool.Pool.Find(s => s.ip.Equals(address.Address));

            if (server != null)
            {
                server.port = port;
                server.ack_count++;
                server.last_connect = time;
                server.try_count = 0;
            }

            while (packet.Remaining() > 5)
            {
                IPAddress additional_ip = packet.ReadIP();
                ushort additional_port = packet.ReadInt16();
                UdpServerPool.AddServer(new UdpServerObject(additional_ip, additional_port));
            }
        }

        public static void ProcessSendNodes(IPEndPoint address, ref Queue<UdpDatagram> datagrams_out)
        {
            byte[] nodes = UserPool.GetSuperNodes();

            if (nodes.Length > 0)
            {
                UdpDatagram datagram = new UdpDatagram();
                datagram.Address = address;
                datagram.Msg = UdpMessage.OP_SERVERLIST_ACKNODES;
                datagram.Packet = UdpPacketHelpers.AckNodes(nodes);
                datagrams_out.Enqueue(datagram);
            }
        }

        public static void ProcessWantCheckFirewall(IPEndPoint address, UdpPacketReader packet, ref Queue<UdpDatagram> datagrams_out, ref List<UdpFirewallObject> firewall_tests)
        {
            if (packet.Remaining() < 2)
                return;

            ushort port = packet.ReadInt16();

            if (firewall_tests.Count < 4) // let's test their firewall
            {
                uint cookie = Helpers.FirewallCookie;
                IPEndPoint addr = new IPEndPoint(address.Address, port);
                firewall_tests.Add(new UdpFirewallObject(addr, (int)Helpers.UnixTime, cookie));

                UdpDatagram datagram = new UdpDatagram();
                datagram.Address = address;
                datagram.Msg = UdpMessage.OP_SERVERLIST_READYTOCHECKFIREWALL;
                datagram.Packet = UdpPacketHelpers.ReadyCheckFirewall(address.Address, cookie);
                datagrams_out.Enqueue(datagram);
            }
            else // go away we're busy
            {
                UdpDatagram datagram = new UdpDatagram();
                datagram.Address = address;
                datagram.Msg = UdpMessage.OP_SERVERLIST_CHECKFIREWALLBUSY;
                datagram.Packet = UdpPacketHelpers.CheckFirewallBusy(address.Address);
                datagrams_out.Enqueue(datagram);
            }
        }

        public static bool ProcessReadyCheckFirewall(IPEndPoint address, UdpPacketReader packet, ref Queue<UdpDatagram> datagrams_out, ref bool received_server_ip)
        {
            if (packet.Remaining() < 8)
                return false;

            uint cookie = packet.ReadInt32();

            if (!received_server_ip)
            {
                received_server_ip = true;
                Settings.ServerIP = packet.ReadIP();
                UserPool.UpdateLocalHosts();
            }

            UdpDatagram datagram = new UdpDatagram();
            datagram.Address = address;
            datagram.Msg = UdpMessage.OP_SERVERLIST_PROCEEDCHECKFIREWALL;
            datagram.Packet = UdpPacketHelpers.ProceedCheckFirewall(cookie);
            datagrams_out.Enqueue(datagram);
            return true;
        }

        public static void ProcessProceedCheckFirewall(UdpPacketReader packet, ref List<UdpFirewallObject> firewall_tests)
        {
            if (packet.Remaining() < 6)
                return;

            ushort port = packet.ReadInt16();
            uint cookie = packet.ReadInt32();
            UdpFirewallObject test = firewall_tests.Find(f => f.cookie == cookie);
            test.address = new IPEndPoint(test.address.Address, port);
            test.StartTest();
        }

        public static void ProcessCheckFirewallBusy(UdpPacketReader packet)
        {
            while (packet.Remaining() > 6)
            {
                IPAddress additional_ip = packet.ReadIP();
                ushort additional_port = packet.ReadInt16();
                UdpServerPool.AddServer(new UdpServerObject(additional_ip, additional_port));
            }
        }

    }
}
