﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ICallisto;

namespace callisto
{
    static class WebPool
    {
        public static WebUser[] Pool;

        public static void CreateUserPool()
        {
            Pool = new WebUser[200];
            
            for (int i = 0; i < Pool.Length; i++)
                Pool[i] = new WebUser();
        }

        public static int NextAvailable
        {
            get
            {
                for (int i = 0; i < Pool.Length; i++)
                    if (!Pool[i].InUse)
                        return i;

                return -1;
            }
        }

        public static void BroadcastAresJoin(IUser userobj)
        {
            foreach (WebUser u in Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Vroom == userobj.Vroom)
                            u.QueuePacket(WebPackets.JoinTo(u, userobj.Name, userobj.Level));
        }

        public static void BroadcastAresPart(IUser userobj)
        {
            foreach (WebUser u in Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Vroom == userobj.Vroom)
                            u.QueuePacket(WebPackets.PartTo(u, userobj.Name));
        }

        public static void BroadcastAresFont(UserObject userobj)
        {
            foreach (WebUser u in Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Vroom == userobj.Vroom)
                            u.QueuePacket(WebPackets.FontTo(u, userobj.Name, userobj.font.NameColor, userobj.font.TextColor));
        }

        public static void BroadcastAresUpdate(IUser userobj)
        {
            foreach (WebUser u in Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Vroom == userobj.Vroom)
                            u.QueuePacket(WebPackets.UpdateTo(u, userobj.Name, userobj.Level));
        }

        public static void BroadcastPrint(String text)
        {
            foreach (WebUser u in Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        u.QueuePacket(WebPackets.NoSuchTo(u, text));
        }

        public static void BroadcastPrint(ushort vroom, String text)
        {
            foreach (WebUser u in Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Vroom == vroom)
                            u.QueuePacket(WebPackets.NoSuchTo(u, text));
        }

        public static void BroadcastCustomNamePublic(IUser userobj, String text)
        {
            foreach (WebUser u in Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Vroom == userobj.Vroom)
                            u.QueuePacket(WebPackets.NoSuchTo(u, userobj.CustomName + text));
        }

        public static void BroadcastPublic(IUser userobj, String text)
        {
            foreach (WebUser u in Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Vroom == userobj.Vroom)
                            u.QueuePacket(WebPackets.PublicTo(u, userobj.Name, text));
        }

        public static void BroadcastEmote(IUser userobj, String text)
        {
            foreach (WebUser u in Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Vroom == userobj.Vroom)
                            u.QueuePacket(WebPackets.EmoteTo(u, userobj.Name, text));
        }

        public static void BroadcastTopicUpdated(String text)
        {
            foreach (WebUser u in Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        u.QueuePacket(WebPackets.TopicTo(u, text));
        }

        public static void BroadcastUrlUpdated(String addr, String text)
        {
            foreach (WebUser u in Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        u.QueuePacket(WebPackets.UrlTo(u, addr, text));
        }

        public static IUser FindUserByName(String name)
        {
            foreach (WebUser u in Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Name == name)
                            return u;

            return null;
        }

        public static IUser FindUserByName(String name, ushort vroom)
        {
            foreach (WebUser u in Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Vroom == vroom)
                            if (u.Name == name)
                                return u;

            return null;
        }

        public static IUser FindUserByNameStartsWith(String name)
        {
            foreach (WebUser u in Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Name.StartsWith(name))
                            return u;

            return null;
        }

        public static IUser FindUserByID(ushort id)
        {
            foreach (WebUser u in Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Id == id)
                            return u;

            return null;
        }

        public static IUser FirstUser
        {
            get
            {
                foreach (WebUser u in Pool)
                    if (u.InUse)
                        if (u.LoggedIn)
                            return u;

                return null;
            }
        }

        public static void SendUserlistToUser(WebUser userobj)
        {
            userobj.QueuePacket(WebPackets.UserlistItemTo(userobj, Settings.BotName, AdminLevel.Host));

            foreach (UserObject u in UserPool.Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == userobj.Vroom)
                            if (!u.cloaked)
                                userobj.QueuePacket(WebPackets.UserlistItemTo(userobj, u.name, u.level));

            if (CallistoServer.Linker.IsLinking && userobj.Vroom == 0)
                foreach (LinkedUserObject u in CallistoServer.Linker.linked_users)
                    if (u.visible)
                        userobj.QueuePacket(WebPackets.UserlistItemTo(userobj, u.name, u.level));

            foreach (WebUser u in Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Vroom == userobj.Vroom)
                            userobj.QueuePacket(WebPackets.UserlistItemTo(userobj, u.Name, u.Level));

            userobj.QueuePacket(WebPackets.UserlistEndTo(userobj));
        }

        public static void SendFontsToUser(WebUser userobj)
        {
            foreach (UserObject u in UserPool.Pool)
                if (u.Connected)
                    if (u.logged_in)
                        if (u.vroom == userobj.Vroom)
                            if (u.font != null)
                                userobj.QueuePacket(WebPackets.FontTo(userobj, u.name, u.font.NameColor, u.font.TextColor));
        }

        public static WebUser FindGhost(IUser userobj)
        {
            foreach (WebUser u in Pool)
                if (u.InUse)
                    if (u.LoggedIn)
                        if (u.Id != userobj.Id)
                            if (u.Name == userobj.Name)
                                if (u.Guid.Equals(userobj.Guid) || u.ExternalIP.Equals(userobj.ExternalIP))
                                    return u;

            return null;
        }

    }
}
