﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;
using System.Threading;
using ICallisto;

namespace callisto
{
    class WebServer
    {
        private uint last_push;

        public WebServer(uint time)
        {
            this.last_push = (time - 1000);
        }

        public void DoTasks(uint time)
        {
            if (!Settings.allow_silverlight)
                return;

            if (time > (this.last_push + 1200))
            {
                this.last_push = time;
                this.ChannelLister();
            }

            foreach (WebUser userobj in WebPool.Pool)
            {
                if (userobj.InUse)
                {
                    userobj.Send();
                    userobj.Receive();

                    if (!userobj.ProtoConnected)
                    {
                        if (userobj.LoadProtocol())
                        {
                            Html5RequestEventArgs e = new Html5RequestEventArgs
                            {
                                Key = userobj.Key,
                                Host = userobj.Host,
                                Key1 = userobj.Key1,
                                Key2 = userobj.Key2,
                                KeyData = userobj.KeyData,
                                OldProto = userobj.OldProto,
                                Origin = userobj.Origin,
                                Time = time
                            };

                            try
                            {
                                byte[] packet = WebPackets.Html5HandshakeReplyPacket(e);
                                userobj.QueuePacket(packet);
                                userobj.ProtoConnected = true;
                                userobj.Time = time;
                                continue;
                            }
                            catch { userobj.Expired = true; }
                        }

                        if (userobj.Expired || (userobj.Time + 10) < time)
                            userobj.Disconnect();
                    }
                    else
                    {
                        if (!userobj.LoggedIn)
                            if (userobj.Expired || (userobj.Time + 15) < time)
                            {
                                userobj.Disconnect();
                                continue;
                            }

                        String message;

                        while ((message = userobj.NextMessage) != null && !userobj.Expired)
                        {
                            int i = message.IndexOf(":");

                            if (i == -1)
                            {
                                userobj.Disconnect();
                                break;
                            }

                            String ident = message.Substring(0, i);
                            String args = message.Substring(i + 1);

                            try
                            {
                                this.Evaluate(userobj, ident, args, time);
                            }
                            catch
                            {
                                userobj.Disconnect();
                                break;
                            }
                        }

                        if (userobj.Expired || (userobj.Time + 120) < time || userobj.HasOverflowed)
                            userobj.Disconnect();
                    }
                }
            }
        }

        public void Stop()
        {
            foreach (WebUser u in WebPool.Pool)
                if (u.InUse)
                    u.Disconnect();
        }

        private void ChannelLister()
        {
            Thread thread = new Thread(new ThreadStart(() =>
            {
                try
                {
                    StringBuilder sb = new StringBuilder();
                    sb.Append("local=" + Settings.ib0t_loc);
                    sb.Append("&port=" + Settings.ServerPort);
                    sb.Append("&name=" + Uri.EscapeDataString(Settings.ServerName));
                    sb.Append("&topic=" + Uri.EscapeDataString(Settings.ServerTopic));

                    byte[] data = Encoding.UTF8.GetBytes(sb.ToString());

                    sb.Clear();

                    WebRequest request = WebRequest.Create("http://chatrooms.marsproject.net/ibot.aspx");
                    request.Method = "POST";
                    request.ContentLength = data.Length;
                    request.ContentType = "application/x-www-form-urlencoded";

                    using (Stream stream = request.GetRequestStream())
                        stream.Write(data, 0, data.Length);

                    WebResponse response = request.GetResponse();
                    response.Close();
                }
                catch { }
            }));

            thread.Start();
        }

        private void Evaluate(WebUser userobj, String ident, String args, uint time)
        {
            if (!userobj.LoggedIn && ident != "LOGIN")
                throw new Exception();

            if (userobj.LoggedIn && ident == "LOGIN")
                throw new Exception();

            switch (ident)
            {
                case "LOGIN":
                    this.ProcessLogin(userobj, args, time);
                    break;

                case "PUBLIC":
                    this.ProcessText(userobj, args, time);
                    break;

                case "EMOTE":
                    this.ProcessEmote(userobj, args, time);
                    break;

                case "COMMAND":
                    this.ProcessCommand(userobj, args, time);
                    break;

                case "PING":
                    userobj.Time = time;
                    break;

                default:
                    throw new Exception();
            }
        }

        private void ProcessLogin(WebUser userobj, String args, uint time)
        {
            userobj.PopulateCredentials(args);
            userobj.Time = time;

            UserObject ghost_user1 = null;
            WebUser ghost_user2 = null;

            if (Proxies.IsProxy(userobj))
            {
                ServerEvents.OnRejected(userobj, NativeRejectionType.Banned);
                userobj.Ban();
                return;
            }

            if (UserRecordManager.IsMaxClientsExcess(userobj, 4))
            {
                ServerEvents.OnRejected(userobj, NativeRejectionType.ClientFlood);
                userobj.Disconnect();
                return;
            }

            if (UserRecordManager.IsJoinFlooding(userobj))
            {
                ServerEvents.OnRejected(userobj, NativeRejectionType.LoginFlood);
                userobj.Disconnect();
                return;
            }

            if (UserPool.NameExists(userobj)) // let's prevent name hijacking
            {
                ghost_user1 = UserPool.FindGhost(userobj);

                if (ghost_user1 != null)
                {
                    ghost_user1.Disconnect(true);
                }
                else
                {
                    ghost_user2 = WebPool.FindGhost(userobj);

                    if (ghost_user2 != null)
                        ghost_user2.Disconnect(true);
                    else
                    {
                        ServerEvents.OnRejected(userobj, NativeRejectionType.NameInUse);
                        userobj.Disconnect();
                        return;
                    }
                }
            }

            UserRecordManager.AddRecord(userobj);

            if (BanManager.IsBanned(userobj))
            {
                ServerEvents.OnRejected(userobj, NativeRejectionType.Banned);

                if (ghost_user1 != null)
                    ghost_user1.ShowUserParting();

                if (ghost_user2 != null)
                    ghost_user2.ShowUserParting();

                userobj.Disconnect();
                return;
            }

            if (!ServerEvents.OnJoinCheck(userobj))
            {
                ServerEvents.OnRejected(userobj, NativeRejectionType.Other);
                CallistoPluginStats.rejection_count++;

                if (ghost_user1 != null)
                    ghost_user1.ShowUserParting();

                if (ghost_user2 != null)
                    ghost_user2.ShowUserParting();

                userobj.Disconnect();
                return;
            }
            else
            {
                bool ghost = ghost_user1 != null || ghost_user2 != null;

                if (!ghost) // anti-bounce for ghosts
                {
                    CallistoPluginStats.join_count++;

                    if (UserPool.UserCount > CallistoPluginStats.peak_user_count)
                        CallistoPluginStats.peak_user_count = UserPool.UserCount;

                    LinkedUserObject lCheck = null;

                    if (CallistoServer.Linker.IsLinking)
                        lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == userobj.Name);

                    if (lCheck != null && userobj.Vroom == 0) // user already exists as a linked user
                    {
                        lCheck.visible = false;
                        UserPool.BroadcastToVroom(userobj.Vroom, WebPackets.AresUpdateUser(userobj));
                        WebPool.BroadcastAresUpdate(userobj);
                    }
                    else
                    {
                        UserPool.BroadcastToVroom(userobj.Vroom, WebPackets.AresJoin(userobj));
                        WebPool.BroadcastAresJoin(userobj);
                    }
                }
                else userobj.AddClientTag("CALLISTO_GHOST");

                UserPool.BroadcastToVroom(userobj.Vroom, WebPackets.AresAvatar(userobj));
                UserPool.BroadcastToVroom(userobj.Vroom, WebPackets.AresPersonalMessage(userobj));

                userobj.LoggedIn = true;
                userobj.QueuePacket(WebPackets.AckTo(userobj, userobj.Name));
                userobj.QueuePacket(WebPackets.TopicFirstTo(userobj, Settings.ServerTopic));

                WebPool.SendUserlistToUser(userobj);
                WebPool.SendFontsToUser(userobj);

                if (!String.IsNullOrEmpty(Settings.Url[0]) && !String.IsNullOrEmpty(Settings.Url[1]))
                    userobj.QueuePacket(WebPackets.UrlTo(userobj, Settings.Url[0], Settings.Url[1]));

                if (!ghost)
                    if (CallistoServer.Linker.IsLinking && userobj.Vroom == 0)
                        CallistoServer.Linker.SendPacket(WebPackets.AresJoin(userobj, true));

                ServerEvents.OnJoin(userobj);

                if (Settings.ban_spambots)
                {
                    if (!Captcha.IsTrusted(userobj.raw_guid))
                    {
                        ServerEvents.OnCaptcha(userobj, CaptchaState.Sending);
                        CaptchaObj cap = Captcha.Create(userobj);

                        foreach (byte[] b in cap.packets)
                            userobj.QueuePacket(b);

                        userobj.captcha_answer = cap.answer;
                    }
                    else userobj.has_captched = true;
                }
                else userobj.has_captched = true;
            }
        }

        private void ProcessText(WebUser userobj, String args, uint time)
        {
            String text = args;

            if (userobj.IsFlooding(text, time))
            {
                if (ServerEvents.OnFlood(userobj, ProtoMessage.MSG_CHAT_CLIENT_PUBLIC))
                {
                    userobj.Disconnect();
                    return;
                }
            }

            if (text.Length > 200)
            {
                userobj.Disconnect();
                return;
            }

            if (!userobj.has_captched)
            {
                String cap_str = args;
                cap_str = Helpers.StripColors(cap_str);
                cap_str = cap_str.Trim();

                if (cap_str.Length > 0)
                {
                    userobj.QueuePacket(WebPackets.PublicTo(userobj, userobj.name, cap_str));
                    int cap;

                    if (int.TryParse(cap_str, out cap))
                    {
                        if (userobj.captcha_answer == cap)
                        {
                            userobj.has_captched = true;
                            ServerEvents.OnCaptcha(userobj, CaptchaState.Success);
                            Captcha.MakeTrusted(userobj.raw_guid);
                            return;
                        }
                    }
                }

                userobj.captcha_attempts++;

                if (userobj.captcha_attempts > 2)
                {
                    ServerEvents.OnCaptcha(userobj, CaptchaState.Exceeded);
                    userobj.Disconnect();
                    return;
                }
                else
                {
                    ServerEvents.OnCaptcha(userobj, CaptchaState.Incorrect);
                    CaptchaObj cap = Captcha.Create(userobj);

                    foreach (byte[] b in cap.packets)
                        userobj.QueuePacket(b);

                    userobj.captcha_answer = cap.answer;
                    return;
                }
            }

            CallistoPluginStats.public_messages++;
            text = text.Replace("\x0002", String.Empty);

            if (text.StartsWith("#"))
            {
                CommandObject cobj = Helpers.EvaluateClientCommandString(text);

                if (CallistoServer.Linker.IsLinking && cobj.tUser != null)
                {
                    if (cobj.tUser.IsLinked)
                    {
                        CallistoServer.Linker.SendPacket(LinkPacketHelpers.AdminCommand(userobj.name,
                            cobj.tUser.Name, cobj.cmdText.Split(new String[] { " " }, StringSplitOptions.None)[0], cobj.args));
                        return;
                    }
                }

                ServerEvents.OnCommand(userobj, cobj.cmdText, cobj.tUser, cobj.args);
            }

            if (!userobj.LoggedIn)
                return;

            ServerEvents.OnTextReceived(userobj, text);

            if (userobj.LoggedIn)
            {
                if (CallistoServer.Linker.IsLinking && userobj.Vroom == 0)
                    CallistoServer.Linker.SendPacket(LinkPacketHelpers.TextReceived(userobj, text));

                text = ServerEvents.OnTextBefore(userobj, text);

                if (text.Length > 0 && userobj.LoggedIn && !userobj.Muzzled) // blocked text will be empty by now
                {
                    if (userobj.CustomName.Length > 0 && Settings.CAN_CUSTOM_NAME)
                    {
                        UserPool.BroadcastToUnignored(userobj, ChatPacketHelpers.NoSuch(userobj.CustomName + text));
                        WebPool.BroadcastCustomNamePublic(userobj, text);
                    }
                    else
                    {
                        UserPool.BroadcastToUnignored(userobj, ChatPacketHelpers.Public(userobj.name, text));
                        WebPool.BroadcastPublic(userobj, text);
                    }

                    ServerEvents.OnTextAfter(userobj, text);

                    if (CallistoServer.Linker.IsLinking && userobj.Vroom == 0)
                        CallistoServer.Linker.SendPacket(ChatPacketHelpers.Public(userobj.name, text));
                }
            }
        }

        private void ProcessEmote(WebUser userobj, String args, uint time)
        {
            String text = args;

            if (userobj.IsFlooding(text, time))
            {
                if (ServerEvents.OnFlood(userobj, ProtoMessage.MSG_CHAT_CLIENT_EMOTE))
                {
                    userobj.Disconnect();
                    return;
                }
            }

            if (text.Length > 200)
            {
                userobj.Disconnect();
                return;
            }

            if (!userobj.has_captched)
                return;

            CallistoPluginStats.public_messages++;
            text = text.Replace("\x0002", String.Empty);

            ServerEvents.OnEmoteReceived(userobj, text);

            if (userobj.LoggedIn)
            {
                if (CallistoServer.Linker.IsLinking && userobj.Vroom == 0)
                    CallistoServer.Linker.SendPacket(LinkPacketHelpers.EmoteReceived(userobj, text));

                text = ServerEvents.OnEmoteBefore(userobj, text);

                if (text.Length > 0 && userobj.LoggedIn && !userobj.Muzzled) // blocked text will be empty by now
                {
                    UserPool.BroadcastToUnignored(userobj, ChatPacketHelpers.Emote(userobj.name, text));
                    WebPool.BroadcastEmote(userobj, text);
                    ServerEvents.OnEmoteAfter(userobj, text);

                    if (CallistoServer.Linker.IsLinking && userobj.Vroom == 0)
                        CallistoServer.Linker.SendPacket(ChatPacketHelpers.Emote(userobj.name, text));
                }
            }
        }

        private void ProcessCommand(WebUser userobj, String args, uint time)
        {
            String text = args;

            if (userobj.IsFlooding(text, time))
            {
                if (ServerEvents.OnFlood(userobj, ProtoMessage.MSG_CHAT_CLIENT_COMMAND))
                {
                    userobj.Disconnect();
                    return;
                }
            }

            if (text.Length > 200)
            {
                userobj.Disconnect();
                return;
            }

            if (!userobj.has_captched)
                return;

            CommandObject cobj = Helpers.EvaluateClientCommandString(text);

            if (CallistoServer.Linker.IsLinking && cobj.tUser != null)
            {
                if (cobj.tUser.IsLinked)
                {
                    CallistoServer.Linker.SendPacket(LinkPacketHelpers.AdminCommand(userobj.name,
                        cobj.tUser.Name, cobj.cmdText.Split(new String[] { " " }, StringSplitOptions.None)[0], cobj.args));
                    return;
                }
            }

            ServerEvents.OnCommand(userobj, cobj.cmdText, cobj.tUser, cobj.args);
        }

    }
}
