﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Globalization;
using ICallisto;

namespace callisto
{
    class WebUser : IUser
    {
        internal int captcha_attempts = 0;
        internal int captcha_answer = 0;
        internal bool has_captched = false;


        internal String Key { get; set; }
        internal Socket Socket { get; set; }
        internal WebDataHeap DataHeap { get; set; }
        internal int Health { get; set; }
        internal bool Expired { get; set; }
        internal String Key1 { get; set; }
        internal String Key2 { get; set; }
        internal byte[] KeyData { get; set; }
        internal bool OldProto { get; set; }
        internal String Host { get; set; }
        internal String Origin { get; set; }
        internal uint Time { get; set; }
        internal bool ProtoConnected { get; set; }
        internal bool LoggedIn { get; set; }

        internal String name = String.Empty;

        private bool pending_keydata = false;
        private List<byte[]> bytes_out = new List<byte[]>();
        private ushort vroom = 0;
        private CallistoCustomDataManager custom_data_manager = new CallistoCustomDataManager();
        private List<String> client_tags = new List<String>();
        private uint last_text_packet;

        internal String custom_name = String.Empty;
        internal bool muz = false;

        #region IUSER

        public bool Muzzled
        {
            get { return this.muz; }
            set
            {
                this.muz = value;

                if (CallistoServer.Linker.IsLinking)
                    CallistoServer.Linker.SendPacket(LinkPacketHelpers.SendMuzzleUpdate(this));
            }
        }

        public String CustomName
        {
            get { return this.custom_name; }
            set
            {
                this.custom_name = value;

                if (this.custom_name == null)
                    this.custom_name = String.Empty;

                if (CallistoServer.Linker.IsLinking)
                    CallistoServer.Linker.SendPacket(LinkPacketHelpers.SendCustomNameText(this));
            }
        }

        public IPAddress ExternalIP { get; private set; }
        public int Id { get; private set; }
        public byte Country { get { return 0; } }
        public bool Browsable { get { return false; } }
        public bool IsLocalHost { get { return false; } }
        public bool IsLinked { get { return false; } }
        public IPAddress LocalIP { get { return this.ExternalIP; } }
        public bool FastPing { get { return false; } }
        public String Guid { get; private set; }
        public ushort FileCount { get { return 0; } }
        public String OrgName { get; private set; }
        public ushort Port { get { return 0; } }
        public String Version { get { return "ib0t web user"; } }
        public String Region { get { return String.Empty; } }
        public String[] IgnoreList { get { return new String[] { }; } }
        public String[] CustomClientTags { get { return this.client_tags.ToArray(); } }
        public Dictionary<String, object> CustomData { get { return this.custom_data_manager.GetData; } }
        public void Redirect(String hashlink) { }
        public void DownloadAvatar(String url) { }
        public void RestoreAvatar() { }
        public void SendCustomClientData(String x, String y, byte[] z) { }
        public void SendPM(String name, String text) { }

        public IUser NextUser
        {
            get
            {
                foreach (WebUser u in WebPool.Pool)
                    if (u.InUse)
                        if (u.LoggedIn)
                            if (u.Id > this.Id)
                                return u;

                return null;
            }
        }

        public AdminLevel Level
        {
            get { return AdminLevel.Regular; }
            set { }
        }

        public ushort Vroom
        {
            get { return this.vroom; }
            set
            {
                if (!this.LoggedIn) // not logged in yet (changing vroom at OnJoinCheck)
                {
                    this.vroom = value;
                    return;
                }

                if (ServerEvents.OnVroomJoinCheck(this, value))
                {
                    ServerEvents.OnVroomPart(this);
                    this.LoggedIn = false;
                    LinkedUserObject lCheck = null;

                    if (CallistoServer.Linker.IsLinking)
                    {
                        lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == this.name);

                        if (this.vroom == 0) // tell linked room that user has parted
                            CallistoServer.Linker.SendPacket(WebPackets.AresPart(this));
                    }

                    if (lCheck != null && this.vroom == 0)
                    {
                        lCheck.visible = true;
                        UserPool.BroadcastToVroom(0, LinkPacketHelpers.Update(lCheck));
                        WebPool.BroadcastAresUpdate(lCheck);
                    }
                    else
                    {
                        UserPool.BroadcastToVroom(this.vroom, WebPackets.AresPart(this));
                        WebPool.BroadcastAresPart(this);
                    }

                    this.vroom = value;
                    lCheck = null;

                    if (CallistoServer.Linker.IsLinking)
                    {
                        lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == this.name);

                        if (this.vroom == 0)
                        {
                            CallistoServer.Linker.SendPacket(WebPackets.AresJoin(this, true));
                            CallistoServer.Linker.SendPacket(WebPackets.AresAvatar(this));
                            CallistoServer.Linker.SendPacket(WebPackets.AresPersonalMessage(this));

                            if (this.CustomName.Length > 0)
                                CallistoServer.Linker.SendPacket(LinkPacketHelpers.SendCustomNameText(this));
                        }
                    }

                    if (lCheck != null && this.vroom == 0) // linked user exists with new name
                    {
                        lCheck.visible = false;
                        UserPool.BroadcastToVroom(0, WebPackets.AresUpdateUser(this));
                        WebPool.BroadcastAresUpdate(this);
                    }
                    else
                    {
                        UserPool.BroadcastToVroom(this.vroom, WebPackets.AresJoin(this));
                        WebPool.BroadcastAresJoin(this);
                    }

                    UserPool.BroadcastToVroom(this.Vroom, WebPackets.AresAvatar(this));
                    UserPool.BroadcastToVroom(this.Vroom, WebPackets.AresPersonalMessage(this));

                    this.LoggedIn = true;
                    this.QueuePacket(WebPackets.AckTo(this, this.Name));

                    WebPool.SendUserlistToUser(this);
                    WebPool.SendFontsToUser(this);

                    ServerEvents.OnVroomJoin(this);
                }
            }
        }

        public IAresFont Font
        {
            get { return null; }
            set { }
        }

        public byte Age
        {
            get { return 0; }
            set { }
        }

        public byte Sex
        {
            get { return 0; }
            set { }
        }

        public bool Cloaked
        {
            get { return false; }
            set { }
        }

        public String Name
        {
            get { return this.name; }
            set
            {
                if (!this.LoggedIn) // not logged in yet (changing name at OnJoinCheck)
                {
                    Helpers.FormatUsername(this, value);
                    return;
                }

                this.LoggedIn = false;
                LinkedUserObject lCheck = null;

                if (CallistoServer.Linker.IsLinking)
                {
                    lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == this.name);

                    if (this.vroom == 0)
                        CallistoServer.Linker.SendPacket(WebPackets.AresPart(this));
                }

                if (lCheck != null && this.vroom == 0) // linked user exists with old name
                {
                    lCheck.visible = true;
                    UserPool.BroadcastToVroom(0, LinkPacketHelpers.Update(lCheck));
                    WebPool.BroadcastAresUpdate(lCheck);
                }
                else
                {
                    UserPool.BroadcastToVroom(this.vroom, WebPackets.AresPart(this));
                    WebPool.BroadcastAresPart(this);
                }

                Helpers.FormatUsername(this, value);
                lCheck = null;

                if (CallistoServer.Linker.IsLinking)
                {
                    lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == this.name);

                    if (this.vroom == 0)
                    {
                        CallistoServer.Linker.SendPacket(WebPackets.AresJoin(this, true));
                        CallistoServer.Linker.SendPacket(WebPackets.AresAvatar(this));
                        CallistoServer.Linker.SendPacket(WebPackets.AresPersonalMessage(this));

                        if (this.CustomName.Length > 0)
                            CallistoServer.Linker.SendPacket(LinkPacketHelpers.SendCustomNameText(this));
                    }
                }

                if (lCheck != null && this.vroom == 0) // linked user exists with new name
                {
                    lCheck.visible = false;
                    UserPool.BroadcastToVroom(0, WebPackets.AresUpdateUser(this));
                    WebPool.BroadcastAresUpdate(this);
                }
                else
                {
                    UserPool.BroadcastToVroom(this.vroom, WebPackets.AresJoin(this));
                    WebPool.BroadcastAresJoin(this);
                }

                UserPool.BroadcastToVroom(this.Vroom, WebPackets.AresAvatar(this));
                UserPool.BroadcastToVroom(this.Vroom, WebPackets.AresPersonalMessage(this));

                this.LoggedIn = true;
                this.QueuePacket(WebPackets.AckTo(this, this.Name));

                WebPool.SendUserlistToUser(this);
                WebPool.SendFontsToUser(this);
            }
        }

        public String PersonalMessage
        {
            get { return "ib0t web user"; }
            set { }
        }

        public byte[] Avatar
        {
            get { return Properties.Resources.web; }
            set { }
        }

        public void Ban()
        {
            BanManager.AddBan(this);
            this.Disconnect();
        }

        public void SetTopic(String text)
        {
            this.QueuePacket(WebPackets.TopicFirstTo(this, text));
        }
        
        public void SendPrint(String text)
        {
            this.QueuePacket(WebPackets.NoSuchTo(this, text));
        }

        public void SendText(String text)
        {
            if (Settings.CAN_CUSTOM_NAME && this.custom_name.Length > 0)
            {
                UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.NoSuch(this.custom_name + text));
                WebPool.BroadcastCustomNamePublic(this, text);
            }
            else
            {
                UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Public(this.name, text));
                WebPool.BroadcastPublic(this, text);
            }

            if (this.vroom == 0)
                if (CallistoServer.Linker.IsLinking)
                    CallistoServer.Linker.SendPacket(ChatPacketHelpers.Public(this.name, text));
        }

        public void SendEmote(String text)
        {
            UserPool.BroadcastToVroom(this.vroom, ChatPacketHelpers.Emote(this.name, text));
            WebPool.BroadcastEmote(this, text);

            if (this.vroom == 0)
                if (CallistoServer.Linker.IsLinking)
                    CallistoServer.Linker.SendPacket(ChatPacketHelpers.Emote(this.name, text));
        }

        public void UpdateURL(String url, String tag)
        {
            this.QueuePacket(WebPackets.UrlTo(this, url, tag));
        }
        
        public void Disconnect()
        {
            this.Disconnect(false);
        }

        #endregion


        internal WebUser()
        {
            this.Socket = null;
        }

        internal WebUser(UserObject userobj, int id, uint time)
        {
            this.Id = id;
            this.Socket = userobj.socket;
            this.Key = String.Empty;
            this.DataHeap = new WebDataHeap();
            this.DataHeap.AddRange(userobj.data_in.ToArray());
            this.Health = 0;
            this.Expired = false;
            this.Key1 = String.Empty;
            this.Key2 = String.Empty;
            this.KeyData = new byte[] { };
            this.OldProto = false;
            this.Host = String.Empty;
            this.Origin = String.Empty;
            this.Time = time;
            this.ProtoConnected = false;
            this.ExternalIP = ((IPEndPoint)this.Socket.RemoteEndPoint).Address;
            this.LoggedIn = false;
            this.Age = 0;
            this.last_text_packet = time;
        }

        internal void AddClientTag(String str)
        {
            this.client_tags.Add(str);
        }

        internal Guid raw_guid;

        internal void PopulateCredentials(String args)
        {
            byte[] g = new byte[16];

            for (int i = 0; i < g.Length; i++)
                g[i] = byte.Parse(args.Substring((i * 2), 2), NumberStyles.HexNumber);

            Guid guid = new Guid(g);
            this.raw_guid = guid;
            this.Guid = Convert.ToBase64String(guid.ToByteArray());

            this.name = args.Substring(32);
            Helpers.FormatUsername(this);
            this.OrgName = this.name;
        }

        internal void QueuePacket(byte[] packet)
        {
            this.bytes_out.Add(packet);
        }

        internal bool InUse
        {
            get { return this.Socket != null; }
        }

        internal void Disconnect(bool ghost)
        {
            if (this.Socket != null)
            {
                this.Send();

                try { this.Socket.Disconnect(false); }
                catch { }
                try { this.Socket.Shutdown(SocketShutdown.Both); }
                catch { }
                try { this.Socket.Close(); }
                catch { }
            }

            this.Socket = null;
            this.Expired = true;

            if (this.LoggedIn && !ghost)
            {
                CallistoPluginStats.part_count++;
                LinkedUserObject lCheck = null;

                if (CallistoServer.Linker.IsLinking)
                    lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == this.name);

                this.LoggedIn = false;
                ServerEvents.OnPart(this);

                if (lCheck != null && this.vroom == 0)
                {
                    lCheck.visible = true;
                    UserPool.BroadcastToVroom(this.vroom, LinkPacketHelpers.Update(lCheck));
                    WebPool.BroadcastAresUpdate(lCheck);
                }
                else
                {
                    UserPool.BroadcastToVroom(this.vroom, WebPackets.AresPart(this));
                    WebPool.BroadcastAresPart(this);
                }

                UserPool.RemoveIgnoredUser(this.Name);

                if (this.vroom == 0 && CallistoServer.Linker.IsLinking)
                    CallistoServer.Linker.SendPacket(WebPackets.AresPart(this));
            }
            else Scripting.ScriptManager.RemoveUser(this);

            this.DataHeap.Clear();
            this.bytes_out.Clear();
        }

        internal void ShowUserParting()
        {
            if (this.LoggedIn)
            {
                CallistoPluginStats.part_count++;
                LinkedUserObject lCheck = null;

                if (CallistoServer.Linker.IsLinking)
                    lCheck = CallistoServer.Linker.linked_users.Find(u => u.name == this.name);

                this.LoggedIn = false;

                if (lCheck != null && this.vroom == 0)
                {
                    lCheck.visible = true;
                    UserPool.BroadcastToVroom(this.vroom, LinkPacketHelpers.UserListItem(lCheck));
                    WebPool.BroadcastAresUpdate(lCheck);
                }
                else
                {
                    UserPool.BroadcastToVroom(this.vroom, WebPackets.AresPart(this));
                    WebPool.BroadcastAresPart(this);
                }

                ServerEvents.OnPart(this);
                UserPool.RemoveIgnoredUser(this.Name);

                if (this.vroom == 0 && CallistoServer.Linker.IsLinking)
                    CallistoServer.Linker.SendPacket(WebPackets.AresPart(this));
            }
        }

        internal void Receive()
        {
            byte[] buffer = new byte[8192];
            SocketError e = SocketError.Success;
            int size = 0;

            try { size = this.Socket.Receive(buffer, 0, this.Socket.Available, SocketFlags.None, out e); }
            catch { }

            if (size > 0)
            {
                this.Health = 0;
                buffer = buffer.Take(size).ToArray();
                this.DataHeap.AddRange(buffer);
            }
            else
            {
                if (e == SocketError.WouldBlock)
                    this.Health = 0;
                else if (this.Health++ > 5)
                    this.Expired = true;
            }
        }

        internal bool LoadProtocol()
        {
            if (this.pending_keydata)
            {
                if (this.DataHeap.Count >= 8)
                {
                    this.KeyData = this.DataHeap.GetRange(0, 8).ToArray();
                    this.DataHeap.RemoveRange(0, 8);
                    return true;
                }
            }
            else
            {
                while (this.DataHeap.CanTakeLine)
                {
                    String str = this.DataHeap.TakeLine();
                    String up = str.ToUpper();

                    if (up.StartsWith("SEC-WEBSOCKET-KEY:"))
                    {
                        str = str.Substring(18);

                        if (str.StartsWith(" "))
                            str = str.Substring(1);

                        this.Key = str;
                    }
                    else if (up.StartsWith("SEC-WEBSOCKET-KEY1:"))
                    {
                        str = str.Substring(19);

                        if (str.StartsWith(" "))
                            str = str.Substring(1);

                        this.Key1 = str;
                    }
                    else if (up.StartsWith("SEC-WEBSOCKET-KEY2:"))
                    {
                        str = str.Substring(19);

                        if (str.StartsWith(" "))
                            str = str.Substring(1);

                        this.Key2 = str;
                    }
                    else if (up.StartsWith("HOST:"))
                    {
                        str = str.Substring(5);

                        if (str.StartsWith(" "))
                            str = str.Substring(1);

                        this.Host = str;
                    }
                    else if (up.StartsWith("ORIGIN:"))
                    {
                        str = str.Substring(7);

                        if (str.StartsWith(" "))
                            str = str.Substring(1);

                        this.Origin = str;
                    }
                    else if (String.IsNullOrEmpty(str.Trim()))
                        if (this.Key1.Length > 0)
                        {
                            this.pending_keydata = true;
                            this.OldProto = true;
                        }
                        else return true;
                }
            }

            return false;
        }

        internal void Send()
        {
            while (this.bytes_out.Count > 0)
            {
                try
                {
                    this.Socket.Send(this.bytes_out[0]);
                    this.bytes_out.RemoveAt(0);
                }
                catch { break; }
            }
        }

        internal bool HasOverflowed
        {
            get { return this.DataHeap.Count > 16384; }
        }

        internal String NextMessage
        {
            get { return this.OldProto ? this.CheckPacketOld() : this.CheckPacketNew(); }
        }

        private String CheckPacketOld()
        {
            if (this.DataHeap.Count >= 2)
            {
                int index = this.DataHeap.IndexOf(255);

                if (index > 0)
                {
                    byte[] buffer = this.DataHeap.GetRange(1, (index - 1)).ToArray();
                    this.DataHeap.RemoveRange(0, (index + 1));
                    return Encoding.UTF8.GetString(buffer);
                }
            }

            return null;
        }

        private String CheckPacketNew()
        {
            if (this.DataHeap.Count >= 6)
            {
                int max_size = this.DataHeap.Count;
                byte msg = this.DataHeap[0];
                int len = (this.DataHeap[1] & 127);
                byte[] buffer;

                if (len == 126)
                {
                    if (max_size < 8)
                        return null;

                    buffer = this.DataHeap.GetRange(2, 2).ToArray();
                    Array.Reverse(buffer);
                    len = (int)BitConverter.ToUInt16(buffer, 0);

                    if (max_size < (8 + len))
                        return null;

                    this.DataHeap.RemoveRange(0, 4);
                }
                else if (len == 127)
                {
                    if (max_size < 14)
                        return null;

                    buffer = this.DataHeap.GetRange(2, 8).ToArray();
                    Array.Reverse(buffer);
                    len = (int)BitConverter.ToUInt64(buffer, 0);

                    if (max_size < (14 + len))
                        return null;

                    this.DataHeap.RemoveRange(0, 10);
                }
                else
                {
                    if (max_size < (6 + len))
                        return null;

                    this.DataHeap.RemoveRange(0, 2);
                }
                if (this.DataHeap.Count >= (len + 4))
                {
                    byte[] key = this.DataHeap.GetRange(0, 4).ToArray();
                    byte[] text = this.DataHeap.GetRange(4, len).ToArray();
                    this.DataHeap.RemoveRange(0, (len + 4));

                    for (int i = 0; i < text.Length; i++)
                        text[i] ^= key[i % 4];

                    switch (msg)
                    {
                        case 136:
                            this.Expired = true;
                            return null;

                        case 129:
                            return Encoding.UTF8.GetString(text);

                        default:
                            return null;
                    }
                }
            }

            return null;
        }

        private List<String> recent_posts = new List<String>();
        private int text_packet_counter = 0;

        public bool IsFlooding(String text, uint time)
        {
            if (Helpers.IsTextFlood(this))
                return true;

            this.recent_posts.Insert(0, text);

            if (this.recent_posts.Count == 5)
            {
                this.recent_posts.RemoveAt(4);
                return this.recent_posts.TrueForAll(str => str == this.recent_posts[0]);
            }

            if (time > (this.last_text_packet + 1))
            {
                this.last_text_packet = time;
                this.text_packet_counter = 0;
                return false;
            }
            else return ++this.text_packet_counter > 3;
        }

    }
}
